#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: DB.pm 189 2006-02-27 15:20:18Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
# mthodes de conversion des infos Khartes
# en DB mysql
# 
#######################################################
package XML::Khartes::DB;
#######################################################

BEGIN {
  use Exporter;
  use DBI;
  use XML::Khartes;
  use Encode;
  our @ISA = qw/Exporter/;
  our @EXPORT= qw/export/;
}

sub new {
  my $class = shift;
  my $o = {@_};
  bless($o, $class);
  return $o;
}

sub corpus {
  my ($o) = @_;
  return $o->{corpus};
}

sub populate {
  my ($o) = @_;
  my @sql = $o->corpus->toSQL;
  my $db = $o->_db;
  map {$db->do($_)} @sql
}

sub _db {
  my ($o) = @_;
  return $o->{db};
}

sub close {
  my ($o) = @_;
  $o->_db->disconnect;
}

#######################################################
# Extension de XML::Khartes::Corpus
#######################################################

sub XML::Khartes::Corpus::toSQL {
  my ($o) = @_;
  my @corpussql = ();
  my %texts = $o->texts ;
  foreach (values %texts) {
    push @corpussql, $_->toSQL;
  }
  return @corpussql;
}

sub  XML::Khartes::Corpus::db { 
  my ($c, $host, $usr, $passwd, $db) = @_;
  $db = "khartes" unless $db;
  $usr = "" unless $usr;
  $host = "localhost" unless $host;
  $passwd = "" unless $passwd;

  my $dsn = "DBI:mysql:database=$db;host=$host";
  my $dbh = DBI->connect($dsn, $user, $passwd);

  return XML::Khartes::DB->new(db => $dbh, corpus => $c);
}

sub XML::Khartes::Corpus::dbCreate {
  my ($c, $host, $usr, $passwd, $db) = @_;
  $db = "" unless $db;
  $usr = "" unless $usr;
  $host = "localhost" unless $host;
  $passwd = "" unless $passwd;

  my $dsn = "DBI:mysql:database=$db;host=$host";
  my $dbh = DBI->connect($dsn, $user, $passwd);


  my @tables = (

    "CREATE DATABASE IF NOT EXISTS khartes",
    
    "USE khartes",

    # Les tables existantes sont dtruites
    
    "DROP TABLE IF EXISTS `kh:chartes`",
    "DROP TABLE IF EXISTS `kh:personnes`",
    "DROP TABLE IF EXISTS `kh:fonctions`",
    "DROP TABLE IF EXISTS `kh:exercer_fonction`",
    "DROP TABLE IF EXISTS `kh:jouer_role`",

    # construction

    "CREATE TABLE `kh:chartes` (
      -- Rfrence dans le projet Kharts
      `id` VARCHAR( 40 ) UNIQUE NOT NULL ,		
      -- Les quatre premiers chiffres sont ncessairement l'anne, mais il y a de telles variations qu'on ne peut faire un champ date
      `temps` VARCHAR( 20 ) NOT NULL ,	  
      `fonds` VARCHAR( 7 ) NOT NULL ,
      `main` VARCHAR( 3 ) NOT NULL ,
      `lieu` VARCHAR( 25 ) NULL,
			`action` VARCHAR( 20 ) NOT NULL ,	  
			`mots` INT NOT NULL,
			`ponctuants` INT NOT NULL,
      `fichier` VARCHAR(1024) NOT NULL
    ) TYPE = MYISAM ;",

    "CREATE TABLE `kh:jouer_role` (
      -- Au moins un rle est jou par une personne dans une charte, mais cette personne peut jouer plusieurs rles
      `ref_charte` VARCHAR( 40 ) NOT NULL ,
      `ref_personne` VARCHAR( 50 ) NOT NULL,
      `role` VARCHAR( 25 ) NOT NULL 
    ) TYPE = MYISAM ;",

    # Sociologie

    "CREATE TABLE `kh:personnes` (
      `nom` VARCHAR( 50 ) NOT NULL ,
      -- rfrence dans le fichier de lemmes
      `id` VARCHAR( 50 ) UNIQUE NOT NULL ,
      -- les personnes sont caractrises par un rang social
      -- FIXME: ce sera certainement  revoir
      `rang` ENUM( 'peuple', 'petite noblesse', 'bourgeoisie', 'bas clerg','haute noblesse'  ) NULL , 
      PRIMARY KEY ( `id` ) 
    ) TYPE = MYISAM ;",

    "CREATE TABLE `kh:exercer_fonction` (
      -- Une personne exerce de zro  n fonctions
      `ref_fonction` INT NOT NULL ,
      `ref_personne` VARCHAR( 50 ) NOT NULL
    ) TYPE = MYISAM ;",

    "CREATE TABLE `kh:fonctions` (
      `id` INT NOT NULL AUTO_INCREMENT ,
      `nom` VARCHAR( 50 ) NOT NULL ,
      PRIMARY KEY ( `id` ) 
    ) TYPE = MYISAM ;",

 
  );

  map {
    $dbh->do($_);
  } @tables;
  
  return XML::Khartes::DB->new(db => $dbh, corpus => $c);
}



#######################################################
# Extension de XML::Khartes
#######################################################

sub XML::Khartes::toSQL {
  my ($o) = @_;
  my $ref = $o->reference;
  my $filename = $o->{file};
  my $time = $o->time;
  my $hand = $o->hand;
	my $action = "";
  my $institution = $o->institution;
  my @sql = ();
  $time =~ s/'/\\'/go;
  $ref =~ s/'/\\'/go;
	
	# comptes
	my $words;
	my $ponct;
	

	
  # personnes impliques
  my $a = $o->Analysis;

  my %authors = $a->authors;
  my %sigillants = $a->sigillants;
	my %actions = $a->actions;
	
  my $l = $o->lexicon;
  
  map {
    my $name = $l->label($_);
    $name =~ s/'/\\'/og;
    push @sql, 
    latin1("REPLACE INTO `kh:personnes` VALUES('$name','$_','')"),
    latin1("REPLACE INTO `kh:jouer_role` VALUES('$ref','$_','auteur')");
  } keys %authors;

  map {
    my $name = $l->label($_);
    $name =~ s/'/\\'/og;
    push @sql, 
    latin1("REPLACE INTO `kh:personnes` VALUES('$name','$_','')"),
    latin1("REPLACE INTO `kh:jouer_role` VALUES('$ref','$_','scelleur')");
  } keys %sigillants;
  
	map {
		$action = $l->label($_);
    $action =~ s/'/\\'/og;
		$action = "" unless $action;
	} keys %actions;
	
  # chartes
  push @sql, latin1("REPLACE INTO `kh:chartes` VALUES('$ref','$time','$institution','$hand','', '$action','$words', '$ponct', '$filename')");
  return @sql;
}

sub latin1 {
  return encode("ISO-8859-1", shift);
}

1;

__END__

