#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: TEI.pm 189 2006-02-27 15:20:18Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
#######################################################
package XML::Khartes::Export::TEI;
#######################################################
#
# Permet l'exportation au format conforme TEI P4X, grce  une extension

use strict;
use base qw (XML::Khartes::Export::Common);

my (
  $elts,

  $header, 
  $filedesc,
  $titlestmt,
  $title,
  $respstmt,
  $publicationstmt,
  $sourcedesc,
  $diplo,
  $dimensions,
  $text, 
  $front, 
  $body, 
  $recto, 
  $verso, 
  $notes, 
);


sub prefix {
  return 
  q#<?xml version="1.0" encoding="ISO-8859-1"?>
  <!DOCTYPE TEI.2 PUBLIC "-//TEI P4//DTD Main Document Type//EN" "tei2.dtd" [
  <!ENTITY % TEI.XML "INCLUDE">
  <!ENTITY % TEI.extensions.ent PUBLIC "-//KML//ENTITIES Base//EN" "khartes.ent">
  <!ENTITY % TEI.extensions.dtd PUBLIC "-//KML//ELEMENTS Base//EN" "khartes.dtd">
  <!ENTITY % TEI.prose "INCLUDE">
  <!ENTITY % TEI.linking "INCLUDE">
  <!ENTITY % TEI.transcr "INCLUDE">
  <!ENTITY % TEI.textcrit "INCLUDE">
  <!ENTITY % TEI.analysis "INCLUDE">]>#
}

sub XML::Khartes::Corpus::Export::TEI::prefix {
  my $return = &prefix;
  $return .= "<teiCorpus.2>
  <teiHeader></teiHeader>";
  return $return;
}

sub XML::Khartes::Corpus::Export::TEI::suffix {
  return "</teiCorpus.2>";
}


sub start_document {
  my ($self, $el) = @_;

  # traductions
  $elts = {
    # ancien nom	nouveau nom	nouveaux attributs	anciens attributs renomms	
    recto =>	["div", 	{type => "recto"}			],
    verso =>	["div", 	{type => "verso"}			],
    add =>	["add"],
    dam =>	["damage"],
    dat =>		["div", 	{type => "verso note"}			],	
    l	=>	["lb"							],
    biblio =>	["bibl", 	{}					],
    i =>		["emph"							],
    analyse =>	["abstract", 	{}					],	
    w => 		["seg", 	{type => "word"}			],
    ponct => 	["descriptio", 	{type => "punctuation"}			],
    mponct => 	["editio",	{type => "punctuation"}			],
    joncteur => 	["editio",	{type => "join"}			],
    separateur => 	["editio",	{type => "split"}			],
  };

  # Arbre de sortie
  $self->{output} = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $outroot = $self->{output}->createElement("TEI.2");
  $self->{output}->setDocumentElement($outroot);
  $self->{current_out} = $self->{output}->documentElement;


  $header = $self->{current_out}->appendChild($self->{output}->createElement("teiHeader"));
  $filedesc = $header->appendChild($self->{output}->createElement("fileDesc"));
  $titlestmt = $filedesc->appendChild($self->{output}->createElement("titleStmt"));
  $title = $titlestmt->appendChild($self->{output}->createElement("title"));
  $respstmt = $titlestmt->appendChild($self->{output}->createElement("respStmt"));
  $respstmt->appendChild($self->{output}->createElement("name"));

  $publicationstmt = $filedesc
  ->appendChild($self->{output}->createElement("publicationStmt"))
  ->appendChild($self->{output}->createElement("p"));
  $sourcedesc = $filedesc
  ->appendChild($self->{output}->createElement("sourceDesc"))
  ->appendChild($self->{output}->createElement("p"));

  $diplo = $header
  ->appendChild($self->{output}->createElement("profileDesc"))
  ->appendChild($self->{output}->createElement("diplomaticDesc"));

  $text = $self->{current_out}->appendChild($self->{output}->createElement("text"));
  $text->appendChild($self->{output}->createElement("front"));
  $body = $text->appendChild($self->{output}->createElement("body"));

  # Notes dans l'arbre de sortie
  $notes = $self->{output}->createElement("div");
  $notes->setAttribute("type", "notes");
  $body->appendChild($notes);
}

sub end_document {
}

sub start_element {
  my ($self, $el) = @_;

  $self->SUPER::start_element($el);

  # passer les lments non ncessaires
  return 0 unless $self->{print};

  my $name = $el->{Name};

  my $seg;


  if (defined $elts->{$name}) {

    my @node = @{$elts->{$name}};

    $seg = $self->{output}->createElement($node[0]); 
    my $atts = $node[1];
    my $old_atts = $node[2];

    # nouveaux attributs
    foreach my $key (keys %{$atts}) {
      $seg->setAttribute($key, $atts->{$key})
    }

    my $id = $el->{Attributes}{"{}id"}{Value}; 
    $seg->setAttribute("ref", $id) if $id;

    # dterminer la position de currentout en fonction du nom de l'lment

    my $p = "";

    if ($name =~ /^recto$/) { $self->{current_out} = $body; $p = 1; } 
    if ($name =~ /^verso$/) { $self->{current_out} = $body; $p = 0; } 
    if ($name =~ /^date?$/) { $p = 1; } 
    if ($name =~ /^biblio$/) { $self->{current_out} = $sourcedesc; $p = 0; } 
    elsif ($name =~ /^analyse$/)  { $self->{current_out} = $diplo; $p = 0;} 

    $self->{current_out}->appendChild($seg);

    if ($p) {
      $self->{current_out} = $seg->appendChild($self->{output}->createElement("p"));
    } else {
      $self->{current_out} = $seg;
    }; 

    # les dates des mentions dorsales
    if ($name =~ /^date?$/) {
      my $date = $self->{current_out}->appendChild($self->{output}->createElement("date"));
      $date->setAttribute("value",$el->{Attributes}{"{}s"}{Value});
    }


  }

  # traitement des notes
  if ($name =~ /^notes$/) { 
    unless ($self->{current_in}->parentNode->nodeName eq "charte") {
      $self->{print} = 0 ;
      return $self->{current_out} = $self->{output}->documentElement 
    };
    return $self->{current_out} = $notes; 
  }
  if ($name =~ /^note$/) {
    my $note = $self->{output}->createElement("note"); 
    $note->setAttribute("target", $el->{Attributes}{"{}id-ref"}{Value});
    $note->setAttribute("descriptio", $el->{Attributes}{"{}description"}{Value}) if $el->{Attributes}{"{}description"}{Value};
    $note->setAttribute("editio", $el->{Attributes}{"{}edition"}{Value}) if $el->{Attributes}{"{}edition"}{Value};;
    $note->setAttribute("private", $el->{Attributes}{"{}prive"}{Value}) if $el->{Attributes}{"{}prive"}{Value};;
    return $self->{current_out} = $notes->appendChild($note); 
  }

  # donnes sur la charte
  if ($name =~ /^charte$/) {
    # dimensions
    $dimensions = $self->{output}->createElement("dimensions");
    $dimensions->setAttribute("value", $el->{Attributes}{'{}dimensions'}{Value});
    # titre (rfrence)
    $title->appendText(
      $el->{Attributes}{"{}an"}{Value} .
      " " .
      $el->{Attributes}{"{}fonds"}{Value} .
			" " .
			$el->{Attributes}{"{}jour"}{Value} .
			"-" .
			$el->{Attributes}{"{}mois"}{Value} 
    );
  }

}

sub end_element {
  my ($self, $el) = @_;
  $self->{current_in} = $self->{current_in}->parentNode;
  return 0 unless $self->{print};
  my $name = $el->{Name};
  if ( $elts->{$name} ) { 
    $self->{current_out} = $self->{current_out}->parentNode ;
    if ($name =~ /^date?$/) { 
      $self->{current_out} = $self->{current_out}->parentNode;
    }
    return $self->{current_out};
  }
  elsif ( $name =~ /^notes$/ ) { 
    $self->{print} = 0;
    return $self->{current_out} = $self->{output}->documentElement; 
  };
	elsif ( $name =~ /^charte$/ ) {
		$diplo->appendChild($dimensions);
	}

}

sub characters {
  my ($self, $text) = @_;

  return 0 unless $self->{print};

  $text = $text->{Data};

  # pour les donnes des transcriptions
  if (
    $self->{current_in}->findnodes("ancestor-or-self::*[name() = 'w' or name() = 'ponct' or name() = 'mponct' or name() = 'kh'][1]"	)
  ) {
      $text =~ s/\s+//sg; # les espaces ne sont pas pertinents
      $self->_characters_kh_tags($self->_characters_kh($text));
    }

    # fin des caractres transcrits
    else {
      $text =~ s/\s+/ /gs;
      $self->{current_out}->appendText($text);	
    }
  }


  sub _characters_kh_tags {
    my ($self, @chars) = @_;
    
    CHAR: foreach (@chars) {

      
      
      
      my $spot = $self->{current_out}; 

      my $char = $_->[0];
      my $khExtension = $_->[1];

      # les crochets marquent les abrviations
      if ($char =~ /^\[$/) {
        $self->{current_out} = $spot->appendChild($self->{output}->createElement("expan"));	
        next CHAR;
      }
      elsif ($char =~ /^\]$/) {
        $self->{current_out} = $self->{current_out}->parentNode;
        next CHAR;
      }
      
      
      if ($khExtension->{editio} or $khExtension->{descriptio}) {
        foreach my $level (reverse sort keys %$khExtension) {
          $spot = $spot->appendChild($self->{output}->createElement($level));
          foreach (keys %{$khExtension->{$level}}) {
            $spot->setAttribute($_, $khExtension->{$level}{$_});
          }
        }

        $spot->appendText($char);
      }

      else {
        $spot->appendText($char);	
      }
    }
  }

 
1;

#######################################################
package XML::Khartes::Corpus::Export::TEI;
#######################################################
