#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: TEX.pm 267 2006-06-01 22:07:11Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
#######################################################
package XML::Khartes::Export::TEX;
#######################################################
#
# sortie LaTeX

use strict;
use base qw (XML::Khartes::Export::Static);
use Switch;

sub new {
  my $class = shift;
  my %options = (
    "editio-punctuation" => 0,
    "editio-segmentation" => 0,
    "editio-uppercase" => 0,
    "editio-diacritics" => 0,
    "editio-turn-ui" => 0,
    "editio-notes" => 0,
    "descriptio-marked" => 1,
    "descriptio-punctuation" => 1,
    "descriptio-diacritics" => 1,
    "descriptio-longs" => 1,
    "descriptio-notes" => 1,
    @_
  );
  my $o  = $class->SUPER::new(%options);
  bless($o, $class);
  return $o;
}

sub prefix {
  return  q(\documentclass[a4paper,10pt]{article}
  \usepackage[french]{babel}
  \usepackage{khartes}
  \usepackage{textcomp}
  \usepackage{dotlessj}
  \usepackage{times}
  \usepackage{tipa}
  \usepackage{graphicx}
  \DeclareTextCompositeCommand\H{\encodingdefault}{\j}{\jaccent\H} % personnal addition
  \usepackage[T1]{fontenc}
  \usepackage{ledmac}
  \setcounter{firstlinenum}{4}
  \setcounter{linenumincrement}{4}
  \footparagraph{A}
  \footparagraphX{A}
  \renewcommand*{\thefootnoteA}{(\arabic{footnoteA})}
  \renewcommand*{\thefootnoteB}{(\alph{footnoteB})}
  \footparagraphX{B}
  \makeatletter
  \renewcommand{\Afootfmt}[3]{%
  \normal@pars
  \parindent=0pt \parfillskip=0pt plus 1fil
  {\bfseries\printlines#1|}\strut
  {\select@lemmafont#1|#2} #3}
  \makeatother
  \begin{document}\sloppy). "\n";
}

sub XML::Khartes::Corpus::Export::TEX::prefix {
  return &prefix;
}

sub XML::Khartes::Corpus::Export::TEX::suffix {
  return &suffix;
}

sub start_document {
  my ($self, $el) = @_;
  $self->SUPER::start_document($el);
}

sub start_element {
  my ($self, $el) = @_;
  $self->SUPER::start_element($el);
  
  my $name = $el->{Name};

  switch ($name) {

		case "charte" {
			my $an = $et->{Attributes}{'{}an'}{Value};
			my $fonds = $et->{Attributes}{'{}an'}{Value};
			$self->output("\\section{$an $fonds}");
		}
		
		case "w" {
			$self->{print} = 1;
			my $prev = $self->{current_in}->prev;
			if ( $prev ) {
				my $prevname =$prev->nodeName;
				# FIXME
				if ($prevname !~ /^(separateur)|(ponct)$/) { $self->output(" ") unless $self->{output} =~ /[\(]$/; }
			}
		}
    case "ponct" { 
      $self->{print} = 0 unless $self->{'descriptio-punctuation'};
      $self->output(" ") unless $self->{'current_in'}->findnodes("ancestor::w");
    }
    case "mponct" { $self->{print} = 0 unless $self->{'editio-punctuation'}; }
    case /^separateur$/ {
      $self->{print} = 0 unless $self->{'editio-segmentation'};
    }
    case "l" {
      my $hyphen = " ";
      $self->{print} = 1;
      $hyphen = "-" if $self->{current_in}->findnodes("ancestor::w[1]") and $self->{'editio-segmentation'}; 
      $self->output($hyphen . '\raisebox{1pt}{\tiny ' . "\n" . '[' . $self->{current_line} . "]}" . $hyphen) unless $self->{current_line} == 1;
    }
    case "del" { 
      return 0 unless $self->{print};
      return 0 if $self->{event};
      $self->{event} = $name;
      switch ($self->{Attributes}{'{}type'}{Value}) {
	case "scratch" {$self->output("[");}
	case "strikeout" {$self->output('\sout{');}
      }
    }
    case "dam" { 
      return 0 unless $self->{print};
      return 0 if $self->{event};
      $self->{event} = $name;
      $self->output('$\langle$'); 
    }
    case "add" { 
      return 0 unless $self->{print};
      return 0 if $self->{event};
      $self->{event} = $name;
      $self->output('\textopencorner'); 
    }
  }
  
  
  
}

sub end_element {
  my ($self, $el) = @_;
  $self->SUPER::end_element($el);
  my $name = $el->{Name};
  switch ($name) {
    case "ponct" {
      $self->{print} = 1;
      $self->output(" ") unless $self->{output} =~ /\\ksmallspace\{\}$/;
    } 
    case "recto" {$self->close_editorial_annotation}
  }
}

sub end_document {
  my $self = shift;
}

sub suffix {
  return q(\end{document});
}

sub characters {
  my ($self, $text) = @_;
  return 0 unless $self->{print};
  my $data = $text->{Data};
  if ($self->{kh}) {
    $self->_characters_kh_tex($self->_characters_kh($data))
  } else {
    $data =~ s/\s+/ /g;
    $self->output($data)
  }
}

sub _characters_kh_tex {
  my ($self, @chars) = @_;

  $self->close_editorial_annotation;

  CHAR: foreach (@chars) {

    my $char = $_->[0];
    my $khExtension = $_->[1];

    # les crochets marquent les abrviations
    if ($char =~ /^\[$/) {
      $self->output('\emph{');	
      next CHAR;
    }
    elsif ($char =~ /^\]$/) {
      $self->output('}');	
      next CHAR;
    }
    # il faut une espace avant les parenthses
    elsif ($char =~ /^\($/) {
      $self->output(' (');
      next CHAR
    }
    # latex ne supporte pas de voir "_" tout seul...
    elsif ($char =~ /^_$/) {
      $self->output('\_');
      next CHAR
    }   
    # ponctuation originale
    elsif ($char =~ /^(\*)?(\?)?(!)?(\|)?$/) {
      $self->output('\textperiodcentered{}') if $1;
      $self->output('\kpela{}') if $2;
      $self->output('\kpelb{}') if $3;
      if ($4) {
	$self->{output} =~ s/\s+$//;
	$self->output('\ksmallspace{}');
      }
      next CHAR
    }
    # joncteur
    elsif ($char =~ /^-$/) {
      unless ($self->{'editio-segmentation'}) {
	$self->output(' ');
	next CHAR
      }
    }   


    my ($prefix, $suffix) = "";

    # Diacritiques

    if (my $dia = $khExtension->{descriptio}{diacritics} and $self->{'descriptio-diacritics'}) {
      $prefix = "\\";
      switch ($dia) {
	case "accent" {$prefix .= "\'"}
	case "double" {$prefix .= "H"}
	case "dot" {$prefix .= "\."}
      }
      $prefix .= "{";
      $suffix .= "}";
    }

    elsif ($dia = $khExtension->{editio}{diacritics} and $self->{'editio-diacritics'}) {
      $prefix = "\\";
      switch ($dia) {
	case "diaresis" {$prefix .= "\""}
	case "accent" {$prefix .= "\'"}
      }
      $prefix .= "{";
      $suffix .= "}";
    }

    # forme
    if ($khExtension->{editio}{uppercase} and $self->{'editio-uppercase'}) { $char = uc($char); }
    if ($khExtension->{descriptio}{uppercase} and $self->{'descriptio-marked'}) { 
      $prefix .= '\textsc{'; 
      $suffix .= '}'; 
    }

    $self->output($prefix, $char, $suffix);

  }
}

sub close_editorial_annotation {
  my ($self) = @_;
  # C'est un peu tricky: s'il y a une manipulation d'dition lance, c'est ici, au besoin, qu'elle doit
  # tre "ferme".
  my $event = $self->{event};
  # intervertit l'espace ventuel qui aurait t ainsr avant la marque ditoriale avec cette marque
  my $space_after = "";
  if ($self->{output} =~ s/\s+$//) { $space_after = " "};
  if ($event and ! $self->{current_in}->findnodes("./ancestor-or-self::$event") ) {
    switch ($event) {
      case "del" { 
	# FIXME
	switch ($self->{Attributes}{'{}type'}{Value}) {
	  case "scratch" {$self->output("]");}
	  case "strikeout" {$self->output('}');}
	}
      }
      case "dam" { $self->output('$\rangle$'); }
      case "add" { $self->output('\reflectbox{\textopencorner}'); }
    }
    $self->{event} = 0;
  }
  $self->output($space_after);

}


1;

