#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: TXT.pm 197 2006-03-19 09:33:26Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
#######################################################
package XML::Khartes::Export::TXT;
#######################################################
#
# sortie LaTeX

use strict;
use base qw (XML::Khartes::Export::Static);
use Switch;

sub new {
  my $class = shift;
	my %passed_opts = @_;
	my %preselect = ();
	if ($passed_opts{descriptio} or $passed_opts{diplo}) {
		%preselect = (
			"editio-punctuation" => 0,
			"editio-segmentation" => 0,
			"editio-uppercase" => 0,
			"editio-diacritics" => 0,
			"editio-turn-ui" => 0,
			"editio-notes" => 0,
			"descriptio-marked" => 1,
			"descriptio-punctuation" => 1,
			"descriptio-diacritics" => 1,
			"descriptio-longs" => 1,
			"descriptio-notes" => 1,
		);
	} elsif  ($passed_opts{editio}) {
		%preselect = (
			"editio-punctuation" => 1,
			"editio-segmentation" => 1,
			"editio-uppercase" => 1,
			"editio-diacritics" => 1,
			"editio-turn-ui" => 1,
			"editio-notes" => 1,
			"descriptio-marked" => 0,
			"descriptio-punctuation" => 0,
			"descriptio-diacritics" => 0,
			"descriptio-longs" => 0,
			"descriptio-notes" => 0,
		);
	} 
  my %options = (
    "editio-punctuation" => 0,
    "editio-segmentation" => 0,
    "editio-uppercase" => 0,
    "editio-diacritics" => 0,
    "editio-turn-ui" => 0,
    "editio-notes" => 0,
    "descriptio-marked" => 1,
    "descriptio-punctuation" => 1,
    "descriptio-diacritics" => 1,
    "descriptio-longs" => 1,
    "descriptio-notes" => 1,
		%preselect,
    %passed_opts,
  );
  my $o  = $class->SUPER::new(%options);
  bless($o, $class);
  return $o;
}

sub prefix {
}

sub XML::Khartes::Corpus::Export::TXT::prefix {
  return &prefix;
}

sub XML::Khartes::Corpus::Export::TXT::suffix {
  return &suffix;
}

sub start_document {
  my ($self, $el) = @_;
  $self->SUPER::start_document($el);
}

sub start_element {
  my ($self, $el) = @_;
  $self->SUPER::start_element($el);
  
  my $name = $el->{Name};

	switch ($name) {
		case "w" {
			$self->{print} = 1;
			my $prev = $self->{current_in}->prev;
			if ( $prev ) {
				my $prevname =$prev->nodeName;
				# FIXME
				if ($prevname !~ /^(separateur)$/) { $self->output(" ") unless $self->{output} and $self->{output} =~ /[\(]$/; }
			}
		}
		case "ponct" { 
			$self->{print} = 0 unless $self->{'descriptio-punctuation'};
			$self->output(" ") unless $self->{'current_in'}->findnodes("ancestor::w");
		}
		case "mponct" { $self->{print} = 0 unless $self->{'editio-punctuation'}; }
		case /^separateur$/ {
			$self->{print} = 0 unless $self->{'editio-segmentation'};
		}
		case "l" {
			my $hyphen = " ";
			$hyphen = "-" if $self->{current_in}->findnodes("ancestor::w[1]") and $self->{'editio-segmentation'}; 
			$self->output($hyphen . "\n" . '[' . $self->{current_line} . "]" . $hyphen) unless $self->{current_line} == 1;
		}
		case "del" { 
			return 0 unless $self->{print};
			return 0 if $self->{event};
			$self->{event} = $name;
			switch ($self->{Attributes}{'{}type'}{Value}) {
				case "scratch" {$self->output("[");}
				case "strikeout" {$self->output('>');}
			}
		}
		case "dam" { 
			return 0 unless $self->{print};
			return 0 if $self->{event};
			$self->{event} = $name;
			$self->output('<'); 
		}
		case "add" { 
			return 0 unless $self->{print};
			return 0 if $self->{event};
			$self->{event} = $name;
			$self->output('{'); 
		}
	}
  
  
  
}

sub end_element {
  my ($self, $el) = @_;
  $self->SUPER::end_element($el);
  return 0 unless $self->{print};
  my $name = $el->{Name};
  switch ($name) {
    case "ponct" {
      $self->{print} = 1;
      $self->output(" ") unless $self->{output} =~ /\|$/;
    } 
    case "recto" {$self->close_editorial_annotation}
  }
}

sub end_document {
  my $self = shift;
}

sub suffix {
  return q(\end{document});
}

sub characters {
  my ($self, $text) = @_;
  return 0 unless $self->{print};
  my $data = $text->{Data};
  if ($self->{kh}) {
    $self->_characters_kh_tex($self->_characters_kh($data))
  } else {
    $data =~ s/\s+/ /g;
    $self->output($data)
  }
}

sub _characters_kh_tex {
  my ($self, @chars) = @_;

  $self->close_editorial_annotation;

  CHAR: foreach (@chars) {

    my $char = $_->[0];
    my $khExtension = $_->[1];

    # les crochets marquent les abrviations
    if ($char =~ /^\[$/) {
      $self->output('(');	
      next CHAR;
    }
    elsif ($char =~ /^\]$/) {
      $self->output(')');	
      next CHAR;
    }
    # il faut une espace avant les parenthses
    elsif ($char =~ /^\($/) {
      $self->output(' (');
      next CHAR
    }
    elsif ($char =~ /^_$/) {
      $self->output('_');
      next CHAR
    }   
    # ponctuation originale
    elsif ($char =~ /^(\*)?(\?)?(!)?(\|)?$/) {
      $self->output('') if $1;
      $self->output('?') if $2;
      $self->output('!') if $3;
      if ($4) {
	$self->{output} =~ s/\s+$//;
	$self->output('|');
      }
      next CHAR
    }
    # joncteur
    elsif ($char =~ /^-$/) {
      unless ($self->{'editio-segmentation'}) {
	$self->output(' ');
	next CHAR
      }
    }   


    my ($prefix, $suffix) = "";

    # Diacritiques

    if (my $dia = $khExtension->{descriptio}{diacritics} and $self->{'descriptio-diacritics'}) {
      $prefix = "\\";
			switch ($dia) {
				# FIXME
				case "double" {$prefix .= "H"}
				case "dot" {$prefix .= "\."}
			}
      $prefix .= "{";
      $suffix .= "}";
    }

    elsif ($dia = $khExtension->{editio}{diacritics} and $self->{'editio-diacritics'}) {
      $prefix = "\\";
			switch ($dia) {
				case "diaresis" {$prefix .= "\""}
				case "accent" {$prefix .= "\'"}
			}
      $prefix .= "{";
      $suffix .= "}";
    }

    # forme
    if ($khExtension->{editio}{uppercase} and $self->{'editio-uppercase'}) { $char = uc($char); }
    if ($khExtension->{descriptio}{shape} and $khExtension->{descriptio}{shape} eq 1 and $self->{'descriptio-marked'}) { 
      $char = uc($char)
    }

    $self->output($prefix, $char, $suffix);

  }
}

sub close_editorial_annotation {
	my ($self) = @_;
	# C'est un peu tricky: s'il y a une manipulation d'dition lance, c'est ici, au besoin, qu'elle doit
	# tre "ferme".
	my $event = $self->{event};
	# intervertit l'espace ventuel qui aurait t ainsr avant la marque ditoriale avec cette marque
	my $space_after = "";
	if ($self->{output} and $self->{output} =~ s/\s+$//) { $space_after = " "};
	if ($event and ! $self->{current_in}->findnodes("./ancestor-or-self::$event") ) {
		switch ($event) {
			case "del" { 
				# FIXME
				switch ($self->{Attributes}{'{}type'}{Value}) {
					case "scratch" {$self->output("]");}
					case "strikeout" {$self->output('<');}
				}
			}
			case "dam" { $self->output('>'); }
			case "add" { $self->output('}'); }
		}
		$self->{event} = 0;
	}
	$self->output($space_after);

}


1;

