#!/usr/bin/perl -w
# Copyright (c) 2005-2006 Universit de Lige, Nicolas Mazziotta
# $Id: IO.pm 189 2006-02-27 15:20:18Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

package XML::Khartes::IO;

BEGIN {
  use Exporter;
  use XML::Khartes::Export;
  use XML::Khartes::Import;
  use XML::LibXML::SAX;
  our @ISA = qw/Exporter/;
  our @EXPORT= qw/export/;
}

# mthode de conversion des fichiers Khartes
# renvoie un string. Est appell par XML::Khartes (voir Import/export)


sub XML::Khartes::convert {
  my ($o, $method, $way, $options) = @_;
  $options->{khartes} = $o;
  # $options is a hashref
  # if $options->{noprefix} is TRUE, there will be no header of any kind
  
  print STDERR "converting text\n" if $o->{warn};
  $method = "TEI" unless $method;

  # automatically loads needed modules
  $method = "XML::Khartes::".$way."::"."$method";
  eval "require $method";
  
  my $parser = XML::LibXML::SAX->new(Handler => $method->new(%$options));
  $parser->parse_string($o->xml->toString(1));

  my $return = "";
  # Useful to add dtd and the like... (poor support in XML::LibXML)
  eval { $return = &{"$method"."::prefix"}; } unless $options->{noprefix};
  my $out = $parser->{Handler}{output};
  if (ref $out eq "XML::LibXML::Document") {
    $return .= $out->documentElement->toString(1); 
  } else {
    $return .= $out if $out; 
  }
  return $return;
}

# This function allows document fragments to be processed
# FIXME currently supports only TEX output
sub XML::LibXML::Element::convert {
  my ($el, $method, $way, $options) = @_;
  
  # print STDERR "converting kharts fragment\n";
  $method = "TEI" unless $method;

  # automatically loads needed modules
  $method = "XML::Khartes::".$way."::"."$method";
  eval "require $method";
  
  my $parser = XML::LibXML::SAX->new(Handler => $method->new(%$options));
  $parser->parse_string($el->toString(1));

  my $return = "";
  # Useful to add dtd and the like... (poor support in XML::LibXML)
  eval { $return = &{"$method"."::prefix"}; } unless $options->{noprefix};
  my $out = $parser->{Handler}{output};
  if (ref $out eq "XML::LibXML::Document") {
    $return .= $out->documentElement->toString(1); 
  } else {
    $return .= $out if $out; 
  }
  eval { $return .= &{"$method"."::suffix"}; } unless $options->{nosuffix};
  return $return;
}

sub XML::Khartes::Corpus::convert {
  my ($o, $method, $way, $options) = @_;
  # if $options->{sort} is TRUE, outputs are sorted according to their ref
  
  print STDERR "converting corpus\n";
  $method = "TEI" unless $method;

  # automatically loads needed modules
  my $module = "XML::Khartes::".$way."::"."$method";
  eval "require $module";

  my $classpath = "XML::Khartes::Corpus::".$way."::"."$method";
  my $return = "";
  eval { $return .= &{"$classpath"."::prefix"}; } ;
  my %texts = $o->texts;
  my @texts = ();
  if ($options->{sort}) {
    map {push @texts, $texts{$_} } sort keys %texts ;
  } else {
    @texts = values %texts;
  }
  undef %texts;
  foreach (@texts) {
    $return .= $_->convert($method, $way, {%$options, noprefix => 1, nosuffix => 1} );
  }
  eval { $return .= &{"$classpath"."::suffix"}; } ;
  return $return;
}

# append element from sax

sub XML::LibXML::Document::fromSAX {
  # permet de construire un DOM pde SAX. Nanmoins, cette routine 
  # ne reproduit pas les donnes caractres 
  my ($doc, $el, $in, $text) = @_;
  my $new = $doc->createElement($el->{Name});
  my %atts = %{$el->{Attributes}};
  foreach (sort keys %atts) {
    my $value = $atts{$_}{Value}; 
    my $name = $atts{$_}{Name};
    $new->setAttribute($name, $value);
  }
  $in->appendChild($new);
  return $new;
}

sub XML::LibXML::Element::paste {
  my ($node, $sax, $name, $atts) = @_;
  my $new = $node->ownerDocument->createElement($name);
  foreach (sort keys %$atts) {
    $new->setAttribute($_,$atts->{$_});
  }
  $sax->{current_out}->appendChild($new);
  return $new;
}
1;
