#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: Common.pm 61 2005-11-02 13:04:35Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
#######################################################
package XML::Khartes::Export::Common;
#######################################################
# Mthodes communes  tous les handlers d'exportation

use base qw (XML::SAX::Base);

# dfinition des traductions
my $elts; 

sub new {
  my $class = shift;
  my %options = (
    'print' => 1,
    current_in => undef,
    current_out => undef,
    input => undef,
    output => undef,
    @_
  );

  my $o  = $class->SUPER::new(%options);

  # Arbre d'entre
  $o->{input} = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $inroot = $o->{input}->createElement("charte");
  $o->{input}->setDocumentElement($inroot);
  $o->{current_in} = $o->{input}->documentElement;


  bless($o, $class);
  return $o;
}

sub start_element {
  my ($self, $el) = @_;
  my $name = $el->{Name};

  # l'arbre d'entre est construit
  $self->{current_in} = $self->{input}->fromSAX($el, $self->{current_in});	

  # traitement des lments  ignorer
  if ($name =~ /^(ponct_data)|(syntaxe)|(diplo)|(lemmes)|(POS)|(C)$/) { $self->{print} = 0 }
  elsif ($name =~ /^(charte)|(notes)|(texte)|(analyse)$/) { $self->{print} = 1 }
  return 0 unless $self->{print};
}

sub prefix {
  # la fonction permet d'insrer du texte au dbut de la valeur de retour
  # Utile quand XML::LibXML ne manipule pas les DTD...
}

sub _characters_kh {
  # transforme les caractres des donnes "kharts"
  # en un hachage qui permettra une exportation de ces donnes,
  # via les fonctions _characters_kh_tags
  #
  # retourne une liste de tableaux:
  # 
  # [caractres, {infos}]
  #
  my ($self, $text) = @_;
  my %convert = qw/
   A
   a
   I
   i
   O
   o
   U
   u
   Y
   y
   A
   a
   E
   e
   I
   i
   O
   o
   U
   u
   y
  /;
  foreach (keys %convert) {
    $text =~ s/$_/$convert{$_}/g;
  }
  my %turn = qw(
    i j
    I j
    j i
    J i
    u v
    U v
    v u
    V u
  );
  my @chars = ();
  while ($text =~ /()?([\[\]\$\w\/\\\?\!\|\*;:\.,])(["\+\\]{0,5})/g) {
    my $char = $2;
    my $diacrit = $3;
    my $turn = $1; 
    my $khExtension = {};
    if ($char =~ /[]/) {
      $khExtension->{editio}{diacritics} = "accent";
    }
    $khExtension->{editio}{diacritics} = "diaresis" if $diacrit =~ //;
    $khExtension->{descriptio}{diacritics} = "accent" if $diacrit =~ //;
    $khExtension->{descriptio}{diacritics} = "double" if $diacrit =~ /["\\]/;
    $khExtension->{descriptio}{diacritics} = "dot" if $diacrit =~ /\+/;
    if ($self->{current_in}->nodeName eq "c") {
      #FIXME!!!!
      $khExtension->{descriptio}{shape} = "1";
      $khExtension->{descriptio}{type} = "letter";
    }
    if ($self->{current_in}->nodeName eq "x") {
      $khExtension->{descriptio}{'baseline-shift'} = "1";
    }
    $khExtension->{editio}{turn} = $turn{$char} if $turn;
    $khExtension->{editio}{uppercase} = "yes" if $char =~ /[A-Zɣ]/;
    $khExtension->{descriptio}{shape} = "long" if $char =~ /[\$]/;

    $char =~ s//e/g;
    $char =~ s//e/g;
    $char =~ s/[\$]/s/g;
    push @chars, [lc($char), $khExtension];
  }	

  return @chars;

}


1;


