#!/usr/bin/perl -w
# Copyright (c) 2005-6 Universit de Lige, Nicolas Mazziotta
# $Id: OldKhartes.pm 267 2006-06-01 22:07:11Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
#######################################################
package XML::Khartes::Import::OldKhartes;
#######################################################
#
# Ce handler permet d'exporter les anciens documents Kharts
# (plutt les documents faciles  encoder) vers la nouvelle version 

use strict;
# du fait de l'isomorphisme de la transcription
# des mots, c'est le module d'exportation qui sert
# de base
use base qw (XML::Khartes::Export::Common);

sub new {
  my $class = shift;
  my %options = (	@_ );
  my $o  = $class->SUPER::new(%options);

  $o->{_newword} = 0;

  # Arbre d'entre
  $o->{input} = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $inroot = $o->{input}->createElement("charte");
  $o->{input}->setDocumentElement($inroot);
  $o->{current_in} = $o->{input}->documentElement;


  bless($o, $class);
  return $o;
}

sub prefix {
  return "<?xml version='1.0' encoding='ISO-8859-1'?>\n";
}


sub start_document {
  my ($self, $el) = @_;

  # Arbre de sortie
  $self->{output} = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $outroot = $self->{output}->createElement("charte");
  $self->{output}->setDocumentElement($outroot);
  $self->{current_out} = $self->{output}->documentElement;
}

sub start_element {
  my ($self, $el) = @_;
  $self->{current_in} = $self->{input}->fromSAX($el, $self->{current_in});
  my $name = $el->{Name};

  if ($name eq "charte") {
    my %atts = %{$el->{Attributes}};
    foreach (sort keys %atts) {
      my $value = $atts{$_}{Value}; 
      my $name = $atts{$_}{Name};
      $self->{current_out}->setAttribute($name, $value);
    }
  } 

  # dans tous les autres cas

  elsif ($name eq "c") {
    $self->{current_out} = $self->{current_out}->paste($self, "w" => {}) if $self->{_newword};
    $self->{current_out} = $self->{output}->fromSAX($el, $self->{current_out});
    $self->{_newword} = 0;
  }
  elsif ($name eq "l") { 
    # FIXME: ne pas compter  chaque fois
    if ($self->{current_in}->findnodes("preceding-sibling::l")){
      my $parent = $self->{current_out}->parentNode;
      $parent->removeChild($self->{current_out}) if $self->{current_out}->trimmed_text eq "";
      $self->{current_out} = $parent 
    } 
    
    my $newline = $self->{output}->fromSAX($el, $self->{current_out});
    # handle hyphens
    $self->{current_out} = $newline ;
    $self->{_newword} = 1;
    $self->{_hyphen} = 0;

  }
  else {
    $self->{current_out} = $self->{output}->fromSAX($el, $self->{current_out});
  }
}

sub end_element {
  my ($self, $el) = @_;
  $self->{current_in} = $self->{current_in}->parentNode;
  $self->{current_out} = $self->{current_out}->parentNode;
}

sub characters {
  my ($self, $text) = @_;
  my $data = $text->{Data};
  my $c = 0;

  $c = 1 if $self->{current_in}->findnodes("ancestor::c");


  if ($self->{current_in}->findnodes("./ancestor-or-self::*[name() = 'not' or name() = 'rem' ][1]")) {
    $self->{current_out}->appendText($data);
    $self->{_newword} = 0;
  }
  elsif ($self->{current_in}->findnodes("./ancestor-or-self::*[name() = 'recto'  or name() = 'verso'  or name() = 'kh'][1]") ) {
    # FIXME
    $data =~ s/\s+/ /gs;
    while ($data =~ /
      ((?<!)[\s_|'])?
      (\|)?
      ([^\s_|']*?)
    ([.,;:)])?
    /gxs) {
      my $modified = $2;
      my $datatext = $3;
      my $punctuation = $4; 
      my $spacer = $1;
      
      if ($datatext =~ /-/) {
        $self->{_hyphen} = 1 ;
        print STDERR $datatext;
      } else {
        $self->{_hyphen} = 0 if $datatext
      }

      $self->{_newword} = 1 if $spacer;
      next if $spacer and $spacer =~ /^\s*$/;
      
      #print STDERR $spacer if $spacer;
      
      my $newnodename = "w";
      $newnodename = "ponct" if $datatext =~ /\s*[\?\*\/!]\s*/;

      if ($spacer and $spacer !~ /^\s+$/ ) {
        $self->{current_out} = $self->{current_out}->parentNode ; 
        if ($spacer eq "|") {
          $self->{current_out} = $self->{current_out}->paste($self, ponct => {}) ;
        } else {
          $self->{current_out} = $self->{current_out}->paste($self, separateur => {}) ;
        }
        $self->{current_out}->appendText($spacer);
        $self->{current_out} = $self->{current_out}->parentNode ; 
      }


      unless ( ( ($c or !$self->{_newword}) and $newnodename ne "ponct")) {
      if ($self->{_newword} or $newnodename eq "ponct") {
        my $oldout = $self->{current_out};
        if ($oldout->nodeName =~ /^(w)|(m?ponct)$/) {
          $self->{current_out} = $self->{current_out}->parentNode ; 
          $self->{current_out}->removeChild($oldout) if $oldout->trimmed_text =~ /^$/;
        }
				my $intervention = $self->{current_in}->nodeName;
				$intervention = "" unless $intervention =~ /^(dam|del|cor|bar)$/;
				if ($intervention) {
					$self->{current_out} = $self->{current_out}->parentNode->parentNode;
				}
        $self->{current_out} = $self->{current_out}->paste($self, $newnodename => {});
				if ($intervention) {
					$self->{current_out} = $self->{current_out}->paste($self, $intervention => {})
				}
        $self->{_newword} = 0;
      }
    } else {
    }
    $self->{current_out}->appendText("$modified") if $modified; 
    $self->{current_out}->appendText("$datatext");
    if ($punctuation) {
      $self->{current_out} = $self->{current_out}->parentNode  if $self->{current_out}->nodeName =~ /^(w)|(m?ponct)$/;
      $self->{current_out} = $self->{current_out}->paste($self, "mponct" => {}) ;
      $self->{current_out}->appendText($punctuation);
      $self->{_newword} = 1;
    }
  }	
} else {
  $self->{current_out}->appendText($data);
}

}


1;
