# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: Lexicon.pm 208 2006-04-17 13:55:50Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;		
use lib "$ENV{HOME}/lib/perl";

#######################################################
package XML::Khartes::Lexicon;
#######################################################

our @ISA = qw/XML::EasXML/;


# trouve les lemmes du mot $id.

sub XML::Khartes::lemma_nodes {
  my ($o, $id) = @_;
  my @lemma = ();
  my $l = $o->lexicon;
  foreach ($o->lemmatization_nodes($id)) {
    push @lemma, $l->find($_->att("lemme"));
  }
  return @lemma;
}

sub XML::Khartes::lemmatization_nodes {
  my ($o, $id) = @_;
  return () unless $id;
  return $o->xml->findnodes("/charte/lemmes/lemme[\@id-ref = '$id']");
}

sub XML::Khartes::lemma_ids {
  my ($o, $id) = @_;
  my @nodes = $o->lemma_nodes($id);
  my @return = ();
  foreach (@nodes) {
    push @return, $_->id if $_;
  }
  return @return;
}


sub XML::Khartes::lemma_entries {
  my ($o, $id) = @_;
  my @nodes = $o->lemma_nodes($id);
  my @return = ();
  foreach (@nodes) {
    push @return, $_->entry if $_;
  }
  return @return;
}

sub XML::Khartes::Syntax::Collection::lemma {
  require XML::Khartes::Syntax;
  my ($o) = @_;
  foreach ($o->twigs) {
    $_->lemma;
  }
  return $o;
}


sub XML::Khartes::Syntax::Twig::lemma {
  # merge les lemmes dans les POS terminaux
  require XML::Khartes::Syntax;
  my ($o) = @_;
  foreach ($o->{xml}->findnodes("descendant-or-self::C")) {
    my $ref = $_->att("id-ref");
    my @lemmes = $o->{khartes}->lemma_entries($ref);
    $_->att("lemme", join(';',@lemmes) ) ;
  }
  return $o;
}

sub XML::Khartes::Syntax::Collection::lemma_id {
  require XML::Khartes::Syntax;
  my ($o) = @_;
  foreach ($o->twigs) {
    $_->lemma_id;
  }
  return $o;
}

sub XML::Khartes::Syntax::Twig::lemma_id {
  # merge les id des lemmes dans les POS terminaux
  require XML::Khartes::Syntax;
  my ($o) = @_;
  foreach ($o->{xml}->findnodes("descendant-or-self::C")) {
    my $ref = $_->att("id-ref");
    my @lemmes = $o->{khartes}->lemma_ids($ref);
    $_->att("lemme_id", join(';',@lemmes) );
  }
  return $o;
}



sub XML::Khartes::lemmatize {
	my ($o, @new_lemma) = @_;
	# @new_lemma is an array of arrays
	# (
	# 	[ $w_id, $l_id, $remove ],
	# 	[ $w_id, $l_id, $remove ],
	# 	[ $w_id, $l_id, $remove ],
	# )
	# si $l_id est faux et $remove vrai, tous les lemmes attribus 
	# $w_id sont effacs
	my $lemmata;
	map {$lemmata = $_} $o->xml->findnodes("/charte/lemmes[1]");

        foreach my $lemma (@new_lemma) {
          my ($w_id, $l_id, $remove) = @$lemma;
          next unless my $w = $o->elt_id($w_id);
          next unless my $l = $o->lexicon->elt_id($l_id) or !$l_id and $remove;
          my @existing_lemma_nodes = $o->lemmatization_nodes($w_id);
          
          if ($remove) {
            if (!$l_id) {
              map {$o->delete($_) if $_->att("id-ref") eq $w_id} @existing_lemma_nodes;
              next
            } 
            else {
              map {$o->delete($_) if $_->att("id-ref") eq $w_id and $_->att("lemme") eq $l_id} @existing_lemma_nodes;
            }
          }
          
          my $exists = 0;
          foreach my $existing (@existing_lemma_nodes) {
            if ($existing->att("id-ref") eq $w_id and $existing->att("lemme") eq $l_id) {
              $exists = 1;
            }
          }

          unless ($exists) {
            my $new = $o->xml->createElement("lemme");
            $new->att("id-ref", $w_id);
            $new->att("lemme", $l_id);
            $lemmata->appendChild($new);
          }
        }
	return $lemmata;
}

# construit une liste des mots sous la forme
# 
# identifiant_du_lemme;catgorie;identifiant_du_mot
# 
sub XML::Khartes::words_as_ids {
	my ($o, $refresh) = @_;
	return $o->{_words_as_ids} if $o->{_words_as_ids} and !$refresh;
	my $string = "";
	foreach ( $o->xml->findnodes('/charte/texte/recto/w') ) {
		my $id = $_->id;
		foreach ($o->lemma_nodes($id)) {
			next unless $_;
			$string .= "\n";
			$string .= $_->id . ";";
			$string .= $_->cat . ";";
			$string .= $id . ";";
		}
	}
	$o->{_words_as_ids} = $string;
	return $string; 
}

# cre un petit lexicon qui correspond aux mots que comporte le corpus
sub XML::Khartes::Corpus::gloss {
  my ($o) = @_;
  my $l = $o->lexicon;
  my $subxml = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $root = $l->root->cloneNode(1);
  $subxml->setDocumentElement($root);
  my $sublex = XML::Khartes::Lexicon->new(xml => $subxml);
  my %texts = $o->texts;
  
  my %usedlemma = ();
  foreach (values %texts) {
    map { $usedlemma{$_->att("lemme")} = 1 } $_->xml->findnodes("/charte/lemmes/lemme");
    map { $usedlemma{$_->att("lemme")} = 1 } $_->xml->findnodes("/charte/analyse/*/*/nom");
  }
  
  my $ids = $sublex->{ids};
  foreach (values %$ids) {
    $root->removeChild($_) unless $usedlemma{$_->id};
  }
  $root->att("version", '$Id: no version$');
  return $sublex;
}

sub new {
	my $class = shift;
	my $self  = $class->SUPER::new(@_);
	bless($self, $class);
	return $self;
}

# trouve le lemme qui correspond  $id
sub find {
	my ($o, $id) = @_;
	my $lemmaNode = $o->elt_id($id);
	bless($lemmaNode, "XML::Khartes::Lexicon::Item") if $lemmaNode;
	return $lemmaNode;
}

sub items {
	my ($o) = @_;
	my @items = ();
	foreach ($o->xml->findnodes("/lemmes/lemme")) {
		push @items, bless($_, "XML::Khartes::Lexicon::Item");
	}
	return @items
}

# trouve la vedette du lemme qui correspond  $id
sub label {
	my ($o, $id) = @_;
	my $node = $o->find($id);
	return $node->entry if $node;
}
# trouve la def du lemme
sub def {
	my ($o, $id) = @_;
	my $node = $o->find($id);
	return $node->def if $node;
}

# trouve la cat du lemme
sub cat {
	my ($o, $id) = @_;
	my $node = $o->find($id);
	return $node->cat if $node;
}
# find bas sur la forme de la vedette. Retourne tous les noeuds dont
# $label est le label.
sub label_find {
	my ($o, $label) = @_;
	my @nodes;
	foreach my $article ($o->xml->findnodes("/lemmes/lemme")) {
		map {push @nodes, $article if $_->trimmed_text eq $label} $article->findnodes("./vedette");
	}
	return @nodes;
}

# ajoute une nouvelle entre
sub add {
	my ($o, $label, $cat, $def) = @_;
	$cat = "" unless $cat;
	$def = "" unless $def;
	$label = "undefined" unless $label;
	my $lemmaNode = $o->xml->createElement('lemme');
	my $labelNode = $o->xml->createElement('vedette');
	my $defNode = $o->xml->createElement('def');
	$o->root->appendChild($lemmaNode);
	$lemmaNode->appendChild($labelNode);
	$lemmaNode->appendChild($defNode);
	$defNode->att("cat", $cat);
	$labelNode->appendText($label);
	$defNode->appendText($def);
	$o->create_id($lemmaNode);
	bless($lemmaNode, "XML::Khartes::Lexicon::Item");
	return $lemmaNode;
}




#######################################################
package XML::Khartes::Lexicon::Item;
#######################################################

our @ISA = qw/XML::LibXML::Element/;

sub entry {
	my ($o, $lemma) = @_;
	my $lemmaNode;
	map {$lemmaNode = $_;} $o->findnodes("./vedette[1]");
	if ($lemma) {
		my $new = $lemmaNode->ownerDocument->createElement('vedette');
		$new->appendText($lemma);
		$lemmaNode->replaceNode($new);
	} else {
		$lemma = $lemmaNode->trimmed_text;
		$lemma =~ s/^\s*(.*?)\s*$/$1/;
		$lemma =~ s/\s+/ /sg;
	}
	return $lemma
}

sub def {
	my ($o, $def, $cat) = @_;
	my $defNode;
	map {$defNode = $_} $o->findnodes("./def");
	if ($def) {
		my $new = $defNode->ownerDocument->createElement('def');
		$new->appendText($def);
		$defNode->att("cat", $cat) if $cat;
		$defNode->replaceNode($new);
	} else{
		$def = $defNode->trimmed_text;
		$def =~ s/^\s*(.*?)\s*$/$1/;
		$def =~ s/\s+/ /sg;
	}
	return $def;
}

sub def_data {
	my ($o, $type, $value) = @_;
	return "" unless $type;
  my $defNode;
  map {$defNode = $_} $o->findnodes("./def[1]");
  if ($value) {
    $defNode->att($type, $value) ;
    $value =~ s/^\s*(.*?)\s*$/$1/;
    $value =~ s/\s+/ /sg;
  } else {
    $value = $defNode->att($type);
  }
  return $value;
}

sub soc {
  my ($o, $soc) = @_;
	$o->def_data("soc", $soc);
}

sub inst {
  my ($o, $inst) = @_;
	$o->def_data("inst", $inst);
}

sub cat {
  my ($o, $cat) = @_;
	$o->def_data("cat", $cat);
}
1;

#######################################################
package XML::Khartes::Lexicon::Collection;
#######################################################

1;

#######################################################
package XML::Khartes::Lexicon::Structure;
#######################################################
# TODO
#
# 
# permet de construire une structure qui pointe vers des items
# elle a la forme:
#
# <lexicon>
#	<section type="article">
#		<section target="_id" >
#			<section target="_id" />
#		</section>
#		<section target="_id">
#			<section target="_id">
#				<section target="_id" />
#			</section>
#			<section target="_id" />
#		</section>
#	</section>
# </lexicon>
our @ISA = qw/XML::EasXML/;

sub new {
	my $class = shift;
	my $self  = $class->SUPER::new(
		xml => &_build_base(),
		lexicon_file => undef,
		@_);
	bless ($self, $class);        
	return $self;
}

sub _build_base {
	my $xml = XML::LibXML::Document->new("1.0", "ISO-8859-1");
	my $root = $xml->createElement("lexicon");
	$xml->setDocumentElement($root);
	return $xml	
}

sub append_new_section {
	my ($o, $id) =@_;
	my $new = $o->new_section($id);
	$o->root->append($new);
}

sub new_section {
	my ($o, $id) = @_;
	my $new = $o->xml->createElement("section");
	$new->att("target", $id);
	return $new;
}

sub lexicon_merge {
	my ($o) = @_;
	my $lexicon = $o->{lexicon};
	$lexicon = XML::EasXML->new(file => $o->{lexicon_file}) unless $lexicon;
	my $glossary = $o->root->cloneNode(1);
	foreach my $node ( $glossary->findnodes("//*[\@target]") ) {
		my $target = $node->att("target");
		my $item = $lexicon->elt_id($target);
		$lexicon->xml->importNode($item);
		$node->appendChild($item);
	}
	return $glossary;
	  
}

1;

