#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
# mthodes de conversion des infos Khartes
# en DB mysql
# 
#######################################################
package XML::Khartes::Lexicon::DB;
#######################################################
use strict;

BEGIN {
  use Exporter;
  use DBI;
  use XML::Khartes;
  use Encode;
  use Data::Dumper;
  our @ISA = qw/Exporter/;
  our @EXPORT= qw/export/;
}

sub new {
  my $class = shift;
  my $o = {@_};
  bless($o, $class);
  return $o;
}

sub corpus {
  my ($o) = @_;
  return $o->{corpus};
}

sub populate {
  my ($o) = @_;
  my $db = $o->_db;
  my @sql = (
		$o->corpus->lexiconToSQL($db), 
		$o->corpus->occurrencesToSQL($db), 
	);
  print STDERR "Populating.\n";
  map {$_->[0]->execute(@{$_->[1]})} @sql
}

sub _db {
  my ($o) = @_;
  return $o->{db};
}

sub close {
  my ($o) = @_;
  $o->_db->disconnect;
}

sub DESTROY {
  my $o = shift;
  $o->close;
}

#######################################################
# Extension de XML::Khartes::Corpus
#######################################################

sub XML::Khartes::Corpus::lexiconToSQL {
  my ($o, $dbh) = @_;
  return $o->lexicon->toSQL($dbh);
}

sub  XML::Khartes::Corpus::lexiconDb { 
  my ($c, $host, $usr, $passwd, $db) = @_;
  $db = "khartes" unless $db;
  $usr = "" unless $usr;
  $host = "localhost" unless $host;
  $passwd = "" unless $passwd;

  my $dsn = "DBI:mysql:database=$db;host=$host";
  my $dbh = DBI->connect($dsn, $usr, $passwd);

  return XML::Khartes::Lexicon::DB->new(db => $dbh, corpus => $c);
}

sub XML::Khartes::Corpus::lexiconDbCreate {
  my ($c, $host, $usr, $passwd, $db) = @_;
  $db = "" unless $db;
  $usr = "" unless $usr;
  $host = "localhost" unless $host;
  $passwd = "" unless $passwd;

  my $dsn = "DBI:mysql:database=$db;host=$host";
  my $dbh = DBI->connect($dsn, $usr, $passwd);


  my @tables = (

    $dbh->prepare("CREATE DATABASE IF NOT EXISTS khartes"),

    $dbh->prepare("USE khartes"),

    # Les tables existantes sont dtruites
    $dbh->prepare("DROP TABLE IF EXISTS `lex:lemmes`"),
		$dbh->prepare("DROP TABLE IF EXISTS `lex:mots`"),
		$dbh->prepare("DROP TABLE IF EXISTS `lex:actualiser`"),


    # lemmes
		# Le lexicon
    $dbh->prepare("CREATE TABLE `lex:lemmes` (
			`xmlid` VARCHAR( 100 ) NOT NULL ,
      `vedette` VARCHAR( 256 ) NOT NULL ,
      `cat` VARCHAR( 20 ) NULL ,
      PRIMARY KEY ( `xmlid` ) 
    ) TYPE = MYISAM ;"),

		# mots
		# les mots attests
    $dbh->prepare("CREATE TABLE `lex:mots` (
      `id` INT NOT NULL AUTO_INCREMENT ,
			`ref_charte` VARCHAR( 40 ) NOT NULL ,
      `xmlid` VARCHAR( 100 ) NOT NULL ,
      `forme` VARCHAR( 256 ) NOT NULL ,
			`position` INT NOT NULL ,
      PRIMARY KEY ( `id` ) 
    ) TYPE = MYISAM ;"),

		# occurrences
		# la relation mots/lemmes
    $dbh->prepare("CREATE TABLE `lex:actualiser` (
      `xmlref_lemme` VARCHAR( 100 ) NOT NULL ,
      `ref_mot` VARCHAR( 100 ) NOT NULL 
    ) TYPE = MYISAM ;")
		
		
  );

  map { $_->execute; } @tables;
  
  return XML::Khartes::Lexicon::DB->new(db => $dbh, corpus => $c);
}

#######################################################
# Extension de XML::Khartes::Lexicon
#######################################################

sub XML::Khartes::Lexicon::toSQL {
	my ($o, $dbh) = @_;
	my @sql = ();

  foreach my $xml ($o->xml->findnodes("/lemmes/lemme")) {
		my $lexitem = bless($xml, "XML::Khartes::Lexicon::Item");
		my $vedette = $lexitem->entry;
		my $cat = $lexitem->cat;
		my $id = $xml->id;    
		my $r = $dbh->selectall_arrayref("SELECT COUNT(*) FROM `lex:lemmes` WHERE xmlid='$id'");
		if ($r->[0][0]) {
			push @sql, [$dbh->prepare("UPDATE `lex:lemmes` SET cat=?, vedette=? WHERE xmlid=?"), [latin1($cat, $vedette, $id)] ];
		} else {
			# push @sql, $dbh->prepare("INSERT INTO lemmes VALUES('','$id','$vedette','$cat')");
			push @sql, [$dbh->prepare("INSERT INTO `lex:lemmes` VALUES(?,?,?)"), [latin1($id, $vedette, $cat)] ];
		}
	}
	return @sql;
}

# FIXME dplacer
#
sub XML::Khartes::Corpus::occurrencesToSQL {
	my ($o, $dbh) = @_;
	my @sql = ();
	my %texts = $o->texts;
	foreach (values %texts) {
		my $ref_charte = $_->reference;
		push @sql, $_->occurrencesToSQL($dbh,$ref_charte);
	}
	return @sql;
}

sub XML::Khartes::occurrencesToSQL {
	my ($o, $dbh, $ref_charte) = @_;
	$ref_charte = $o->reference unless $ref_charte;
	my @sql = ();
	my $xml = $o->xml;
	
	my $i = 0;
	
	foreach ($xml->findnodes("/charte/texte/recto/w")) {
		my $string = $_->toString(1, "ISO-8859-1");
		$string =~ s/\s+/ /g;
		next unless $string;
		my $id = $_->id;
		push @sql, [$dbh->prepare("INSERT INTO `lex:mots` VALUES('',?,?,?,?) "), [$ref_charte,$id,$string,++$i] ];
		push @sql, [$dbh->prepare("SET \@word=LAST_INSERT_ID()"),[]];
		foreach ( $o->lemma_ids($id) ) {
			push @sql, [$dbh->prepare("INSERT INTO `lex:actualiser` VALUES(?,\@word) "), [$_] ]
		}
	}
	return @sql
}


sub latin1 {
	my @strings = @_;
	my @return;
	foreach (@strings) { push @return, encode("ISO-8859-1", $_); }
	return @return;
}

1;

__END__

# vim:sw=2 ts=2
