# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: Punctuation.pm 189 2006-02-27 15:20:18Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

package XML::Khartes::Punctuation;

require XML::Khartes;

use Carp;

use strict;



sub XML::Khartes::Syntax::Collection::punctuation {
  my ($o) = @_;
  foreach ($o->twigs) { $_->punctuation; }
  return $o;
}

sub XML::Khartes::Syntax::Twig::punctuation {
  my ($o, @args) = @_;
	return $o->{punctuation}{$args[0]}{$args[1]} if @args and $o->{punctuation};
  #
  # TODO: options
  #
  my $first_w = $o->_get_first_word;
  my $last_w = $o->_get_last_word;
  
  # trouve le signe qui prcde
  $o->{punctuation}{start}{punct} = _prevSign($first_w);
  
  # trouve la lettre marque qui commence
  $o->{punctuation}{start}{letter} = _prevLetter($first_w);

  # trouve le signe qui suit
  $o->{punctuation}{end}{punct} = _nextSign($last_w);
  
  # trouve la lettre marque qui suit
  $o->{punctuation}{end}{letter} = _nextLetter($last_w);
 
	return $o->{punctuation}{$args[0]}{$args[1]} if @args ;
  return $o->{punctuation};
}

sub _sign {
	# cherche le ponctogramme contigu  $w
	# dans la direction choisie
	my ($w, $direction) = @_;
	# direction =: prev|next
	return 0 unless $w;
	$direction .= "Sibling";
	my $node = $w->$direction;

	while ($node) {
		unless ($node) { $w = undef; last };
		$w = $node;
		my $name = $w->nodeName;
		if ($name eq 'ponct') { $w = $node; last };
		if ($name eq 'separateur') { $w = $node; last };
		$w = undef;
		if ($name eq 'w') {
			if ($node->textContent =~ /^\s*$/) { $node = $node->$direction; next }
			last
		}
		elsif ($name eq 'mponct') { $node = $node->$direction; next }
		elsif ($name eq 'part')   { $node = $node->$direction; next }
		elsif ($name eq 'sub')    { $node = $node->$direction; next }
		elsif ($name eq 'l')      { $node = $node->$direction; next }
		
		last
	};

	return $w unless $w and $w->nodeName !~ /^(?:separateur)|(?:ponct)$/;
	return 0;
}

sub _prevSign {
  my ($w) = @_;
  return _sign($w, "prev")
}

sub _nextSign {
  my ($w) = @_;
  return _sign($w, "next")
}

sub _prevLetter {
  my ($first_w) = @_;
  __find_c($first_w);
}

sub __find_c {
  # renvoie la lettre marque si elle dbute le mot
  # tient compte du marquage.
  my ($w) = @_;
  return 0 unless $w;
  EXT: foreach ($w->childNodes) {
    last EXT if $_->nodeType == 3 and $_->data !~ /^\s*$/;
    my $name = $_->nodeName;
    if ($name =~ /^(?:del)$/) { next; }
    if ($name =~ /^(?:dam)|(?:c)$/) { return $_; }
    if ($name =~ /^(?:add)$/) { return __find_c($_)}
  }
}

sub _nextLetter {
  my ($last_w) = @_;
  my $next_elt = $last_w;
  while ($next_elt) {
    # cette boucle permet de trouver le mot qui suit.
    $next_elt = $next_elt->nextSibling;
    last unless $next_elt;
    my $name = $next_elt->nodeName;
    next unless $name eq "w";
    next unless $next_elt->trimmed_text;
    last;
  }
  return __find_c($next_elt);
}

1;
__END__

