#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: not versioned$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

######################################################
package XML::Khartes::Server;
######################################################

use strict;
use lib "$ENV{HOME}/lib/perl";
use IO::Socket::INET;
use XML::Khartes;
use XML::Khartes::Export;
use XML::Khartes::Syntax;
use XML::Khartes::Punctuation;
use XML::Khartes::Syntax::Structure;
use Switch;
use Scalar::Util;
use Data::Dumper;


use XML::Khartes::Server::Client;
use XML::Khartes::Server::Method;
use XML::Khartes::Server::DataStruct;


#=====================================================#
sub  new 
#=====================================================#
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	print STDERR 
"
	Je suis le serveur de corpus Kharts
	------------------------------------

Copyright 2006, Universit de Lige, Nicolas Mazziotta, 
sous les termes de la licence artistique de Perl

ATTENTION: NE PAS M'UTILISER SUR INTERNET. 
Je ne suis pas scuris. 
Je ne limite pas la charge processeur alloue aux transactions. 
Utilisez la mthode 'server_clean' de l'API client pour viter les fuites de mmoire.

";
	print STDERR "[". localtime() ."]"." Construction du corpus\n";
	my $c = XML::Khartes::Corpus->new(@_);
	my $object = {
		server => IO::Socket::INET->new(
			LocalPort => 81, 
			Listen => 10, 
			Type => 1, 
			Reuse => 1
		), 
		corpus => $c,
	};
	$object->{reference}{$c} = $c;
	bless($object, $class);
	print STDERR "[". localtime() ."]"." J'coute  prsent le port 81\n\n";
	return $object;
}


#=====================================================#
sub  server
#=====================================================#
{	
	return shift->{server};
}

#=====================================================#
sub  corpus 
#=====================================================#
{	
	return shift->{corpus};
}

#=====================================================#
sub  shutdown 
#=====================================================#
{
	my ($o, $method) = @_;
	$method = 1 unless $method;
	return shutdown($o->server, $method);
}

#=====================================================#
sub close
#=====================================================#
{
	my ($o) = @_;
	return close($o->server);
}

#=====================================================#
sub clean
#=====================================================#
{
	my ($o, @ref)	= @_;
	unless (@ref) {
		map {
			delete $o->{reference}{$_};
		} keys %{$o->{reference}};
		delete $o->{reference};
		$o->{reference}{$o->corpus} = $o->corpus;
	} else {
		foreach (@ref) {
			delete $o->{reference}{$_};
		}
	}
}

#=====================================================#
sub accept 
#=====================================================#
{
	my ($o, @args) = @_;
	return $o->server->accept(@args);
}

#=====================================================#
sub reference 
#=====================================================#
{
	my ($o, $key) = @_;
	return $o->{reference}{$key};	
}

#=====================================================#
sub references 
#=====================================================#
{
	my ($o) = @_;	
	return join(",", keys %{$o->{reference}})
}

#=====================================================#
sub react 
#=====================================================#
{
	my ($o, $string) = @_;
	# TODO prvoir un header
	my $xmlmethod = XML::Khartes::Server::Method->new;
	my @data = $xmlmethod->object_method_args_from_xml($string, $o);
	return "" unless @data;
	my ($ref, $method, @args) = @data;
	my $return;
	print STDERR "[".localtime()."] Dbut de traitement";
	print STDERR "\n\tObjet:     $ref" if $ref;
	print STDERR "\n\tMthode:   $method" if $method;
	print STDERR "\n\tArguments: " . join(", ", @args) if @args;
	if ($ref and my $refobject = $o->reference($ref)) {
		eval {
			my $struct = XML::Khartes::Server::DataStruct->new;
			$return = $struct->data_to_xmlstring($o, $refobject->$method(@args));
		};
	} else {
		eval {
			my $struct = XML::Khartes::Server::DataStruct->new;
			$return = $struct->data_to_xmlstring($o, $o->$method(@args));
		};
	}
	if ($@) {
		$@ =~ /^\s*(.*)\s*$/s;
		print STDERR "\n[".localtime()."] Quelque chose s'est mal pass (je n'ai rien effectu!):";
		print STDERR "\n\t$1";
	} else {
		print STDERR "\n[".localtime()."] Traitement effectu\n";
	}
	return "" unless $return;
	# Si la chane ressemble  une adresse de rfrence perl, la stocker
	if ($return =~ /^(?:\w+::)*\w+=(?:SCALAR|ARRAY|HASH)\(0x[a-z0-9]{1,}\)$/o) {
		$o->{reference}{$return} = $return;
	}
	return $return;
}

#=====================================================#
sub  msg
#=====================================================#
{
	my ($message) = @_;
	print STDERR "[". localtime() ."]"." $message\n";
}

1;


__END__

#

