# Copyright (c) 2006 Universit de Lige, Nicolas Mazziotta 
# $Id$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Module client et module d'autochargement des mthodes
# pointant sur des objets sur le serveur
#
#######################################################
package XML::Khartes::Server::Client;
######################################################

use strict;
use IO::Socket::INET;
use Switch;
use File::Temp qw/tempfile/;

#=====================================================#
sub  new
#=====================================================#
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $object = {
		server => IO::Socket::INET->new("localhost:81"), 
	};
	bless($object, $class);
	return $object;
}

#=====================================================#
sub  server
#=====================================================#
{
	return shift->{server}
}

#=====================================================#
sub  close
#=====================================================#
{ 
	close(shift->server) 
}

#=====================================================#
sub  shutdown  
#=====================================================#
{
	my ($o, $mode) = @_;
	$mode = 1 unless $mode;
	my $server = $o->server;
	return shutdown($server, $mode);
}

#=====================================================#
sub  _connection_renew 
#=====================================================#
{
	my ($o) = @_;
	$o->{server} = IO::Socket::INET->new('localhost:81');
	return $o;
}

#=====================================================#
sub  corpus 
#=====================================================#
{
	my ($o) = @_;
	$o->_connection_renew;
	return $o->submit('corpus');	
}

#=====================================================#
sub  references 
#=====================================================#
{
	my ($o) = @_;
	$o->_connection_renew;
	return $o->submit('references');		
}

#=====================================================#
sub  server_clean 
#=====================================================#
{
	my ($o, @ref)	= @_;
	$o->_connection_renew;
	return $o->submit('clean', @ref);
}

#=====================================================#
sub  submit 
# Soumet une requte au serveur. Ces requtes ont la 
# forme d'appels de mthodes structures en XML par la
# fonction object_method_args_to_xml de la classe 
# abstraite XML::Khartes::Server::Method.
#
# Le rsultat consiste en des donnes XML que la 
# mthode xmlstring_to_data (de 
# XML::Khartes::Server::DataStruct) convertit en 
# structures de donnes Perl.
#=====================================================#
{	my ($o, @args) = @_;
	my $server = $o->server;
	my $xmlmethod = XML::Khartes::Server::Method->new();
	print $server $xmlmethod->object_method_args_to_xml(@args);
	$o->shutdown;
	my $answer = "";
	# TODO: lock files
	my ($fh, $tempfile) = tempfile("tmp/tmpXXXXXX", SUFFIX => ".xml", UNLINK => 1);
	while (<$server>) { print $fh $_ };
	CORE::close($fh);
	my $datastruct = XML::Khartes::Server::DataStruct->new();
	return $datastruct->xmlfile_to_data($tempfile);
}


1;

#######################################################
package  XML::Khartes::Server::Client::Ref;
#######################################################

use strict;

our $AUTOLOAD;

#=====================================================#
sub new 
#=====================================================#
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $object = {
		value => undef,
		@_
	};
	my $value = $object->{value};
	# Si la chane ressemble  une adresse de rfrence perl, 
	# consacrer et retourner l'objet 
	# TODO: quelque chose de plus propre (possible?)
	if ($value =~ /^(?:\w+::)*\w+=(?:SCALAR|ARRAY|HASH)\(0x[a-z0-9]{1,}\)$/o) {
		bless($object, $class);
		return $object;
	} 
	# sinon, retourner la chane
	else {
		return $value;
	}
}

#=====================================================#
sub server 
#=====================================================#
{	return shift->{server};}

#=====================================================#
sub value 
#=====================================================#
{
	my ($o, $ref) = @_;
	$o->{value} = $ref if $ref;
	return $o->{value};
}

#=====================================================#
sub dump
#=====================================================#
{
	my ($o) = @_;
	return Dumper($o);
}
#=====================================================#
sub AUTOLOAD 
#=====================================================#
{
	my ($o, @args) = @_;	
	my ($method) = $AUTOLOAD =~ /^.*::(.*?)$/;
	my @return;
	my $server = XML::Khartes::Server::Client->new("localhost::81");
	@return = $server->submit(
		$o->value,
		$method,
		@args
	);
	$server->shutdown;
	return @return if @return > 1;
	return shift @return;
}

#=====================================================#
sub DESTROY 
#=====================================================#
{}

1;

# vim:ts=2 sw=2

