# Copyright (c) 2006 Universit de Lige, Nicolas Mazziotta 
# $Id$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
#######################################################
package  XML::Khartes::Server::DOM::DataStruct;
# XXX Version DOM de XML::Khartes::Server::DataStruct
# Beaucoup plus propre, mais ne peut grer les fichiers 
# trop gros (avec 1Go de RAM et 1Go de SWAP, provoque 
# une erreur fatale sur un stream de 40Mo).
# 
# Une classe abstraite qui permet de convertir 
# structures de donnes Perl en XML et vice-versa.
#
# Certaines mthodes ont besoin qu'on leur passe 
# le serveur en argument (elles ne sont effectues 
# que ct serveur) pour garder une trace des 
# rfrences aux objets retourns.
#######################################################

use strict;
use Switch;
use XML::Khartes;

#=====================================================#
sub  new
#=====================================================#
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $object = {
		@_
	};
	bless($object, $class);
	return $object;
}

#=====================================================#
sub  data_to_xmlstring
#=====================================================#
{
	my ($o, $server, @args) = @_;
	my $xmldoc = XML::LibXML::Document->new;
	my $root = XML::LibXML::Element->new("data");
	$xmldoc->setDocumentElement($root);
	foreach (@args) {
		my $item_element = XML::LibXML::Element->new("member");
		$root->appendChild($item_element);
		$o->load_data_in_xmlnode($item_element, $_, $server);
	}
	return $xmldoc->toString;
}

#=====================================================#
sub  xmlstring_to_data
#=====================================================#
{
	my ($o, $xmlstring, $server) = @_;
	my $xmlparser = XML::LibXML->new(); 
 	my $xmldoc = "";
	# FIXME!! Il faudrait tout convertir en UTF-8
	# (en particulier l'encodage des textes. Il y a du boulot
	$xmlstring =~ s/(<\?xml version=['"]1.0['"])(\?>)/$1 encoding='ISO-8859-1' $2/;
	eval { $xmldoc = $xmlparser->parse_string($xmlstring); } or print STDERR $@ && return ();
	my @return = ();
	# FIXME: faire un parseur SAX (plus conomique)
	foreach ($xmldoc->root->findnodes("./member")) {
		# chaque membre n'a qu'un enfant ou du texte
		push @return, $o->read_data_from_xmlnode($_, $server);
	}
	if (@return > 1) {
		return @return;
	} else {
		return shift @return;
	}
}

#=====================================================#
sub  load_data_in_xmlnode
# transforme les rfrences en xml (rcursif)
# et les intgre au noeud xml pass en argument
# Retourne: XML::LibXML::Node (le noeud pass)
#=====================================================#
{
	my ($o, $xmlnode, $ref, $server) = @_;
	# l'argument $server permet de stocker les objets sur le serveur.
	my $type = ref $ref;
	# si l'argument n'est pas une rfrence, l'insrer tel quel
	# (important pour la rcursivit)
	unless ($type) {
		my $cdata = XML::LibXML::CDATASection->new("$ref");
		$xmlnode->appendChild($cdata);
		return $xmlnode;
	}
	if ($ref !~ /^.*=.*$/) {
		my $ref_node = XML::LibXML::Element->new(lc($type));	# le noeud prend le nom du type de rfrence
		$xmlnode->appendChild($ref_node);
		switch ($type) {
			case "SCALAR"	{
				$o->load_data_in_xmlnode($ref_node, ${$_}, $server)
			}
			case "ARRAY" {
				foreach (@{$_}) {
					my $item_node = XML::LibXML::Element->new("item");
					$ref_node->appendChild($item_node);
					$o->load_data_in_xmlnode($item_node, $_, $server);
				}
			}
			case "HASH" {
				my %hash = %{$_};
				foreach (keys %hash) {
					my $pair_node = XML::LibXML::Element->new("pair"); 
					my $key_node = XML::LibXML::Element->new("key");
					my $value_node = XML::LibXML::Element->new("value");

					$pair_node->appendChild($key_node);
					$pair_node->appendChild($value_node);
					$ref_node->appendChild($pair_node);

					$key_node->appendText($_); # pas de rcursivit pour les clefs
					$o->load_data_in_xmlnode($value_node, $hash{$_}, $server);
				}
			} 
		}
	}
	else {
		my $ref_node = XML::LibXML::Element->new("object");
		$xmlnode->appendChild($ref_node);
		if (ref $ref eq "XML::Khartes::Server::Client::Ref") {
			$ref_node->appendText($ref->value);
		} else {
			$ref_node->appendText($ref);
		}
		# chaque rfrence  un objet du serveur doit tre indexe
		$server->{reference}{$ref} = $ref if $server;
	}
	return $xmlnode;
}

#=====================================================#
sub  read_data_from_xmlnode
# traverse rcursivement un noeud xml et transforme
# ses noeuds en structures de donnes perl
#=====================================================#
{
	my ($o, $xmlnode, $server) = @_;
	my $data;
	foreach my $child ($xmlnode->childNodes) {
		switch ($child->nodeType) {
			case "3" { # Text Node
				$data = $child->toString;
			}
			case "4" { # CDATA Section Node
				($data) = $child->toString =~ /^<!\[CDATA\[(.*)\]\]>$/s;
			}
			case "1" { # Element Node
				my $type = $child->nodeName;
				switch ($type) {
					case "object" {
						unless ($server) {
							$data = XML::Khartes::Server::Client::Ref->new(value => $_->trimmed_text);
						} else {
							$data = $server->reference($child->trimmed_text);
						}
					}
					case "scalar" {
						my $scalar = $o->read_data_from_xmlnode($_, $server);
						$data = \$scalar;
					}
					case "array" {
						my @array = ();
						foreach ($child->findnodes("./item/*")) {
							push @array, $o->read_data_from_xmlnode($_, $server);
						}
						$data = \@array;
					}
					case "hash" {
						my %hash = ();
						foreach ($child->findnodes("./pair")) {
							my ($key, $value) = ("","");
							foreach ($_->findnodes("./*")) {
								my $name = $_->nodeName;
								if ($name eq "key") {
									$key = $_->trimmed_text;
								} else {
									$value = $o->read_data_from_xmlnode($_, $server)
								}
							}
							$hash{$key} = $value;
						}
						$data = \%hash;
					}
				}
			}
		}
	}
	return $data;
}

1;

# vim:ts=2 sw=2

