# Copyright (c) 2006 Universit de Lige, Nicolas Mazziotta 
# $Id$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
#######################################################
package  XML::Khartes::Server::DataStruct;
# Une classe abstraite qui permet de convertir 
# structures de donnes Perl en XML et vice-versa.
#
# Certaines mthodes ont besoin qu'on leur passe 
# le serveur en argument (elles ne sont effectues 
# que ct serveur) pour garder une trace des 
# rfrences aux objets retourns.
#######################################################

use strict;
use Switch;
use Data::Dumper;
use XML::Khartes;
use File::Temp qw/tempfile/;

#=====================================================#
sub  new
#=====================================================#
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $object = {
		@_
	};
	bless($object, $class);
	return $object;
}

#=====================================================#
sub  data_to_xmlstring
#=====================================================#
{
	my ($o, $server, @args) = @_;
	# On doit faire les documents xml " la main", sans API, 
	# parce que les fichiers obtenus sont trop longs pour tre 
	# parss en DOM!
	my $string = "<?xml version='1.0' encoding='ISO-8859-1'?><data>";
	foreach (@args) {
		$string .= "<member>";
		$string .= $o->load_data_in_xmlnode($_, $server);
		$string .= "</member>";
	}
	$string .= "</data>";
	return $string;
}

#=====================================================#
sub  xmlfile_to_data
#=====================================================#
{
	my ($o, $file, $server) = @_;
	my $xmlparser = XML::LibXML::SAX->new(Handler => SAXHandlers->new(server => $server)); 
	# FIXME!! Il faudrait tout convertir en UTF-8
	# (en particulier l'encodage des textes). Il y a du boulot
	eval { $xmlparser->parse_uri($file); } or print STDERR $@ && return ();
	my @return = @{$xmlparser->{Handler}{members}};
	if (@return > 1) {
		return @return;
	} else {
		return shift @return;
	}
}

#=====================================================#
sub  xmlstring_to_data
#=====================================================#
{
	my ($o, $xmlstring, $server) = @_;
	my $xmlparser = XML::LibXML::SAX->new(Handler => SAXHandlers->new(server => $server)); 
	# FIXME!! Il faudrait tout convertir en UTF-8
	# (en particulier l'encodage des textes). Il y a du boulot
	$xmlstring =~ s/(<\?xml version=['"]1.0['"])(\?>)/$1 encoding='ISO-8859-1' $2/;
	eval { $xmlparser->parse_string($xmlstring); } or print STDERR $@ && return ();
	my @return = @{$xmlparser->{Handler}{members}};
	if (@return > 1) {
		return @return;
	} else {
		return shift @return;
	}
}

#=====================================================#
sub  load_data_in_xmlnode
# transforme les rfrences en xml (rcursif)
# et les intgre au noeud xml pass en argument
# Retourne: XML::LibXML::Node (le noeud pass)
#=====================================================#
{
	my ($o, $ref, $server) = @_;
	# l'argument $server permet de stocker les objets sur le serveur.
	my $string = "";
	my $type = ref $ref;
	# si l'argument n'est pas une rfrence, l'insrer tel quel
	# (important pour la rcursivit)
	unless ($type) {
		return "<![CDATA[$ref]]>";
	}
	if ($ref !~ /^.*=.*$/) {
		$string .= "<".lc($type).">";
		switch ($type) {
			case "SCALAR"	{
				$string .= $o->load_data_in_xmlnode(${$_}, $server)
			}
			case "ARRAY" {
				foreach (@{$_}) {
					$string .= '<item>' .	$o->load_data_in_xmlnode($_, $server) . '</item>';
				}
			}
			case "HASH" {
				my %hash = %{$_};
				foreach (keys %hash) {
					$string .= '<pair>';
					$string .= "<key>$_</key>";
					$string .= '<value>' .$o->load_data_in_xmlnode($hash{$_}, $server). '</value>';
					$string .= '</pair>';
				}
			} 
		}
		$string .= "</".lc($type).">";
	}
	else {
		$string .= "<object>";
		if (ref $ref eq "XML::Khartes::Server::Client::Ref") {
			$string .= $ref->value;
		} else {
			$string .= $ref;
		}
		# chaque rfrence  un objet du serveur doit tre indexe
		$server->{reference}{$ref} = $ref if $server;
		$string .= "</object>";
	}
	return $string;
}


1;

#######################################################
package  SAXHandlers;
#######################################################

use strict;
use base qw (XML::SAX::Base);

use Data::Dumper;

#=====================================================#
sub  new
#=====================================================#
{
  my $class = shift;
	my %options = (
		members => [],
		process_stack => [],
		@_
	);
	my $o  = $class->SUPER::new(%options);
  bless($o, $class);
  return $o;
}

#=====================================================#
sub  start_element
#=====================================================#
{
	my ($o, $e) = @_;
  $o->SUPER::start_element($e);
	my $stack_pos = scalar @{$o->{process_stack}} - 1;
	my $name = $e->{Name};
	my $data;

	$o->{current} = $name;

	switch ($name) {
		case "array" {
			push @{$o->{process_stack}}, [];
		}
		case "hash" {
			push @{$o->{process_stack}}, {};
		}
	}
}

#=====================================================#
sub  end_element
#=====================================================#
{
	my ($o, $e) = @_;
  $o->SUPER::end_element($e);
	my $stack_pos = scalar @{$o->{process_stack}} - 1;
	my $name = $e->{Name};

	switch ($name) {
		case /^(?:member)|(?:argument)$/ {
			# mettre le dernier lment de la pile de process
			# comme dernier membre;
			push @{$o->{members}}, pop @{$o->{process_stack}};
		}
		case "object" {
			my $ref = pop @{$o->{process_stack}};
			unless ($o->{server}) {
				push @{$o->{process_stack}}, XML::Khartes::Server::Client::Ref->new(value => $ref);
			} else {
				push @{$o->{process_stack}}, $o->{server}->reference($ref);
			}
		}
		case "scalar" {
			my $scalar = $o->{process_stack}[$stack_pos];
			$o->{process_stack}[$stack_pos] = \$scalar;
		}
		case "item" {
			# on sort l'item de la pile; l'lment prcdent est toujours une 
			# rf  un tableau
			my $item = pop @{$o->{process_stack}};
			push @{$o->{process_stack}}, $item 
		}
		case "pair"	 {
			my $value = pop @{$o->{process_stack}};
			my $key = pop @{$o->{process_stack}};
			$o->{process_stack}[$stack_pos - 2]{$key} = $value;
		} 
	}

}



#=====================================================#
sub  characters
#=====================================================#
{
	my ($o, $text) = @_;
		push @{$o->{process_stack}}, $text->{Data};
}
1;

# vim:ts=2 sw=2

