# Copyright (c) 2005-2006 Universit de Lige, Nicolas Mazziotta
# $Id: Syntax.pm 267 2006-06-01 22:07:11Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

require XML::Khartes;
require XML::Khartes::Punctuation;

use Carp;

use strict;

package XML::Khartes::Syntax;


BEGIN {
  use Exporter;
  our @ISA = qw/Exporter/;
  our @EXPORT= qw/Syntax/;
}


sub XML::Khartes::Syntax {
  my ($o, %options) = @_;
  return $o->{syntax} if $o->{syntax} ;
  my ($tokenpath, $syntaxpath, $textpath, $type, $name, $resp) = 
  (
    $options{tokenpath}, 
    $options{syntaxpath},
    $options{textpath}, 
    $options{type},
    $options{name},
    $options{resp},
  );
  # $tokenpath est le xpath des units de base de l'analyse syntaxique
  # $type est le type d'analyse  
  # $syntaxpath est le xpath de l'analyse
  # $textpath est le xpath du texte de rfrence
  $tokenpath = "/*/texte/recto/w" unless $tokenpath;
  $syntaxpath = "/*/syntaxe[1]" unless $syntaxpath;
  $textpath = "/*/texte[1]" unless $textpath;
  $syntaxpath .= "[\@type = '$type']" if $type;
  $syntaxpath .= "[\@resp = '$resp']" if $resp;
  $syntaxpath .= "[\@name = '$name']" if $name;
  my ($text, $syntax);
  # TODO: un moyen de manipuler plusieurs analyses 
  foreach ( $o->xml->findnodes("$textpath") ) { $text = $_; last}
  foreach ( $o->xml->findnodes("$syntaxpath") ) { $syntax = $_; last}
  $syntax = $o->_build_syntax($tokenpath, $type, $name, $resp) unless $syntax;
  my $s = XML::Khartes::Syntax::Tree->new(
    khartes => $o,
    text => $text, 
    file => $o->{file},
    xml => $syntax,
    merged => 0
  ) unless !$text or !$syntax ;	
  $o->{syntax} = $s;
  return $s;
  return 0;
}

sub XML::Khartes::_build_syntax {
  my ($o, $tokenpath, $type, $name, $resp) = @_;
  $tokenpath = "/*/texte/recto/w" unless $tokenpath;
  $type = "syntaxe" unless $type;
  $name = $type unless $name;
  $resp = $ENV{USER} unless $resp; 
  my $lnode = "";
  my $root = $o->root;
  map {$lnode = $_} $root->findnodes("./linguistique[1]");
  unless ($lnode) {
    $lnode = $root->getOwnerDocument->createElement("linguistique");
    $root->appendChild($lnode);
  }
  my $s = $o->xml->createElement("analyse");
  $s->att("type", $type);
  $s->att("name", $name);
  $s->att("resp", $resp);
  $lnode->appendChild($s);

  foreach ($o->xml->findnodes($tokenpath)) {
    next if $_->trimmed_text =~ /^\s*$/;
    my $c = $o->xml->createElement("C");
    $c->att("id-ref", $_->id);
    $o->create_id($c);
    $s->appendChild($c);
  }
  return $s;
}

sub XML::Khartes::Corpus::Syntax {
  my ($o) = @_;
  my @twigs;
  foreach my $kh (values %{$o->{khartes}}) {
    push @twigs, $kh->Syntax
  }
  return  XML::Khartes::Syntax::Collection->new(
    twigs => [@twigs]
  );
}

1;

#######################################################
package XML::Khartes::Syntax::Tree;
#######################################################

our @ISA = qw/XML::Khartes::Syntax::Twig/;


sub new {
  my $class = shift;
  my $o  = $class->SUPER::new(@_);
  my @text = $o->{text}->findnodes("./recto/*[name() = 'w' or name() = 'ponct' or name() = 'l' or name() = 'separateur']");
  my @notes = $o->{xml}->findnodes("./notes/note");
  $o->{text} = [@text];
  $o->{notes} = [@notes];
  # TODO: rendre a moins tordu: l'objet text pass par 
  # l'utilisateur est transform; ce n'est pas trs explicite
  bless($o, $class);
  return $o;
}

sub khartes {
	my ($o) = @_;
	return $o->{khartes};
}

sub merge_forms {
  my ($o) = @_;
  return 1 if $o->{merged};
  foreach my $c ($o->{xml}->findnodes(".//C")) {
    my $id = $c->att("id-ref");
    foreach (@{$o->{text}}) {
      next if $_->id ne "$id";
      $c->appendChild($_->cloneNode(1));
      last
    }
  }
  $o->{merged} = 1;
}

sub index_pos {
  # indexe les pos en indiquant les mots vers lesquels les c qu'ils contiennent
  # pointent
  my ($o) = @_;
  return 1 if $o->{index};
  my %index;
  foreach my $pos ($o->{xml}->findnodes(".//*[name() = ('POS' or 'C')]")) {
    foreach my $c ($pos->findnodes("descendant-or-self::C")) {
      $index{$pos->id}{$c->att("id-ref")} = undef;
    }
  }
  $o->{index} = {%index};	
}


#######################################################
package XML::Khartes::Syntax::Collection;
#######################################################

# TODO: un moyen d'importer d'un fichier

sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $o = {
    file => undef,
    twigs => [],
    expression => undef,
    "time" => undef,
    #   previous => {},
    @_
  };
  bless($o, $class);
  return $o;
}

sub twigs {
  my ($o) = @_;
  return @{$o->{twigs}};
}

sub append {
  my ($o, @collection) = @_;
  # Attention: concatne n'importe quelle collection de resultats,
  # peu importe leurs expression et temps
  map {
    push @{$o->{twigs}}, @{$_->{twigs}};
  } @collection;
  $o->{'time'} = localtime();
  return $o;
}


sub extract {
  my ($o, %options) = @_;
  my $result = XML::Khartes::Syntax::Collection->new(
    options => {%options}, 
    expression => $options{expression},
    punctuation => $options{punctuation}, # voir XML::Khartes::Punctuation;
    twigs => [],
    pattern => $options{match},
    previous => {
      expression => $o->{expression},
      pattern => $o->{pattern},
      punctuation => $o->{punctuation},
      'time' => $o->{'time'},
      previous => $o->{'previous'}
    }
  );
  foreach my $twig ( @{$o->{twigs}} ) {
    my $collect = $twig->extract(%options);
    push @{$result->{twigs}}, @{$collect->{twigs}}
  }
  $result->{'time'} = localtime();
  return $result;
}

sub extract_chain {
  my ($o, @chain) = @_;
  # @chain is an array of hashrefs containing the options of each successive extraction
  # eg: 
  # {expression => "."} 
  # => {expression => ".."} 
  # => {expression => "..", focus => 1, notes => 1}
  my $pool = $o;
  foreach (@chain) {
    $pool = $pool->extract(%$_);
  }
  return $pool;
}


#=====================================================#
sub  toXMLString
#=====================================================#
{
	my ($o, %options) = @_;
	my ($time,$expression) = ($o->{'time'}, $o->{'expression'});
	$expression =~ s/'/&#x27;/g;
	my $output = "<?xml version='1.0' encoding='ISO-8859-1'?>\n<results time='$time' expression='$expression'>";
	# TODO prequeries
  foreach my $twig ( @{$o->{twigs}} ) {
    $output .= $twig->toXMLString(%options);
  }
	$output .= "\n</results>";
	return $output;
}

sub toXML {
  my ($o,%options) = @_;
  my $result = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $root = $result->createElement("results");
  $result->setDocumentElement($root);
  $root->att("expression", $o->{expression});
  $root->att("time", $o->{"time"});
  my $current = $o;
  my $prequeries;
  if ($current->{previous}) {
    $prequeries = $result->createElement("prequeries");
    $root->appendChild($prequeries);
  }
  # rcursivement: tout ce qui prcde
  while ($current = $current->{previous}) {
    last unless $current;
    my $prequery = $result->createElement("query");
    $prequeries->appendChild($prequery);
    $prequery->att("expression", $current->{expression});
    $prequery->att("time", $current->{"time"});
    $prequery->att("pattern", $current->{"pattern"});
  }
  foreach my $twig ( @{$o->{twigs}} ) {
    $root->appendChild( $twig->toXML(%options) );
  }
  return $result;
}

#=====================================================#
sub  working_copy
#=====================================================#
{
	my ($o) = @_;
	my @twig_copies = ();
	# on fait des copies des arbres de la collection
	foreach ($o->twigs) {
		push @twig_copies, $_->working_copy;
	}
	# on copie l'objet dans un hashage
	my %options = %$o;
	# on remplace les arbres de la copie par les copies de travail
	$options{twigs} = [@twig_copies];
	# on consacre et on retourne
	return bless({%options}, ref $o);
}

1;

#######################################################
package XML::Khartes::Syntax::Twig;
#######################################################


sub new {
  my $proto = shift;
  my $class = ref($proto) || $proto;
  my $o = {
    xml => undef,   # An XML::LibXML::Element
    text => undef,  # An XML::LibXML::Element:
    # le texte original du rsultat (attention: c'est fort diffrent pour le Tree
    parent => undef, # A XML::Khartes::Syntax::Twig
    tree => undef,   # A XML::Khartes::Syntax::Tree
    line => undef,
    file => undef,
    focus => undef, # hash list
    id => undef,    # the POS id
    notes => undef, # notes about the pos
    @_
  };
  bless($o, $class);
  unless ($o->{xml}){
    warn "result missing";
    return 0; 
  }

  # l'anctre arbre d'un arbre est lui-mme
  $o->{tree} = $o unless $o->{tree};
  return $o;
}

sub line {
  my ($o) = @_;
  return $o->{line};
}

sub file {
  my ($o) = @_;
  return $o->{file};
}

sub tree {
	my ($o) = @_;
	return $o->{tree};
}

sub khartes {
	my ($o) = @_;
	return $o->{khartes};
}

sub id {
  my ($o) = @_;
  return $o->{id};
}

sub xml {
  my $o = shift;
  return $o->{xml}
}

sub extract {
  my ($o, %options) = @_;
	%options = (
		line => 1,
		%options
	);

  my (	$expression,          $focus,          $match,          $notes , $punctuation, $find_line) = 
  (	$options{expression}, $options{focus}, $options{match}, $options{notes}, $options{punctuation}, $options{line});

  # TODO: better handling of focus (several levels, etc)
  my %focus;
  if ($focus) {
    foreach ($o->{xml}->findnodes("descendant-or-self::C")) {
      $focus{c}->{$_->att("id-ref")} = undef;
    }
    # TODO: more generic
    foreach ($o->{xml}->findnodes(".")) {
      $focus{pos} = {
				id => $_->id,
				#label => $_->att("label"),
				#role =>  $_->att("role"),
				#semantics => $_->att("part")
      };
      last;
    }
  }

  my @results;

  foreach ($o->{xml}->findnodes($expression)) {

    my $result = XML::Khartes::Syntax::Twig->new(
      parent => $o, 
      xml => $_, 
      file => $o->{file},
			khartes => $o->{khartes},
      id => $_->id,
      focus => {%focus}
    );

    # trouve l'arbre originel
    $result->{tree} = $result->_find_tree;

    # notes
    $result->{notes} = $result->_get_notes if $notes ;

    # vrifie si on matche l'expression passe
    if ($match) { next unless $result->_match($match); }

    # trouve le n de ligne
		if ($find_line) {
    	$result->{line} = $result->_get_line;
		} else {
			$result->{line} = undef
		}

    # ponctuation
    if ($punctuation) { $result->punctuation; }
    
    push @results, $result;
  };

  my $time= localtime();
  
  return XML::Khartes::Syntax::Collection->new(
    twigs => [@results], 
    expression => $expression,
    punctuation => $punctuation,
    "time" => $time
  );
}

sub extract_chain {
  my ($o, @chain) = @_;
  # @chain is an array of hashrefs containing the options of each successive extraction
  # eg: 
  # {expression => "."} 
  # => {expression => ".."} 
  # => {expression => "..", focus => 1, notes => 1}
  my $pool = $o;
  foreach (@chain) {
    $pool = $pool->extract(%$_);
  }
  return $pool;
}

sub _match {
  # FIXME: ajouter les espaces devant et derrire les mots
  my ($o, $match) = @_;
  $match = qr"$match"o;
  $o->{text} = $o->merge_forms(order => "original");
  my $text = "";
  foreach (@{$o->{text}}) { $text .= $_->textContent }
  $text =~ s/[\*\?\!\/]//go;
  $text =~ s/\s+/ /gos;
  $text =~ s/^\s+(.*?)\s+$/$1/go;
  unless ($text =~ $match) {return 0}
  return 1
}

sub _find_tree {
  # parcourt les objets de parents en parents jusqu' la racine de l'arbre.
  my ($o) = @_;
  my $tree = $o;
  while (my $parent = $tree->{parent}) {$tree = $parent}
  return $tree;
}

sub _get_notes {
  # trouve toutes les notes d'un rsultat en particulier
  # retourne une rfrence au tableau de ces notes
  my ($o) = @_;
  my @notes;
  foreach my $note (@{$o->{tree}->{notes}}) {
    my $ref = $note->att("id-ref");
    if ($o->{xml}->findnodes("descendant-or-self::*[\@id = '".$ref."']")) {
      push @notes, $note->cloneNode(1);
    }
  }
  return [@notes];
}

#=====================================================#
sub _get_line 
#=====================================================#
{
  # trouve la ligne d'un mot (du premier mot par dfaut).
  my ($o, $w) = @_;
  $w = $o->_get_first_word unless $w;
	return $o->{__line}{$w} if $o->{__line}{$w};
  my $l = 0;
  map {$l++} $w->findnodes("./preceding::l[ancestor::recto]") if $w;
	$o->{__line}{$w} = $l;
  return $l;
}

#=====================================================#
sub _get_first_word 
#=====================================================#
{
  my ($o)	= @_;
	return $o->{__first_word} if $o->{__first_word};
  my $c = $o->{tree}{text};
  foreach my $c (@$c) {
    next if $c->nodeName ne "w";
    my $id = $c->id;
    next unless $o->{xml}->findnodes("descendant-or-self::C[attribute::id-ref = '$id']");
		$o->{__first_word} = $c;
    return $c;
  }
}

#=====================================================#
sub _get_last_word 
#=====================================================#
{
  my ($o)	= @_;
	return $o->{__last_word} if $o->{__last_word};
  my $c = $o->{tree}{text};
  foreach my $c (reverse @$c) {
    next if $c->nodeName ne "w";
    my $id = $c->id;
    next unless $o->{xml}->findnodes("descendant-or-self::C[attribute::id-ref = '$id']");
		$o->{__last_word} = $c;
    return $c;
  }
}

#=====================================================#
sub  pos_edge
#=====================================================#
# Retourne une liste disant si le constituant est au bord de gauche 
# et au bord de droite (0,1), (0,0), (1,0) ou (1,1)
{
	my ($t, $first_word, $last_word) = @_;
	my $firstlinepos = 1;
	my $lastlinepos = 1;
	my $f_line = $t->_get_line($first_word);
	my $f_prev = $first_word->prevSibling("w");
	while ($f_prev and $f_prev->trimmed_text =~ /^\s*$/) {$f_prev = $f_prev->prevSibling("w")}
	my $l_line = $t->_get_line($last_word);
	my $f_next = $last_word->nextSibling("w");
	while ($f_next and $f_next->trimmed_text =~ /^\s*$/) {$f_next = $f_next->nextSibling("w")}
	if ($f_prev and $t->_get_line($f_prev) == $f_line) { $firstlinepos = 0 }
	if ($f_next and $t->_get_line($f_next) == $l_line) { $lastlinepos = 0 }
	return ($firstlinepos, $lastlinepos);
}

#=====================================================#
sub  pos_int
#=====================================================#
#
# Retourne la position dans la structure intgrante
#
{
	my ($t, $int, $begin, $end) = @_;
	# $begin et $end sont les positions de la structure intgrante
	my $k = $t->khartes;
	unless ($int) {
		my $int_twigs = "";
		my $role = "";
		while	(! $int_twigs or $role eq "groupe")	{ # TODO rendre plus gnrique
			$int_twigs = $t->extract(expression => "..");
			map {
				my $l_role = $_->xml->att("role");
				$role = $l_role if $l_role;
				$t = $_;
			} $int_twigs->twigs;
		};
		map { $int = $_ } $int_twigs->twigs;
 	}	
	$begin = $k->word_pos($int->_get_first_word->id) unless $begin;
	$end =   $k->word_pos($int->_get_last_word->id) unless $end;
	my $posintegr = "";
	my $first = $k->word_pos($t->_get_first_word->id);
	my $last = $k->word_pos($t->_get_last_word->id);
	my ($is_first, $is_last) = qw/0 0/;
	if ($first == $begin) {	$is_first = 1 }
	if ($last == $end) {	$is_last = 1 }
	return ($is_first, $is_last);
}

#=====================================================#
sub  words_count
#=====================================================#
{
	my ($t) = @_;
	my %ids;
	my $n = 0;
	foreach ($t->xml->findnodes('descendant-or-self::C')) {
		my $target = $_->att("id-ref");
		$n++ unless $ids{$target};
		$ids{$target}++;
	}
	return $n;
}


#=====================================================#
sub  toXMLString
# srialise directement les rsultats en une chane XML
# (plus lger que de recrer tout un document)
#=====================================================#
{
	my ($o, %options) = @_;
	$options{punctuation} = $o->{punctuation} unless defined $options{punctuation};
	my ($line,$file,$id) =  ($o->{line},$o->{file},$o->{id});
	$line = "" unless $line;
	# <result>
	my $output = "<result line='$line' file='$file' pos='$id'>";
  if ($o->{focus}) {
		# <focus>
		my $id = $o->{focus}{pos}{id};
    $output .= "<focus pos='$id'/>" if $id;
		# </focus>
  }
  if ($options{notes} and $o->{notes}) { 
		# DEPRECI
  }

  if ($options{punctuation}) {
		$output .= "<punctuation>";
		$output .= "<start>";
		$output .= "<punct>";
    $output .= $o->{punctuation}{start}{punct}->toString if $o->{punctuation}{start}{punct};
		$output .= "</punct>";
		$output .= "<letter>";
    $output .= $o->{punctuation}{start}{letter}->toString if $o->{punctuation}{start}{letter};
		$output .= "</letter>";
		$output .= "</start>";
		$output .= "<end>";
		$output .= "<punct>";
    $output .= $o->{punctuation}{end}{punct}->toString if $o->{punctuation}{end}{punct};
		$output .= "</punct>";
		$output .= "<letter>";
    $output .= $o->{punctuation}{end}{letter}->toString if $o->{punctuation}{end}{letter};
		$output .= "</letter>";
		$output .= "</end>";
		$output .= "</punctuation>";
  }

  # main
  map {  $output .= $_->toString } $o->merge_forms(%options);
	
	$output .= "</result>";
	# </result>
	return $output;
}

sub toXML {
	# NOTE: demande beaucoup de ressources!
	# pour la srialisation, prfrer toXMLString()
  my ($o, %options) = @_;
  $options{punctuation} = $o->{punctuation} unless defined $options{punctuation};
  my $result = XML::LibXML::Document->new("1.0", "ISO-8859-1");
  my $root = $result->createElement("result");
  # information about the focalized pos
  if ($o->{focus}) {
    my $focus = $result->createElement("focus");
    $focus->att("pos", $o->{focus}{pos}{id});
		#$focus->att("label", $o->{focus}{pos}{label});
		#$focus->att("role", $o->{focus}{pos}{role});
		#$focus->att("semantics", $o->{focus}{pos}{semantics});
    $root->appendChild($focus)
  }
  # notes
  if ($options{notes} and $o->{notes}) {
    my $notes = $result->createElement("notes");
    foreach (@{$o->{notes}}) {
      $notes->appendChild($_);
    }
    $root->appendChild($notes);
  }
  # ponctuation
  if ($options{punctuation}) {
    my $ponct = $result->createElement("punctuation");
    my $ponct_s = $result->createElement("start");
    my $ponct_e = $result->createElement("end");

    my $ponct_s_p = $result->createElement("punct");
    my $ponct_s_l = $result->createElement("letter");
    $ponct_s_p->appendChild($o->{punctuation}{start}{punct}->cloneNode(1)) if $o->{punctuation}{start}{punct};
    $ponct_s_l->appendChild($o->{punctuation}{start}{letter}->cloneNode(1)) if $o->{punctuation}{start}{letter};

    my $ponct_e_p = $result->createElement("punct");
    my $ponct_e_l = $result->createElement("letter");
    $ponct_e_p->appendChild($o->{punctuation}{end}{punct}->cloneNode(1)) if $o->{punctuation}{end}{punct};
    $ponct_e_l->appendChild($o->{punctuation}{end}{letter}->cloneNode(1)) if $o->{punctuation}{end}{letter};

    $ponct->appendChild($ponct_s);
    $ponct_s->appendChild($ponct_s_p);
    $ponct_s->appendChild($ponct_s_l);
    $ponct->appendChild($ponct_e);
    $ponct_e->appendChild($ponct_e_p);
    $ponct_e->appendChild($ponct_e_l);

    $root->appendChild($ponct);
  }

  # main
  $result->setDocumentElement($root);
  map {  $root->appendChild($_) } $o->merge_forms(%options);
  $root->att("line", $o->{line});
  $root->att("file", $o->{file});
  $root->att("pos", $o->{id});
  return $root;
}


sub merge_forms {
  my ($r, %options) = @_;
  my $s = $r->{xml};
  #
  # Options (* = default):
  # 	order => *original | syntax | both
  #       TODO: implement "both"
  # 	
  #	focus => focus
  # 	notes => 1|0
  #
  $options{order} = "original"    unless defined $options{order};
  $options{focus} = \$r->{focus}  unless defined $options{focus};
  $options{notes} = \$r->{notes}  unless defined $options{notes};

  my $node_id = $s->getAttribute("id");
  my $result = XML::LibXML::Document->new("1.0", "ISO-8859-1");

  my $line = "";

  my $firstloop = 1;

  my $disconstituency = 0;

  $result->setDocumentElement($result->createElement("tmp"));

  if ($options{order} =~ /^(original)|(both)$/) {

    # dans certains cas, la fusion a dj t effectue
    return $r->{text} if $r->{text};

    my $root = "";
    $root = $result->createElement("text");
    my $contextnode = $root;
    $result->root->appendChild($root);

    my $c = $r->{tree}{text};

    foreach my $c (@$c) {
      my $name = $c->nodeName;

      my $id = $c->getAttribute("id");
      my $print = 0;

      if ($name eq "w") { 
        $r->{tree}->index_pos;
        if (exists $r->{tree}{index}{$s->id}{$id}) { 
          $print = 1; 
        } else {
          next unless $c->hasChildNodes;
          $disconstituency = 1;
        }
      }
      else { 
        my $next = $c->nextSibling;
        while ($next) {
          last if $next->nodeName eq "w" and $next->hasChildNodes;
          $next = $next->nextSibling;
        }
				if ($next) {
					$id = $next->getAttribute("id");
					if ($s->findnodes("descendant-or-self::C[attribute::id-ref = '$id']")) {	
						$print = 1; 
					} 
					else { $disconstituency = 1; }
				}
      }
      # les formes  reprendre sont copies
      if ($print) {
        if ($firstloop) {
          # au premier passage, on vrifie la ligne
          
					if ($name eq "l") {
						$line = $c;					
					} else {
            my @line = $c->findnodes("./preceding::l");
            $line = pop @line;
          } 
          $line = $line->getAttribute("n") if $line;						
          $firstloop = 0;
          $disconstituency = 0;

        }

        my $clone = $c->cloneNode(1);

        if ($disconstituency) {
          $disconstituency = 0;
          my $disc = $result->createElement("gap");
          $contextnode->appendChild($disc);
        }
        $contextnode->appendChild($clone); 
      }

    }
  } 
  # sinon, la fusion correspond  l'arbre syntaxique;
  if ($options{order} =~ /^(syntax)|(both)$/) {
    $r->{tree}->merge_forms ;
    $r->{tree}{merged} = 1;
    my $t = $r->{xml}->cloneNode(1);
    $result->root->appendChild($t);
  }

  # en cas de focus
  if ($options{focus}) {
    foreach my $id (keys %{$r->{focus}{c}}) {
      map {
        if ($options{lemmata}) {
          my $text;
          unless (eval {$text = $r->{tree}{khartes};}) {
            warn "I cannot map any text to this twig"
          }
          else {
            # FIXME: remove warning
            # my $lemma = $text->label($id);
            my @lemma_entries = $text->lemma_entries($id);
            $_->att("lemma", join(';',@lemma_entries)) if @lemma_entries;	
          } 
        } 
        $_->att("occ", 1);
      } $result->findnodes(".//w[\@id = '$id']");

    }

  }

  return ($result->root->childNodes);	
}

# cre une copie "de travail" de l'arbre syntaxique. 
# Cette copie n'est jamais sauve
# TODO: faire en sorte que Syntax retourne une copie
sub working_copy {
  my ($o) = @_;
	# On recopie simplement l'objet
	my %copy = %{$o};
	# Ensuite, on remplace le xml par une copie
	$copy{xml} = $o->{xml}->cloneNode(1); 
	# on retourne la rfrence  la copie, consacre
	return(bless {%copy}, ref $o);	
}

1;

1;

__END__

=head1 NAME

XML::Khartes::Syntax 

=head1 SYNOPSIS

=over 2

=item use XML::Khartes

=item my $k = XML::Khartes->new(file => "foo.kml");

=item my $s = $k->Syntax;

=item my $c = $s->extract(expression => '//POS[@label = "foo"]');

=item my $c2 = $c->extract(expression => '..', focus => 1);

=item $c->toXML(order => "original")->print;

=back

=head1 TODO

method to extract punctuation

finish documentation

=head1 BUGS

Please report any bug to the author.

=head1 SEE ALSO

L<XML::Khartes>.

=head1 AUTHOR

Nicolas Mazziotta <NicolasMazziotta@ulgacbe>

=head1 COPYRIGHT

2005 Universit de Lige (Belgium), Nicolas Mazziotta.
This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.
