#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id$
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 
# mthodes de conversion des infos Khartes
# en DB mysql
# 
#######################################################
package XML::Khartes::Syntax::DB;
#######################################################
use strict;

BEGIN {
  use Exporter;
  use DBI;
  use XML::Khartes;
  use Encode;
  use Data::Dumper;
  our @ISA = qw/Exporter/;
  our @EXPORT= qw/export/;
}

sub new {
  my $class = shift;
  my $o = {@_};
  bless($o, $class);
  return $o;
}

sub corpus {
  my ($o) = @_;
  return $o->{corpus};
}

sub populate {
  my ($o) = @_;
  my $db = $o->_db;
  my @sql = $o->corpus->syntaxToSQL($db);
  print STDERR "Populating.\n";
  map {$db->do($_)} @sql
}

sub _db {
  my ($o) = @_;
  return $o->{db};
}

sub close {
  my ($o) = @_;
  $o->_db->disconnect;
}

sub DESTROY {
  my $o = shift;
  $o->close;
}

#######################################################
# Extension de XML::Khartes::Corpus
#######################################################

sub XML::Khartes::Corpus::syntaxToSQL {
  my ($o, $dbh) = @_;
  my @corpussql = ();
  my %texts = $o->texts ;
  foreach (keys %texts) {
    print STDERR "Converting data from $_.\n";
    push @corpussql, $texts{$_}->Syntax->toSQL($dbh);
  }
  return @corpussql;
}

sub  XML::Khartes::Corpus::syntaxDb { 
  my ($c, $host, $usr, $passwd, $db) = @_;
  $db = "khartes" unless $db;
  $usr = "" unless $usr;
  $host = "localhost" unless $host;
  $passwd = "" unless $passwd;

  my $dsn = "DBI:mysql:database=$db;host=$host";
  my $dbh = DBI->connect($dsn, $usr, $passwd);

  return XML::Khartes::Syntax::DB->new(db => $dbh, corpus => $c);
}

sub XML::Khartes::Corpus::syntaxDbCreate {
  my ($c, $host, $usr, $passwd, $db) = @_;
  $db = "" unless $db;
  $usr = "" unless $usr;
  $host = "localhost" unless $host;
  $passwd = "" unless $passwd;

  my $dsn = "DBI:mysql:database=$db;host=$host";
  my $dbh = DBI->connect($dsn, $usr, $passwd);


  my @tables = (

    "CREATE DATABASE IF NOT EXISTS khartes",
    
    "USE khartes",

    # Les tables existantes sont dtruites
    
    "DROP TABLE IF EXISTS `synt:constituants`   ",
    "DROP TABLE IF EXISTS `synt:contenir`     ",
    "DROP TABLE IF EXISTS `ponct:occurrences` ",
    
    # Units analyses

    "CREATE TABLE `synt:constituants` (
      `id` INT NOT NULL AUTO_INCREMENT ,
      `ref_charte` VARCHAR( 40 ) NOT NULL ,
      -- parent
      `ref_parent` INT , 
      -- rfrence dans la charte (id xml)
      `xmlid` VARCHAR( 40 ) NOT NULL ,		
      -- la morphologie est l'analyse de la forme
      `morpho` VARCHAR( 40 ) NOT NULL ,
      -- le label est une information complmentaire (NI, NP, NL,...)
      `label` VARCHAR( 40 ) NOT NULL ,
      -- la catgorie est le sous-type
      `role` VARCHAR( 40 ) NULL,
      -- les deux types de segmentation sont dans la mme table
      `ordre` ENUM( 'diplomatique', 'syntaxique' ) NOT NULL ,
			-- mot
			`ref_mot` INT ,
      PRIMARY KEY ( `id` ) 
    ) TYPE = MYISAM ;",


		
    # ponctuation

    "CREATE TABLE `ponct:occurrences` (
      `id` INT NOT NULL AUTO_INCREMENT ,
      -- Une partie est marque  un ou plusieurs endroits par un ou plusieurs signes
      `ref_constituant` INT NOT NULL ,
      `forme` VARCHAR( 30 ) DEFAULT ' ' ,
      -- rfrence dans la charte (id xml)
      `reference` VARCHAR( 40 ) NULL, 	
      `position` ENUM( 'before','start','end','after' ) NOT NULL ,
      -- comme il peut y avoir plusieurs signes, ils ont un numero d'ordre
      `numero` INT NOT NULL ,
      PRIMARY KEY ( `id` ) 
    ) TYPE = MYISAM ;"
  );

  map {
    $dbh->do($_);
  } @tables;
  
  return XML::Khartes::Syntax::DB->new(db => $dbh, corpus => $c);
}

#######################################################
# Extension de XML::Khartes::Synatx::Twig
#######################################################

sub XML::Khartes::Syntax::Twig::toSQL {
	
	my ($o, $dbh) = @_;
	my $charte_id = $o->{khartes}->reference;
	my @sql = ();
	my $r = $o->extract(expression => ".//*[name() = ('POS' or 'C')]"); 

	foreach my $twig ($r->twigs) {

		my $xml = $twig->{xml};
		my $name = $xml->nodeName;
		my $pos_id = $xml->id;
		my $parent = $xml->parentNode;
		my $parent_id = $parent->id;
		
		my @children;
		my $word_id = "";

		if ($name eq 'C') {
			$word_id = $xml->att("id-ref");
		}
		elsif ( @children = $xml->findnodes("./*[count(following-sibling::*) + count(preceding-sibling::*) = '0']") ) {
			foreach (@children) { $word_id = $_->att("id-ref"); last}
		}
		
		$word_id = "" unless $word_id;
		
		my $ordre = "syntaxique";
		my $label = $xml->att("label");
		my $role = $xml->att("role");
		my $morpho = $xml->att("morpho");
		$label = "" unless $label;
		$role = "" unless $role;
		$morpho = "" unless $morpho;
		
		push @sql, latin1("SET \@parent_id = (SELECT id FROM `synt:constituants` WHERE ref_charte='$charte_id' and xmlid='$parent_id')");
		push @sql, latin1("SET \@word_id = (SELECT id FROM `lex:mots` WHERE ref_charte='$charte_id' and xmlid='$word_id')");

		my $r = $dbh->selectall_arrayref("SELECT COUNT(*) FROM `synt:constituants` WHERE ref_charte='$charte_id' and xmlid='$pos_id'");
		if ($r->[0][0]) {
			push @sql, latin1("UPDATE `synt:constituants` 
				SET morpho='$morpho',label='$label',ordre='$ordre',ref_mot=\@word_id,ref_parent=\@parent_id 
				WHERE ref_charte='$charte_id' and xmlid='$pos_id'"
			);
			push @sql, latin1("SET \@constituant_id = LAST_INSERT_ID()");
		} else {
			push @sql, latin1("INSERT INTO `synt:constituants` 
				VALUES('','$charte_id',\@parent_id,'$pos_id','$morpho','$label','$role','$ordre',\@word_id)"
			);
			push @sql, latin1("SET \@constituant_id = LAST_INSERT_ID()");
		}

		# TODO: sparer
		my $p = $twig->punctuation;
		my %positions = ();
		$positions{start} = $p->{start}{letter};
		$positions{before} = $p->{start}{punct};
		$positions{end} = $p->{end}{punct};
		$positions{after} = $p->{end}{letter};
		foreach (qw/start before end after/) {
			my $text = "";
			my $id = "";
			if ($positions{$_}) {
				$text = $positions{$_}->trimmed_text;
				$id = $positions{$_}->id;
			}
			push @sql, latin1("INSERT INTO `ponct:occurrences` VALUES('',\@constituant_id,'$text','$id','$_','')");
		}
	}
	return @sql;
}


sub latin1 {
  return encode("ISO-8859-1", shift);
}

1;

__END__

# vim:ts=2 sw=2
