#!/usr/bin/perl -w
# Copyright (c) 2005-2006 Universit de Lige, Nicolas Mazziotta
# $Id: Structure.pm 267 2006-06-01 22:07:11Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
# 

#######################################################
package XML::Khartes::Syntax::Structure;
#######################################################
use strict;

BEGIN {
  use XML::Khartes::Syntax;
  use Data::Dumper;
  our @ISA = qw/Exporter/;
  our @EXPORT= qw/export/;
}

	#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
	#		 Extensions
	#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#

sub XML::Khartes::Syntax::Collection::get_structure {
	my ($o, %options) = @_;
	my @results = ();
	foreach ($o->twigs) {
		my @structures = $_->get_structure->structures;
		push @results, @structures if @structures;
	}
	return XML::Khartes::Syntax::Structure::Collection->new( structures => [@results] );
}

sub XML::Khartes::Syntax::Twig::get_structure {
	# XXX ATTENTION: mthode trs complique
	
	# - retourne une liste dcrivant la/les structure(s) (rcriture(s)) du twig
	# - prend une srie d'arguments qui permettent de traite diffremment les lments
	my $o = shift;
	my %passed = @_;
	my $info = delete $passed{info};
	$info = {} unless $info;
	my %options = (
		# valeurs par dfaut du projet Kharts
		att => "role",							# L'attribut qui contient les infos de structure
		ignore =>	[qw/ignore/],			# Les valeurs de l'attribut  ignorer
		default => "base",				  # La valeur assigne par dfaut aux lments dont l'attribut 'att' est FAUX
		flatten => [qw/args/],			# Valeur  remplacer par ses enfants
		duplicate => [qw/groupe/],	# Les valeurs de l'attribut qui provoquent un ddoublement de la description
		coord => [qw/Co/], 					# Les valeurs de l'attribut qui marquent la coordination
																
		# informations  prcalculer TODO (tout n'est pas implment)
		info => {
			function => undef,     
			coord => undef,
			relat => undef,
			nature => undef,
			"length" => undef,
			discont => undef,
			line => undef,
			posintegr => undef,
			posP0 => undef,
			%$info
		},
		%passed
	);
	# on replace toutes les valeurs des options dans des hachages, pour pouvoir faire des tests d'existence
	my %ignore = (); map {$ignore{$_}++} @{$options{ignore}} if $options{ignore};
	my %duplicate = (); map {$duplicate{$_}++ unless $duplicate{$_}} @{$options{duplicate}} if $options{duplicate};
	my %coord = (); map {$coord{$_}++ unless $coord{$_}} @{$options{coord}} if $options{coord};
	my %flatten = (); map {$flatten{$_}++ unless $flatten{$_}} @{$options{flatten}} if $options{flatten};

	my $att = $options{att};
	
	my @structures = ();

	# deux groupements d'arguments sont extraits: ceux qui ne provoquent pas un ddoublement de structure 
	# et les autres
	my @basic_stucture = ();
	my @duplicate_structures = ();

	# pour chaque enfant de la structure tudie
	foreach my $twig ($o->extract( expression => "./*", line => $info->{line} )->twigs) {
		my $value =	$twig->xml->att($att);
		$value = $options{default} unless $value;
		
		# Si la valeur d'attribut est  ignorer, on passe au suivant
		next if $ignore{$value} or $coord{$value};

		if ($flatten{$value}) {
			# la commande est rcursive
			# efface simplement l'lment en le remplaant par son contenu
			foreach ($twig->get_structure(%options)){ 
				foreach ($_->structures) { push @basic_stucture, $_->constituents	}
			}
		}
		elsif ($duplicate{$value}) {
			# la commande est rcursive
			# il faut empcher les structures de s'applatir, sinon, les groupements ventuels
			# sont ignors et chaque lment de leur contenu provoque un ddoublement.
			#
			# Pour la raison inverse, il faut applatir les structures qui provoquent des ddoublements
			push @duplicate_structures, $twig->get_structure(%options, flatten => [keys %duplicate]) 
		}
		else {
			push @basic_stucture, XML::Khartes::Syntax::Structure::Constituent->new(twig => $twig, name => $value,);
		}
	}
	
	if (@duplicate_structures) {
		# chaque constituant d'une structure qui provoque le ddoublement cre une structure 
		# supplmentaire  ajouter  la collection
		foreach my $c (&__extract_from_duplicate_recursive(\@duplicate_structures, \%flatten, %options)) {
			push @structures, XML::Khartes::Syntax::Structure->new(constituents => [@basic_stucture, @$c])
		}
	}	
	else {
		# par contre, quand il n'y a pas de ddoublement, on ajoute simplement une structure
		push @structures, XML::Khartes::Syntax::Structure->new(constituents => [@basic_stucture])
	}

	return XML::Khartes::Syntax::Structure::Collection->new( structures => [@structures] );
}

#=====================================================#
sub  __extract_from_duplicate_recursive
#=====================================================#
{
	my ($duplicate_structures, $flatten, %options) = @_;
	return () unless $duplicate_structures;
	my @return = ();
	
	# on explore progressivement le tableau en combinant chaque 
	# lment avec tous les autres
	while (my $coll = shift @$duplicate_structures) {
		my @base = ();
		foreach ($coll->_extract_from_duplicate_collection($flatten, %options)) {
			push @base, $_
		};
		my @updated_return;
		foreach my $stuct (@return) {
			foreach (@base) {
				push @updated_return, [@$_, @$stuct];
			}
		}
		unless (@return) {@return = @base}
		# la combinaison remplace l'ancienne valeur  retourner
		else { @return = @updated_return }
	}

	return @return;
}

	#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#
	#		 Class methods
	#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#-#

sub new {
  my $class = shift;
  my $o = {
		twig => undef,
		constituents => [],
		
		function => undef,
		coord => undef,
		relat => undef,
		nature => undef,

		"length" => undef,
		discont => undef,
		line => undef,
		posintegr => undef,
		posP0 => undef,
		@_
	};
  bless($o, $class);
  return $o;
}

sub reference {	return shift->{twig}->khartes->reference }
sub file { return shift->{twig}->khartes->file }
sub pos_id { return shift->{twig}->xml->id }
sub line { return shift->{twig}->khartes->line }
sub twig { return shift->{twig} }

sub lemma {
	# renvoie les lemmes qui correspondent  la cl
	my ($o, $rules, $att) = @_;
	$rules = [ 
		{ E0 => "./*[\@role = 'P0']" }, 
		{ P0 => "self::*[C[\@lemme != 'avoir' and  \@lemme != 'estre']]" },
		{ P0 => "self::*[\@morpho = 'pp']" },
		{ P0 => "self::*[\@morpho = 'ppt']" },
		{ P0 => "self::*" },
	] unless $rules;
	$att = "lemme" unless $att;

	my %constituents;
	
	foreach ($o->constituents) {
		push @{$constituents{$_->{name}}}, $_->{twig}
	}

	my $passif = "";
	
	foreach my $rule (@$rules) {
		my $lemma;
		foreach my $key ( keys %$rule ) {
			my $path = $rule->{$key};
			foreach (@{$constituents{$key}}) {
				foreach ($_->xml->findnodes($path."/C")) {
					$lemma = $_->att($att);
					last
				};
			}
		}
		
		return $lemma if $lemma;
		
	}
	
	return "";

}

sub aux {
	my ($o, $role) = @_;	
	
	foreach ($o->constituents) {
		if ($_->{name} eq $role) {
			# PAS PROPRE XXX
			my $xml = $_->twig->xml;
			my $aux = $xml->att("aux");
			return $aux if $aux;
		}
	}
	return '';
}


sub scheme {
	my ($o, $ignore) = @_;
	@$ignore = qw/E0 C5/ unless defined $ignore;
	my %ignore;
	map {$ignore{$_}++} @$ignore;
	my %constituents;
	foreach ($o->constituents) {
		my $name = $_->{name};
		$constituents{$name}++ unless !$name or $ignore{$name}
	}
	return join("+", sort keys %constituents);
}

sub toString {
	my $o = shift;
	my %options = (
		reference => 1,
		lemma => "",      # les rgles pour trouver le mot qui porte le lemme; 
		scheme => [],     # les clefs ignores
											# pour lemma et scheme, une valeur fausse appelle les rgles par dft; une undef empche la recherche
		constituents => [],  # la liste des noms des  constituants  imprimer
		@_
	);
	my @c = $o->constituents(@{$options{constituents}});
	return () unless @c;
	my @results = ();
	
	push @results, "# ref: " . $o->file . "#" . $o->pos_id if $options{reference};
	push @results, "# lemma: " . $o->lemma($options{lemma}) if defined($options{lemma});
	push @results, "# scheme: " . $o->scheme($options{scheme}) if defined($options{scheme});
	push @results, "# diathese: passif" if $o->aux("P0");
#	@options{qw/reference lemma scheme/} = (undef, undef);

	foreach (@c) { 
			push @results, $_->toString(%options); 
	}
	return @results;
}

sub constituents {
	my ($o, @keys) = @_;
	my $c = $o->{constituents};
	return @$c if $c and ! @keys;
	# si on a donn une liste de constituants
	my %keys = ();
	map { $keys{$_}++ } @keys;
	my @filtered = ();
	foreach (@$c) {
		push @filtered, $_ if $keys{$_->{name}} 
	}
	return @filtered;
}

1;

#######################################################
package XML::Khartes::Syntax::Structure::Constituent;
#######################################################
use strict;

sub new {
	my $class = shift;
	my $o = {
		twig => undef,
		name => undef,
		@_
	};
	bless($o, $class);
	return $o;
}

sub twig { return shift->{twig};}
sub key {	return shift->{name};}

sub toString {
	my $o = shift;
	my $string = "";
	my %options = (
		bare => 1,
		@_
	);
	$string .= $o->{name};
	unless ($options{bare}) {
		$string .= ": " ;
		$string .= $o->{twig}->toXML->convert('TXT', 'Export', {nosuffix => 1});
	};
	return $string;
}


1;

#######################################################
package XML::Khartes::Syntax::Structure::Collection;
#######################################################
use strict;

sub new {
	my $class = shift;
	my $o = {
		structures => undef,
		@_
	};
	bless($o, $class);
	return $o;
}

sub toString {
	my ($o, %options) = @_;
	my @s = $o->structures;
	return () unless @s;
	my @results = ();
	foreach (@s) {
		push @results, [$_->toString(%options)];
	}
	return @results;
}

sub structures {
	my $o = shift;
	my $s = $o->{structures};
	return @$s if $s;
	return ();
}

#=====================================================#
sub  _extract_from_duplicate_collection
#=====================================================#
{
	my ($coll, $flatten, %options) = @_;
	my @return;
	foreach ($coll->structures) {
		foreach my $c ($_->constituents) {
			my @c = ();
			# c'est ici qu'on applatit si besoin
			if ($flatten->{$c->key}) {
				foreach ($c->twig->get_structure(%options)) {
					foreach ($_->structures) {
						push @c, $_->constituents
					}
				}
			} else {
				@c = ($c);
			}
			push @return, \@c
		}
	}
	return @return;
}



1;

__END__

# vim:ts=2 sw=2
