# Copyright (c) 2005-2006 Universit de Lige, Nicolas Mazziotta
# $Id: Text.pm 58 2005-11-02 12:59:17Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

=head1 NAME

XML::Khartes::Text

=head1 SYNOPSIS

=over 2

=back

=cut

require XML::Khartes;

use Carp;

use strict;

package XML::Khartes::Text;

BEGIN {
	use Exporter;
	our @ISA = qw/Exporter/;
	our @EXPORT= qw/Text/;
}


sub XML::Khartes::Token {
	my ($o, $id) = @_;
	my $xml = $o->xml->elt_id($id);
	return XML::Khartes::Text::Token->new(kh => $o, xml => $xml);
}

=head1 XML::Khartes EXTENSION

=over 2 

=item Method

=over 2 

=item B<XML::Khartes-E<gt>Text>

=item I<parameters>: none.

=item I<returns>: the XML::Khartes::Tree of the document.

=cut

#######################################################
package XML::Khartes::Text::Token;
#######################################################


sub new {
	my $proto = shift;
        my $class = ref($proto) || $proto;
	my $o = {
		kh => undef,
		xml => undef,
		type => undef,
		@_
	};
	$o->{kh}->provide_ids($o->{xml});
	bless($o, $class);
	return $o;
}

sub xml {
	my $o = shift;
	return $o->{xml};
}

sub insert {
	my ($o, $spot, $position) = @_;
	my $xmlnode = $o->{xml};
	my $xmldoc = $xmlnode->getOwnerDocument;
	my $parent = $spot->xml->parentNode;
	my $xml = $spot->xml;
	POSITION: {
		if ($position eq "before") {
			$parent->insertBefore($xmlnode, $xml);
			my $ref = $o->{kh}->create_id($xmlnode);
			my $id = $xml->id;
			foreach my $pointer ( $xmldoc->findnodes(".//*[\@id-ref = '$id']") ) {
				if ($pointer->nodeName eq "C") {
					my $c = $xmldoc->createElement("C");
					$c->att("id-ref", $ref);
					$o->{kh}->create_id($c);
					$pointer->parentNode->insertBefore($c, $pointer);
				}
			}
			last POSITION;
		}
		if ($position eq "after") {
			$parent->insertAfter($xmlnode, $xml);
			my $ref = $o->{kh}->create_id($xmlnode);
			my $id = $xml->id;
			foreach my $pointer ( $xmldoc->findnodes(".//*[\@id-ref = '$id']") ) {
				if ($pointer->nodeName eq "C") {
					my $c = $xmldoc->createElement("C");
					$c->att("id-ref", $ref);
					$o->{kh}->create_id($c);
					$pointer->parentNode->insertAfter($c, $pointer);
				}
			}
			last POSITION;
		}
		# overwrites the spot
		if ($position eq "over") {
			my $id = $xml->id;
			$parent->replaceNode($xmlnode);
			$xmlnode->setAttribute("id", $id);
			$o->{kh}{ids}{$id} = $xmlnode;
			last POSITION;
		}
	}
	
	return $o;
}



1;

#######################################################
package XML::Khartes::Text::Token::Collection;
#######################################################

sub new {
	my $proto = shift;
        my $class = ref($proto) || $proto;
	my $o = {
		tokens => [],
		@_
	};
	bless($o, $class);
	return $o;
}

sub append {
	my ($o, $t) = @_;
	push @{$o->{tokens}}, $t;
	return @{$o->{tokens}};
}

sub insert {
	my ($o, $spot, $position) = @_;
}
1;


#######################################################
package XML::Khartes::Text::Word;
#######################################################

our @ISA = qw/XML::Khartes::Token/;

sub new {
	my $class = shift;
	my $o  = $class->SUPER::new(@_);
	bless($o, $class);
	return $o;
}

sub replace {
	
}

1;

#######################################################
package XML::Khartes::Text::Ponct;
#######################################################

our @ISA = qw/XML::Khartes::Token/;

1;


__END__

=back

=head1 TODO

method to extract punctuation

finish documentation

=head1 BUGS

Please report any bug to the author.

=head1 SEE ALSO

L<XML::Khartes>.

=head1 AUTHOR

Nicolas Mazziotta <NicolasMazziotta@ulgacbe>

=head1 COPYRIGHT

2005 Universit de Lige (Belgium), Nicolas Mazziotta.
This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.
