#!/usr/bin/perl -w
# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: Kh-edit.pl 196 2006-03-19 09:33:02Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

use strict;		

our $VERSION = '0.3';

#######################################################
#		 Predeclarations
#######################################################

# installation path
my ($name,$path,$suffix);

#######################################################
#		 Imports
#######################################################

use Getopt::Long qw(:config bundling auto_help auto_version);
use Carp;
use File::Basename;
use Pod::Usage;

use Tk;
use Tk::LabFrame;
use Tk::ROText;
use Tk::BrowseEntry;

BEGIN {
  # find where the program has been installed
  my $filename = $0;
  eval {$filename = readlink $0 if -l $0};
  ($name,$path,$suffix) = fileparse($filename,qw/pl/);
}

END {
  print STDERR "\nExited without error code [".localtime()."]\n";
}

# load homemade modules
use lib "$path";
use XML::Khartes;
use Tk::KhIDE;

#######################################################
#		 globals
#######################################################

# parse command-line
my (
  $lexiconfile,
  $infopath,
  $textpath,
  $analysispath,
  $rcfile,
  $goto,
) = (
  "$ENV{KHLEXIQUE}/lemmes.xml",
  "/*/attribute::*",
  "/*/texte[1]",
  "/*//*[name() = 'syntaxe' or name() = 'diplo' or (name() = 'analyse' and ancestor::linguistique)]",
  "",
  ""
);

my %options = (
  'lexiconfile' => \$lexiconfile,
  'infopath' => \$infopath,
  'textpath' => \$textpath,
  'analysispath' => \$analysispath,
  'rcfile' => \$rcfile,
  'goto' => \$goto,  
);

GetOptions (\%options, 
"lexiconfile:s",
"infopath:s",
"textpath:s",
"analysispath:s",

"goto:s",

"rcfile:s",

"displaypanels:s",
"infoboxes:s",

"author",
"licence",
"manual"
) or pod2usage(2);

# return info if needed

if ($options{author}) {	pod2usage( -verbose => 99, -sections => 'AUTEUR', -exitval => 0) }
if ($options{licence}) { pod2usage(-verbose => 99, -sections => "LICENCE", -exitval => 0) }
if ($options{manual}) { pod2usage(-verbose => 2, -exitval => 0) }

# display panels
my %displaypanels;

if ($options{displaypanels}) {
  map {$displaypanels{$_} = 1} split(",",$options{displaypanels})
} else {
  %displaypanels = qw/info 1 text 1 analysis 1/;
}

# infoboxes
my %infoboxes;
if ($displaypanels{info}) {

  if ($options{infoboxes}) {
    my @info = ();
    my %info = ();
    map { map {push @info, $_} split('\s*=\s*',$_) } split('\s*,\s*', $options{infoboxes}) ;
    %info = @info;
    foreach (keys %info) {
      if ($info{$_} =~ /^\d+$/){
        $infoboxes{$_} = $info{$_} 
      } else {
        print STDERR " I skipped the '$_' infobox (not numeric width value).\n"
      }
    }
  }
  else {
    %infoboxes = qw/
    an 7
    dimensions 17
    fonds 7
    main 3
    jour 3
    mois 3
    version 50
    /;
  }

}



# load arguments
my $file_name;

map {
  /^(.*?)((?:#|\s+)(.*?))?$/;
  $file_name = $1;
  $goto = $3 unless $goto
} @ARGV;

# build IDE
my $mw = Tk::KhIDE->new(
  -lexiconfile => $lexiconfile,
  -infopath => $infopath,
  -textpath => $textpath,
  -analysispath => $analysispath,
  
  # TODO mettre la configuration du programme
  # et des colonnes dans le mme rc.
  -rcfile => $rcfile,

  -displaypanels => [keys %displaypanels],
  -infoboxes => {%infoboxes},
  -cfg => "$path"."/cfg",
  -img => "$path"."/png",
);

# set geometry
my $sw = $mw->winfo('screenwidth'); 
my $sh = $mw->winfo('screenheight'); 

#$mw->wm("geometry", $sw . "x" . $sh); 

# set options
#$mw->optionAdd('*Label.font' => 'normal');
#$mw->optionAdd('*Menu.font' => 'normal');
#$mw->optionAdd('*LabEntry.font' => 'normal');
#$mw->optionAdd('*Entry.font' => 'normal');
#$mw->optionAdd('*Button.font' => 'normal');
#$mw->optionAdd('*LabEntry.font' => 'normal');
#$mw->optionAdd('*Checkbutton.font' => 'normal');
#$mw->optionAdd('*NoteBook.relief' => 'flat');
#$mw->optionAdd('*Listbox.selectBorderWidth' => '0');
#$mw->optionAdd('*ComboBox.selectBorderWidth' => '0');
#$mw->optionAdd('*BrowseEntry.selectBorderWidth' => '0');
#$mw->optionAdd('*Entry.selectBorderWidth' => '0');
#$mw->optionAdd('*Button.relief' => 'flat');
#$mw->optionAdd('*Menu.highlightThickness' => '0');
#$mw->optionAdd('*Menu.borderWidth' => '1');
#$mw->optionAdd('*Menu.activeBorderWidth' => '1');
#$mw->optionAdd('*Scrollbar.borderWidth' => '1');
#$mw->optionAdd('*Scrollbar.relief' => 'flat');
#$mw->optionAdd('*highlightThickness' => '0');

my $xmlbind = XML::EasXML->new(file => $mw->cfg . "/mainbindings.xml");

# FIXME
# $mw->loadBindings($mw, "top", $xmlbind->root);
$mw->bind("<Control-s>" => sub { $mw->saveFile; &Tk::break });
$mw->bind("<Control-f>" => sub { $mw->promptId; &Tk::break });
$mw->bind("<Control-o>" => sub { $mw->fileDialog("open"); &Tk::break });

# open file
$mw->loadFile($file_name) if $file_name;

$mw->goto_id($goto) if $goto;

MainLoop;


__END__

=head1 NAME

Kh-edit. Interface d'analyse linguistique de documents XML.

=head1 SYNOPSIS

Kh-edit [options] [fichier xml bien form]

Options:

xpaths:

=over 8

=item * --infopath      Spcifie le chemin xpath des informations gnrales 
  
=item * --textpath      Spcifie le chemin xpath du texte
  
=item * --analysispath  Spcifie le chemin xpath des analyses

=back

=over 8

=item * --goto		Dplace le focus sur l'lment qui a l'id mentionn

=back

interface:

=over 8

=item * --displaypanels Spcifie quelles fentres seront affiches pour modifier le document
  
=item * --infoboxes     Spcifie les botes  afficher pour la modification de la fiche du   
                  document, sous la forme de couples info=taille spars de virgules

=back
                  
informations:

=over 8

=item * --help          imprime ce message et quitte
  
=item * --manual        imprime le manuel et quitte
  
=item * --version       imprime la version et quitte
  
=item * --author        imprime les informations sur l'auteur et quitte
  
=item * --licence       imprime les informations de licence et quite

=back

=head1 DESCRIPTION

Kh-edit est un script perl qui gnre l'interface utile  l'annotation linguistique de documents XML.
La manipulation des documents est assume par le module XML::Khartes.
Il se sert de modules Tk spcifiques pour permettre un accs visuel simple aux annotations. 
Kh-edit ne permet pas l'extraction des donnes analyses.

La configuration de l'interface est assume par Kh-edit lui-mme, mais peut tre faite manuellement.

Le manuel de l'utilisateur est disponible sur L<http://cipl.philo.ulg.ac.be/linguistique>.

=head1 OPTIONS

=over 8

=item * B<--infopath> chemin xpath des informations gnrales 

Les valeurs textuelles des noeuds terminaux correspondant au chemin xpath pass en 
argument seront traites et envoyes dans la fiche d'informations.
Par dfaut, le chemin est 

/*/attribute::*"

c'est--dire tous les attributs du noeud racine. Les informations effectivement affiches 
dans l'interface sont donnes par l'option --infoboxes.

=item * B<--textpath> chemin xpath du texte

Les lments  correspondant au chemin xpath pass en 
argument seront traites et envoyes dans l'onglet de texte.
Par dfaut, le chemin est 

/*/texte[1]"

c'est--dire le premier enfant <texte> du noeud racine.

=item * B<--analysispath> chemin xpath des analyses

Les lments  correspondant au chemin xpath pass en 
argument seront traites et envoyes dans l'onglet d'analyse linguistique hirarchique.
Par dfaut, le chemin est 

//*[name() = 'syntaxe' 
  or name() = 'diplo' 
  or (name() = 'analyse' 
  and ancestor::linguistique)]"

c'est--dire tous les descendants du noeud racine nomms "diplo" ou "syntaxe", ainsi que tous les enfants <analyse> des 
noeuds <linguistique>.

=item * B<--goto> id

Si prsent, aprs le chargement du fichier, dplace le focus de manire  ce qu'il se trouve sur l'id pass en argument.


=item * B<--displaypanels> fentres  afficher

Spcifie les fentres  afficher pour la modification du document, sous la forme de noms spars 
de virgules. Les valeurs possibles sont: I<info>, I<text>, I<analysis>. Par dfaut, elles sont toutes les trois appeles.

I<info>: la bote qui contient la fiche du document 
(informations extralinguistiques);

I<text>: le panneau qui permet l'dition du texte;

I<info>: le paneau qui permet de faire des analyses arborescentes.

=item * B<--infoboxes> informationss  afficher

Spcifie les botes  afficher pour la modification de la fiche du document, sous la forme de couples clef=valeur spars 
de virgules. La valeur correspond  la largeur du champs  afficher (en caractres). Le programme va chercher ces informations 
dans le noeud renseign par --infopath. Par dfaut, les valeurs sont:

an=7,dimensions=17,fonds=7,main=3,jour=3,mois=3

=item * B<--help>		

Imprime les options de la ligne de commande et quitte

=item * B<--manual>

Imprime le manuel et quitte

=item * B<--version>	

Imprime la version et quitte

=item * B<--author>	

Imprime les informations sur l'auteur et quitte

=item * B<--licence>	

Imprime les informations de licence et quite


=back

=head1 CAVEATS

Les bibliothques XML::Khartes ont t initialement conues dans le but de traiter des documents diplomatiques
valides par rapport  une DTD spcifique. Il se peut ds lors que certaines donnes soient a priori incompatibles
avec KhEdit.

=head1 BUGS

Merci de faire part de tout dysfonctionnement  l'auteur.

=head1 VOIR AUSSI

XML::Khartes, Tk::KhIDE.

=head1 AUTEUR

Nicolas Mazziotta, Universit de Lige E<lt>nicolas.mazziotta@ulg.ac.beE<gt>

=head1 LICENCE

Kh-edit a t dvelopp dans le cadre du projet Kharts, sous la direction de 
Marie-Guy Boutier,  l'Universit de Lige.

Copyright (c) 2005-2006 Universit de Lige, Nicolas Mazziotta
This program is free software; you can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

