# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: KhAnalysis.pm 154 2006-01-25 08:23:28Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


#######################################################
package Tk::KhAnalysis;
#######################################################
# A widget that contains the analysis and ways to alter it

use strict;		
use base qw/Tk::Frame/;
use Tk::KhTree;
use Tk::KhTreeConfig;
use Tk::KhTreeSearch;
use Tk::HidePane;
use Tk::Dialog;

Construct Tk::Widget "KhAnalysis";

sub ClassInit{
	my ($class,$mw) = @_;
	$class->SUPER::ClassInit($mw);
}

sub Populate {
	my ($w, $args) = @_;
	
	$w->ConfigSpecs(
		-xmlconfig => ["PASSIVE", "xmlconfig", "XmlConfig", undef],
	);
	
	$w->{_xmlconfig} = $args->{-xmlconfig};
	
	delete $args->{-xmlconfig};
	
	$w->SUPER::Populate($args);

	my %options = %$args;
	
		
	# Affichage
	my %defaultPack = qw(
		-expand 1
		-fill both
		-side left
	);
	my %defaultBorder = qw(
		-relief groove
		-borderwidth 2
	);
	
	my $toolbar = $w->Component("Frame", "toolBox")->pack(-side => "left", -fill => "y", -expand => 0);
	my $rightpane = $w->Frame()->pack(%defaultPack, qw/-side right/);

	my $infobar = $rightpane->HidePane()->pack(-fill => "x", -side => "top");
	$infobar->Subwidget("frame")->configure(-pady => 2);
	$infobar->toggle;
	
	my $nb = $rightpane->KhNoteBook(%options)->pack(%defaultPack, -side => 'top');

	$w->Advertise(nb => $nb);
	$w->Advertise(info => $infobar);

	$w->Delegates(	
		'DEFAULT' => $nb, 
		'addAnalysis' => $w,
		'Subwidget' => $w,
		#'_buttons' => $toolbar,
	);

	
	$w->_buttons
	
}



sub loadInfo {
	my ($w) = @_;
	my $tree = $w->_getRaisedTree;
	return 0 unless $tree;
	my $i = $w->Subwidget("info");

	$i->empty;

	# name
	my $ne = $i->LabEntry(-label => "nom", -labelPack => [qw/-side left/], -textvariable =>\my $name)->pack(qw/-side left/);
	
	# type
	my @choices = ();
	map {
		my $value = $_->value;
		push @choices, $value
	} $tree->xmlconfig->root->findnodes("/config/widget[\@type = 'tree']/\@kind");
	$i->Label(-text => "type")->pack(qw/-side left/);
	my $te;
	my $type;
	$te = $i->MatchEntry(
		-textvariable => \$type,
		-choices => \@choices,
		-entercmd => sub { 
			# update
			my $tree = $w->_getRaisedTree;
			# FIXME: why is there an error??
			print STDERR "Please ignore this message!\n";
			return 0 if $type eq $tree->type;
			$tree = $tree->type($type);
			$w->_readvertise($tree);
			my $nb = $w->Subwidget("nb");
			$nb->pageconfigure($nb->raised, -label => $type) unless $tree->name;
		}
	)->pack(qw/-side left/);
	
	# bindings
	my $re = $i->LabEntry(-label => "responsable", -labelPack => [qw/-side left/], -textvariable => \my $resp)->pack(qw/-side left/);
	
	my %widgets = ("name" => $ne, resp => $re);
	foreach my $att (keys %widgets) {
		# FIXME: why is there an error??
		$widgets{$att}->bind("<Return>" => sub { $w->_changeInfo( $widgets{$att}, $att) } );
	}
	
	my $xmldata = $tree->{Configure}{-xmldata};
	$name = $xmldata->att("name");
	$type = $xmldata->att("type");
	$resp = $xmldata->att("resp");
}

sub _changeInfo {
	my ($w, $widget, $att) = @_;
	my $nb = $w->Subwidget("nb");
	my $tree = $w->_getRaisedTree;
	my $value = $widget->get();
	$tree->$att($value);
	if ($att eq "name") { $nb->pageconfigure($nb->raised, -label => $value);} 
}




sub eradicateAnalysis {
	my $w = shift;
	my $r = $w->raised;
	my $y = $w->toplevel->prompt(
		-text => "Vous vous apprtez dtruire irrmdiablement l'analyse visible sur cet onglet. tes-vous certain de vouloir continuer?",
	);
	if ($r and $y eq "oui") {
		my $k = $w->toplevel->{_xml};
		my $t = $w->_getRaisedTree()->xml;
		$k->delete($t);
		$w->delete($r) 
	};
}

sub duplicateAnalysis {
	my ($w) = @_;

	my $tree = $w->_getRaisedTree;
	return $w->addAnalysis() unless $tree;

	my $refxml = $tree->xml;
	my $xmldata = $refxml->cloneNode(1);
	$refxml->parentNode->appendChild($xmldata);
	
	$w->addAnalysis("", $xmldata);
	
}


sub addAnalysis {
	my ($w, $xmlconfig, $xmldata) = @_;

	unless ($xmlconfig) {
		$xmlconfig = $w->{_xmlconfig} 
	}
	
	unless ($xmldata) {
		# FIXME: permettre de mettre ici le chemin des units  analyser
		$xmldata = $w->toplevel->{_xml}->_build_syntax("", "nouveau");
	}
	
	my $type = $xmldata->att("type");
	$type = $xmldata->nodeName unless $type;
	my $name = $xmldata->att("name");
	$name = $type unless $name;

	my %options = Tk::KhTree::loadCols(
		$type, 
		$xmlconfig->root
	); # see this method in KhTree for explanation

	$options{-xmlconfig} = $xmlconfig;
	$options{-xmldata} = $xmldata;

	my %pages = ();
	map { $pages{$_} = 1 } $w->pages;

	my $n = 0;
	my $pagename = $type.$n;
	while ($pages{$pagename}) {
		$pagename = $type.++$n
	}
		
	my $analysistab = $w->tabAdd(
		-name => $pagename, 
		-label => $name, 
		-widget => "KhTree",
		-options => {
			%options,
			-type => $type,
			-name => $name,
		},
		-pack => [qw/-expand 1 -fill both/]
	);

	$w->pageconfigure(
		$pagename, 
		-raisecmd => sub  {
			my $nb = $w->Subwidget("nb");
			my $tree = $nb->Subwidget($pagename)->Subwidget("scrolled");
			if ($tree->{_configchange}) {
				my $t = $tree->loadConfig ;
				# re-advertise
				# TODO: remettre a avec celui de KhTreeConfig
				$nb->Advertise($pagename => $t);
			}
			$w->loadInfo;
		}
		
	);

	# FIXME: why pass type explicitely?
	$analysistab->loadImages($type, $xmlconfig->root);
	$analysistab->loadStructures($type, $xmlconfig->root);
	$w->toplevel->loadBindings($analysistab, $type, $xmlconfig->root);
}

sub configuration {

	my $w = shift;
	
	my $tree = $w->_getRaisedTree;
	my $type = $tree->type;

	my $xmlconfig = "";

	map { $xmlconfig = $_ } $tree->xmlconfig->root->findnodes("/config/widget[\@type = 'tree' and \@kind = '$type'][1]");
	
	unless ($xmlconfig) {
		my $create = $tree->toplevel->prompt(
			-text => "Aucune configuration n'est associe  ce type d'arbre. Dsirez-vous crer une section spcifique? Sinon, le panneau de configuration va fermer."
		);
		return 0 if $create ne "oui";

		my %kinds = ();
		map {$kinds{$_->att("kind")} = $_} $tree->xmlconfig->root->findnodes("/config/widget[\@type = 'tree']");
		
		my $clone =  $tree->toplevel->prompt(
			-text => "Dsirez-vous cloner une configuration existante?"
		) if %kinds;
		if ($clone eq "oui") {
			my $key = $w->toplevel->prompt(
				-text => "Quelle configuration dsirez-vous cloner?",
				-choices => \%kinds,
			);
			unless ($key eq "non") {
				my $clonenode = $kinds{$key}->cloneNode(1);
				$clonenode->att(kind => $type);
				$xmlconfig = $tree->xmlconfig->root->appendChild($clonenode);
			}
		} 
		unless ($xmlconfig) {
			$xmlconfig = $tree->xmlconfig->root->addNewChild("","widget");
			$xmlconfig->att(type => "tree");
			$xmlconfig->att(kind => $type);
			$xmlconfig->addNewChild("","images")->att(ext => ".png");
			$xmlconfig->addNewChild("","structures");
			$xmlconfig->addNewChild("","bindings");
			$xmlconfig->addNewChild("","columns");
		}
	}

	my $r = $w->raised;
	my $name = "" ;
	$name = "" . $w->pagecget($r, "-label") . "" if $r;
	my $mw = MainWindow->new(-title => "Configuration de l'analyse $name");

	my $sw = $mw->winfo('screenwidth'); 
	my $sh = $mw->winfo('screenheight'); 

	$mw->wm("geometry", $sw . "x" . $sh); 



	$mw->optionAdd('*Label.font' => 'normal');
	$mw->optionAdd('*Menu.font' => 'normal');
	$mw->optionAdd('*Entry.font' => 'normal');
	$mw->optionAdd('*Button.font' => 'normal');
	$mw->optionAdd('*LabEntry.font' => 'normal');
	$mw->optionAdd('*Checkbutton.font' => 'normal');
	$mw->optionAdd('*NoteBook.relief' => 'flat');
	$mw->optionAdd('*Listbox.selectBorderWidth' => '0');
	$mw->optionAdd('*ComboBox.selectBorderWidth' => '0');
	$mw->optionAdd('*BrowseEntry.selectBorderWidth' => '0');
	$mw->optionAdd('*Entry.selectBorderWidth' => '0');
	$mw->optionAdd('*Button.relief' => 'flat');
	$mw->optionAdd('*Menu.highlightThickness' => '0');
	$mw->optionAdd('*Menu.borderWidth' => '1');
	$mw->optionAdd('*Menu.activeBorderWidth' => '1');
	$mw->optionAdd('*Scrollbar.borderWidth' => '1');
	$mw->optionAdd('*Scrollbar.relief' => 'flat');
	$mw->optionAdd('*highlightThickness' => '0');
	

	my %images = qw/
		right	forward
		left 	back
		down 	down
		up	up
		duplicate	kontact_contacts
		cut	editcut
		copy	editcopy
		paste	editpaste
		configure	edit
		delete	editdelete
		tabadd	tab_new
		tabremove	tab_remove
		tabduplicate	tab_duplicate
		reload	reload
		wrap	package
		
	/;
	foreach (keys %images) {
		my $file = $w->toplevel->img."/".$images{$_}.".png";
		$mw->Photo($_, file => $file);
	}
	
	
	$mw->KhTreeConfig(
		-widget => $tree, 
		-widgetcontainer => $w,
		-xmlconfig => $xmlconfig,
	)->pack(-expand => 1, -fill => "both");
	
}

sub find_dialog {
  my ($w) = @_;
  my $tree = $w->_getRaisedTree;
  my $mw = MainWindow->new(-title => "Rechercher dans l'arbre");
  $mw->KhTreeSearch(-widget => $tree)->pack;
}

sub _readvertise {
  # everytime o widget is destroyed and reloaded, it has to be readvertised
  my ($w, $t) = @_;
  my $nb = $w->Subwidget("nb");
  $nb->Advertise($nb->raised => $t);
}

sub _getRaisedTree {
	my ($w) = @_;
	my $nb = $w->Subwidget("nb");
	my $r = $nb->raised();
	my $tree = $nb->Subwidget($r)->Subwidget('scrolled') if $r;
	return $tree if $tree;
}

sub _buttons {
	my $w = shift;
	
	
	my $toolbar = $w->Subwidget("toolBox");
	
	my %defaultBorder = qw(
		-relief groove
		-borderwidth 2
	);
	my $movesFrame = $toolbar->Frame(%defaultBorder)->pack;
	
	$movesFrame->Button(-image => "up", -command => 
		sub  { $w->_getRaisedTree->move("up") }
	)->pack;
	$movesFrame->Button(-image => "left", -command => 
		sub  { $w->_getRaisedTree->move("left") }
	)->pack;
	$movesFrame->Button(-image => "down", -command => 
		sub  { $w->_getRaisedTree->move("down") }
	)->pack;
	$movesFrame->Button(-image => "delete", -command => 
		sub  { $w->_getRaisedTree->safe_delete }
	)->pack;
	
	$movesFrame->Button(-image => "copy", -command => 
		sub  { $w->_getRaisedTree->copy }
	)->pack;
	$movesFrame->Button(-image => "paste", -command => 
		sub  { $w->_getRaisedTree->paste }
	)->pack;
       
	$movesFrame->Button(-image => "find", -command => ['Tk::KhAnalysis::find_dialog', $w])->pack;

	my $analysisManipFrame = $toolbar->Frame(%defaultBorder)->pack;
	$analysisManipFrame->Button(-image => "configure", -command => ['Tk::KhAnalysis::configuration', $w])->pack;
	$analysisManipFrame->Button(-image => "tabadd", -command => ['Tk::KhAnalysis::addAnalysis', $w])->pack;
	$analysisManipFrame->Button(-image => "tabduplicate", -command => ['Tk::KhAnalysis::duplicateAnalysis', $w])->pack;
	$analysisManipFrame->Button(-image => "tabremove", -command => ['Tk::KhAnalysis::eradicateAnalysis', $w])->pack;
}


1;


