# Copyright (c) 2005-2006 Universit de Lige, Nicolas Mazziotta
# $Id: KhIDE.pm 196 2006-03-19 09:33:02Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.



#######################################################
package Tk::KhIDE;
#######################################################
# A base container

use Fcntl ':flock';
use File::Temp qw/tempfile/;
use File::Basename;


use base qw/Tk::MainWindow/;
use Tk::PNG;

use Tk::KhInfo;
use Tk::KhText;
use Tk::KhAnalysis;
use Tk::KhTreeConfig;
use Tk::KhNoteBook;

Construct Tk::Widget "Tk::KhIDE";

sub ClassInit{
	my ($class,$mw) = @_;
	$class->SUPER::ClassInit($mw);
}

sub Populate {

  my ($w, $args) = @_;

  unless ( $args->{"-rcfile"} and -f $args->{"-rcfile"} ) {
    $args->{"-rcfile"} = $args->{"-cfg"} . "/.kheditrc";
  }
  
=start comment

 FIXME

  open( RC, ">", $args->{"-rcfile"} ) or return 0;
  flock(RC, LOCK_EX) or return 0;

  $w->{_rc} = {};
  
  while (<RC>) {
    next if /^(?:\s*#\s*)|(?:\s*)$/;
    /^(.*?):(.*?)(#.*)?$/;
    $w->{_rc}{$1} = $2
  }
  
=end comment 
=cut
  
  $w->SUPER::Populate($args);
  $w->ConfigSpecs(-lexiconfile => ["PASSIVE", "lexiconFile", "LexiconFile"]);
  $w->ConfigSpecs(-infopath => ["PASSIVE", "infoPath", "InfoPath"]);
  $w->ConfigSpecs(-textpath => ["PASSIVE", "textPath", "TextPath"]);
  $w->ConfigSpecs(-analysispath => ["PASSIVE", "analysisPath", "AnalysisPath"]);
  $w->ConfigSpecs(-rcfile => ["PASSIVE", "rcFile", "RcFile"]);
  $w->ConfigSpecs(-infoboxes => ["PASSIVE", "infoboxes", "infoBoxes"]);
  $w->ConfigSpecs(-displaypanels => ["PASSIVE", "displaypanels", "DisplayPanels", [qw/info text analysis/]]);
  $w->ConfigSpecs(-cfg => ["PASSIVE", "cfg", "Cfg"]);
  $w->ConfigSpecs(-img => ["PASSIVE", "img", "Img"]);

  #######################################################
  #		TODO TODO TODO
  # quelque chose de plus propre pour indexer les formes
  # (voir KhTree::Branch)
  #######################################################

  


  # Affichage
  my %defaultPack = qw(
    -expand 1
    -fill both
    -side left
  );
  my %defaultBorder = qw(
    -relief groove
    -borderwidth 2
  );

  my $starttime = (time)[0];
  print STDERR "Setting environnment.\n";
  print STDERR " Setting default look-and-feel.\n";

  $w->loadCommonImages($args->{-img});

  $w->fontCreate("normal", -size => "10", -family => "Helvetica");
  $w->fontCreate("italic", -size => "10", -slant => "italic", -family => "Helvetica");
  $w->fontCreate("small", -size => "8", -family => "Helvetica");
  $w->fontCreate("bold", -weight => "bold", -size=> 10, -family => "Helvetica");

  # dialog

  $w->{_dialog} = $w->toplevel->DialogBox(-buttons => [qw/oui non/]);

  # menu
  my @menuitems = (
    [
    Cascade => "~Fichier", -menuitems => [
    [Button => "~Ouvrir",     -command => sub {my $f = $w->fileDialog("open"); $w->loadFile($f) if -f $f}],
    [Button => "Enregi~strer", -command => sub {$w->saveFile}],
    [Button => "Enregistrer so~us", 
    -command => sub {
      my $f = $w->fileDialog("save"); 
      $w->saveFile($f); 
      $w->{_file} = $f if $f;
    }
    ],			
    [Separator => ""],
    [Button => "~Quitter", -command => sub {$w->destroy} ],
    ]
    ],
    [
    Cascade => "~dition", -menuitems => [
    [Button => "~Trouver un id", -command => sub  {$w->promptId}]
    ]
    ]
  );
  my $menubar = $w->Menu(-type=> "normal", -menuitems => \@menuitems, -borderwidth=> 0);

  $w->configure(-menu => $menubar);

  # Barre d'informations gnrales
  print STDERR " Building main information box.\n";

  my $infobar = $w->HidePane(-pady => 2)
  ->pack(-expand => 0, -side => "top", -fill => "both")
  ->KhInfo(-boxes => $args->{'-infoboxes'})
  ->pack(%defaultPack);

  # Bote  onglets
  print STDERR " Building notebook.\n";

  my $notebook = $w->KhNoteBook(
  )->pack(-fill => "both", -expand => 1);

  $notebook->tabAdd(
      -name => "textbox",
    -label => "Texte",
    -widget => "KhText",
    -pack => [qw/-expand 1 -fill both/]
    );
  $notebook->tabAdd(
      -name => "analysisbox",
    -label => "Analyses",
    -widget => "KhNoteBook",
    -pack => [qw/-expand 1 -fill both/]
    );

  $w->Advertise(info => $infobar);
  $w->Advertise(notebook => $notebook);      

  my $endtime = (time)[0];

  print STDERR "It took me " , $endtime - $starttime , " seconds to set environnment.\n\n";


}

sub img {
  my ($w) = @_;
  return $w->{Configure}{-img};
}


sub goto_id {
  my ($w, $id) = @_;

  my $focused = $w->Subwidget("notebook")->goto_id($id);
  

  return $focused if $focused;
  
  my $quit = $w->prompt(-text => "Je n'ai pas trouv l'lment qui portait l'identifiant '$id'. Soit cet lment n'existe pas, soit il n'est pas affich dans un des onglets, soit il ne s'agit pas d'une unit linguistiquement pertinente. \n\nVoulez-vous quitter le programme?");
  $w->DESTROY if $quit ne "non";  
  return 0;
}

sub cfg {
	my ($w) = @_;
	return $w->{Configure}{-cfg};
}

sub promptId {
  my ($w) = @_;
  my $id = $w->prompt(-text => "Entrez l'id que vous dsirez chercher suivi de <Retour>. 
  
  Appuyez sur 'non' pour quitter.", -entry => "non");
  if ($id and $id ne 'non') {
    $w->goto_id($id);
  }
  
}

sub prompt {
	my ($w, %options) = @_;

	my $dialog = $w->{_dialog};
	
	my $b = $dialog->Subwidget("B_oui");
	my $b_non = $dialog->Subwidget("B_non");
	
	$b->configure( -text => 'oui' );
	$b_non->configure( -text => 'non' );
	
	
	# tidy up
	map { $_->destroy } $dialog->Subwidget("top")->children;
	
	if ($options{-text}) {
		my $text = $dialog->ROText(
			-height => 15, 
			-width => 40,
			-wrap => "word"
			
		)->pack;
		$text->insert("0.0", $options{-text});
	}
        if ($options{-choices}) {
          $b->configure(-command => sub {
            $dialog->{selected_button} = $b->cget('-text')
          });

          my $be;
          my $value;
          $be = $dialog->BrowseEntry(
            -state => "readonly", 
            -choices => [sort keys %{$options{-choices}}],
            -variable => \$value,
            -browsecmd => sub { $b->configure( -text => $value ); }
          )->pack;
          $value = $be->get(1);
          $b->configure(-text => $value);
	} elsif ($options{-entry}) {
	  $b->configure(-command => sub {
	    $dialog->{selected_button} = $b->cget('-text')
	  });

	  my $e;
	  my $value = "non";
	  $e = $dialog->Entry(
	    -textvariable => \$value,
	  )->pack;
	  $b->configure(-textvariable => \$value);
	  $e->selectionRange(0,"e");
	}
	$b->configure(-textvariable => undef);
	return $w->{_dialog}->Show;
}


sub fileDialog {
  my ($w, $mode) = @_;
  my $fileName;
  my $fileTypes = [
  ["Fichiers kml", [".kml"]],
  ["Fichiers xml", [".xml"]],
  ];

  if ($mode eq "open") {
    $fileName = $w->getOpenFile(-filetypes => $fileTypes, -initialdir => $w->{_rc}{lastdir});
  } elsif ($mode eq "save") {
    $fileName = $w->getSaveFile(-filetypes => $fileTypes, -initialdir => $w->{_rc}{lastdir});
  } else {
    $fileName = 0;
  }
  if ($fileName) {
    return $fileName;
  }
}

sub khartes {
  my $w = shift;
  return $w->{_xml};
}

sub loadFile {
  my ($w, $file) = @_;
  
  my $fh = "";
  
  open($fh, "< ", $file) or return 0;
  flock($fh, LOCK_EX) or return 0;
  
  close($w->{_FH}) if $w->{_FH};

  my $starttime = (time)[0];
  print STDERR "Building document DOM tree";

  # TODO: more generic
  my $corpus = XML::Khartes::Corpus->new(files => [$file], lexicon_file => $w->cget("-lexiconfile"));
  
  my $kh = "";

  my %texts = $corpus->texts;

  foreach (values %texts) { $kh = $_; last };

  my $endtime = (time)[0];
  print STDERR " [",$endtime - $starttime," seconds].\n";
  print STDERR "Loading data in widgets.\n";

  $w->Subwidget("notebook")->clear;

  $w->{_xml} = $kh;
  $w->{_file} = $file;
  $w->{_FH} = $fh;

  PANELS: foreach (@{$w->cget("-displaypanels")}) {
		print;
    if (/^\s*info\s*$/) {
      $w->loadInfo($kh);
      next
    }
    elsif (/^\s*text\s*$/) {
      $w->loadText($kh);
      next
    }
    elsif (/^\s*analysis\s*$/) {
      $w->loadAnalysis($kh);
      next
    }
    else { 
      print STDERR "Skipping unknown panel type $_\n";
      next 
    }
  }

  return $kh;
}

sub saveFile {
	my ($w, $file) = @_;
	my $encoding = $w->{_xml}->xml->encoding;
	$file = $w->{_file} unless $file;
	print STDERR "\nSaving $file... ";
	#my  $text =  "<?xml version='1.0' encoding='$encoding'?>\n";
	$text .= $w->{_xml}->toFile($file, 1);
	#open OUT, ">", $file;
	#print OUT $text;
	#close OUT;
	print STDERR "Done! [". localtime() ."]";
	return $text;
}

sub loadBindings {
	my ($top, $w, $type, $xml) = @_;
	return 0 unless $xml;
	foreach my $binding ($xml->findnodes("/config/widget[\@kind = 'default' or \@kind = '$type']/bindings/binding")) {
		my $key = $binding->att("key");
		$key =~ s/<(.*)>/$1/;
		my $action = $binding->att("action");
		my @arguments = ();
		foreach my $argument ($binding->findnodes("argument")) {
			my $type = $argument->att("type");
			if ($type eq "hash") {
				my %hash = ();
				foreach ($argument->findnodes("pair")) {
					my $key = $_->att("key");
					my $value = $_->att("value");
					$hash{$key} = $value;
				}
				push @arguments, {%hash};
			} 
			elsif ($type eq "array") {
				my @array = ();
				foreach ($argument->findnodes("item")) {
					my $value = $_->att("value");
					push @array, $value
				}
				push @arguments, [@array];
			}
			else {
				push @arguments, $argument->att("value");
			}
		}
		$w->bind("<$key>", [$action, @arguments]) if $key;
	}
	
}

sub loadCommonImages {
	my ($w, $img) = @_;
	my %images = qw/
		right	forward
		left 	back
		down 	down
		up	up
		duplicate	kontact_contacts
		cut	editcut
		copy	editcopy
		paste	editpaste
		configure	edit
		delete	editdelete
		tabadd	tab_new
		tabremove	tab_remove
		tabduplicate	tab_duplicate
		reload	reload
		wrap	package
		undo	undo
		redo	redo
		addleft	undo
		addright	redo
		exec	exec
		ok	ok
		no	no
                find    find
	/;
	foreach (keys %images) {
		my $file = $img ."/". $images{$_}.".gif";
		$w->Photo($_, file => $file);
	}
}

sub loadInfo {
	my ($w, $kh) = @_;
	
	my $starttime = (time)[0];
	print STDERR " Loading data in information box";
	
	my $infoBox = $w->Subwidget("info");
	
	my $path = $w->cget("-infopath");
	
	foreach my $att ($kh->xml->findnodes($path)) {
		# TODO: ne pas limiter aux attributs
		$infoBox->loadInfo($att->nodeName, $att->value);
	}
	
	my $endtime = (time)[0];
	print STDERR " [",$endtime - $starttime," seconds].\n";
	
	return $infoBox;
}

sub loadText {
	my ($w, $kh) = @_;
	
	my $nbBox = $w->Subwidget("notebook");
	$nbBox->delete("textbox");
	
	my $starttime = (time)[0];
	print STDERR " Loading data in text box";

	my $textBox = $nbBox->tabAdd(
		-name => "textbox",
		-label => "Texte",
		-widget => "KhText",
		-pack => [qw/-expand 1 -fill both/]
	);
	
	my $path = $w->cget("-textpath");
	map {$textBox->loadXML($_)} $kh->xml->findnodes($path);
	
	my $endtime = (time)[0];
	print STDERR " [",$endtime - $starttime," seconds].\n";
	
	return $textBox;
}

sub loadAnalysis {
	my ($w, $kh) = @_;
	my $starttime = (time)[0];
	print STDERR " Loading data in analysis box";

	my $nbBox = $w->Subwidget("notebook");
	$nbBox->delete("analysisbox") ;
	
	my $xmlconfig = XML::EasXML->new(file => $w->cfg . "/treesconfig.xml");
	
	my $analysisBox = $nbBox->tabAdd(
		-name => "analysisbox",
		-label => "Analyses arborescentes",
		-widget => "KhAnalysis",
		-options => {
			-xmlconfig => $xmlconfig
		},
		-pack => [qw/-expand 1 -fill both/]
	);
	my $path = $w->cget("-analysispath");
	
	
	
	$w->Subwidget("notebook")->pageconfigure(
		"analysisbox", 
		-createcmd => sub {
			map {
				$analysisBox->addAnalysis($xmlconfig, $_)
			} $kh->xml->findnodes($path);

		}
	);
	
	my $endtime = (time)[0];
	print STDERR " [",$endtime - $starttime," seconds].\n";
}

sub rcwrite {
  my ($w, $key, $val) = @_;
  $w->{_rc}{$key} = $val;
  open(RC, ">" . $w->cget(-rcfile)) or return 0;
  my $rc = $w->{_rc};
  foreach (keys %rc) {
    print RC $_ . ": " . $rc{$_} . "\n";
  }
  close(RC);
  return 1;
}


1;

