# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: KhText.pm 154 2006-01-25 08:23:28Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


#######################################################
package Tk::KhText;
#######################################################
# A widget that contains the text and ways to alter it

use strict;		
use lib "$ENV{HOME}/lib/perl";
use base qw/Tk::Frame/;
use Tk::ROText;
use Tk::KhWord;

Construct Tk::Widget "KhText";

sub ClassInit{
	my ($class,$mw) = @_;

	$mw->eventAdd('<<Focus>>' => '<Button-1>');
	$mw->bind($class, '<Button-2>' => sub  { Tk::break; });

	$class->SUPER::ClassInit($mw);
}

sub Populate {
	my ($w, $args) = @_;
	$w->SUPER::Populate($args);

	# Affichage
	my %defaultPack = qw(
		-expand 1
		-fill both
		-side left
	);
	my %defaultBorder = qw(
		-relief groove
		-borderwidth 2
	);
	$w->{line_counter} = 0;
	$w->{xml} = "";
	
	my $text = $w->Scrolled("ROText", -scrollbars => "ose", -wrap => "word")->pack(%defaultPack);
	$text = $text->Subwidget("scrolled");

	$w->Advertise(text => $text);
	
	my $editBox = $w->Component(
		"KhWord", 
		"editBox", 
		-label => "dition", 
		-labelside => "acrosstop",
	)->pack(-fill => "x");
	$editBox->{current_id} = "";

	# fonts
	$text->tagConfigure("_bold", -font => "bold");
	$text->tagConfigure("_focused", -background => "gold", -font => "bold");
	$text->tagConfigure("_empty", -foreground => "purple", -font => "bold");
		
	$w->Delegates('DEFAULT'=>$text, 'loadXML' => $w, 'goto_id' => $w); 
}

sub loadXML {
	my ($w, $xml) = @_;
	$w->{xml} = $xml;
	# $xml is an XML::LibXML::Node object;
	$w->_traverseXML($xml);
}

sub goto_id {
  my ($w) = @_;
  # TODO
  print STDERR "\ngoto is not implemented for KhText widget.";
  return 0;
}

#######################################################
#		 Traverse XML 
#######################################################
#TODO: SAX

sub _traverseXML {
	# TODO: a generic function
	my ($w, $node) = @_;
	my $name = $node->nodeName;
	my $type = $node->nodeType;
	my $keepon = 0;
	if ($type == 3) {
		$w->_text($node);
	} else {
		my $sub = "_node_$name";
		eval { $keepon = $w->$sub($node); };
	}
	if ($keepon) {
		foreach ($node->childNodes) { 
			$w->_traverseXML($_) 
		} ;
	}
}

sub _node_recto { 
	my ($w, $node) = @_;
	$w->{line_counter} = 0;
	return 1 
} 

sub _node_texte { return 1 } 

*_node_separateur = *_node_w;

sub _text {
	my ($w, $node) = @_;
	my @id;
	my $parent = $node->parentNode;
	eval {
		while ($parent->nodeName ne "recto") {
			push @id, $parent->id;
			$parent = $parent->parentNode;
		}
	};
	my $text = $node->data;
	$text =~ s/\s//g;
	$text =~ s/[\$]/s/g;
	$text =~ s//S/g;
	$text =~ tr/*/i/;
	$text =~ s/[]//g;
	$w->insert("e", $text, [@id]);
}

sub _node_l {
	my ($w, $node) = @_;
	my $id = $node->id;
	$id = $w->toplevel->{_xml}->create_id($node) unless $id;
	$w->tagConfigure($id, -font => "bold");
	my $l = ++$w->{line_counter};
	my $newline = "";
	$newline = "\n" if ! $node->findnodes("ancestor::w[1]") and $l > 1;
	$w->tagConfigure($id, -foreground => "blue", -font=> "bold");
	$w->insert("e", "$newline", [], "[$l]", $id);
	$w->tagBind($id, "<<Focus>>" => [\&update, $id]);
	return 1;
}

sub _node_c {
	my ($w, $node) = @_;
	my $id = $node->id;	
	$id = $w->toplevel->{_xml}->create_id($node) unless $id;
	$w->tagConfigure($id, -font => "bold");
	return 1;
}

sub _node_w {
	my ($w, $node) = @_;
	my $id = $node->id;	
	$w->tagConfigure($id, -font => "normal");
	$w->tagBind($id, "<<Focus>>" => [\&update, $id]);
	if ($node->textContent) {
		$w->insert("e", " ") if $node->nodeName ne "separateur" and $node->prev->nodeName ne "separateur";
	} else {
		$w->insert("e", " ([]) ", [$id, "_empty"]);
	}
	return 1;
}


sub _node_ponct {
	my ($w, $node) = @_;
	my $id = $node->id;	
	$w->tagConfigure($id, -foreground => "red", -font => "normal");
	$w->insert("e", " ") unless $node->findnodes("./ancestor::w[1]");
	$w->tagBind($id, "<<Focus>>" => [\&update, $id]);
	return 1;
	
}

sub update {
	my ($w, $id) = @_;
	$w->tagRemove("_focused", "0.0", "end");
	$w->tagAdd("_focused", $w->tagRanges($id));
	
	my $editbox = $w->parent->parent->Subwidget("editBox");
	$editbox->update($id);
	
}

1;

