# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: KhTree.pm 196 2006-03-19 09:33:02Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.


#######################################################
package Tk::KhTree;
#######################################################

use strict;		
use XML::EasXML;
use base qw/Tk::Tree/;
use Tk::ItemStyle;
use Tk::HidePane;
use Storable;
use Scalar::Util;

Construct Tk::Widget "KhTree";

# TODO: empcher _cut de se servir du clipboard si ce n'est pas lui qui est appel par l'utilisateur.
# (en d'autres termes, il faut un clipboard alternatif pour les oprations internes).

sub ClassInit{
  my ($class,$mw) = @_;

  $class->SUPER::ClassInit($mw);

}

sub Populate {
  my ($w, $args) = @_;
  $args->{-selectmode} = "extended";
  $w->ConfigSpecs(
    -colconf => ["METHOD", "colconf", "ColConf", []],
    -structurenames => ["METHOD", "structurenames", "StructureNames", []],
    -xmlconfig => ["PASSIVE", "xmlconfig", "XmlConfig", undef],
    -xmldata => ["PASSIVE", "xmldata", "XmlData", undef],
    -type => ["METHOD", "type", "Type", undef],
    -name => ["METHOD", "name", "Name", undef],
    -notes => ["PASSIVE", "notes", "Notes", 1],
  );

  

  $args->{-header} = 1;

  my $xmldata = $args->{-xmldata};
  my $type = $xmldata->att("type");
  my $name = $xmldata->att("name");
  my $nodeName = $xmldata->nodeName;
  $xmldata->att("type" => $nodeName) unless $type;
  $xmldata->att("name" => $type) unless $name;

  $w->{_style} = {};
  $w->{-widget} = {};
  $w->{_cols} = {};
  $w->{_images} = {};
  $w->{_clipboard} = "";
  $w->{_cut} = 0;
  $w->{_undo_queue} = [];

  $w->{_configchange} = 1;
  $w->{_iddb} = {}; # base de donnes qui relie les ids xml des noeuds de l'arbre aux lments de l'affichage

  $w->SUPER::Populate($args);

  # redefine bindings
  $w->bind('<ButtonPress-1>',[ 'Button1' ] );
  $w->bind('<Shift-ButtonPress-1>',[ 'ShiftButton1' ] );
  $w->bind('<Control-ButtonRelease-1>','Control_ButtonRelease_1');
  $w->bind('<ButtonRelease-1>','ButtonRelease_1');
  $w->bind('<Double-ButtonRelease-1>','NoOp');
  $w->bind('<B1-Motion>',[ 'Button1Motion' ] );
  $w->bind('<B1-Leave>',[ 'AutoScan' ] );

  $w->bind('<Double-ButtonPress-1>',['Double1']);

  $w->bind('<Control-B1-Motion>','Control_B1_Motion');
  $w->bind('<Control-ButtonPress-1>',['CtrlButton1']);
  $w->bind('<Control-Double-ButtonPress-1>',['CtrlButton1']);

  $w->bind('<B1-Enter>','B1_Enter');

  $w->bind('<Up>',['UpDown', 'prev']);
  $w->bind('<Down>',['UpDown', 'next']);

  $w->bind('<Shift-Up>',['ShiftUpDown', 'prev']);
  $w->bind('<Shift-Down>',['ShiftUpDown', 'next']);

  #$w->bind('<Left>', ['LeftRight', 'left']);
  #$w->bind('<Right>',['LeftRight', 'right']);

  $w->PriorNextBind($w);
  $w->MouseWheelBind($w);

  $w->bind('<Return>', ['KeyboardActivate']);
  $w->bind('<space>',  ['KeyboardBrowse']);
  $w->bind('<Home>',   ['KeyboardHome']);
  $w->bind('<End>',    ['KeyboardEnd']);


  $w->YMouseWheelBind($w);
  $w->XMouseWheelBind($w);

  # limit bidings to the ones defined explicitely for this widget
  $w->bindtags([$w->toplevel, $w]);

}

sub anchorSet {
  my ($w, @args) = @_;
  $w->SUPER::anchorSet(@args);
  my $id = $w->info("data", $w->info("anchor"));
  my $kh = $w->toplevel->khartes;
  my $ref = $kh->target_id($id);
  return 0 unless $ref;
  my @lemma_ids = $kh->lemma_ids($ref);
  my $info = "";
  map { 
    my $lexitem = $kh->lexicon->find($_);
     $info .= $lexitem->entry . " " . $lexitem->cat . " " 
  } @lemma_ids;
  print "\nlemmatisation: $info" if $info;  
}

sub type {
  # return the updated *tree* (important to readvertise)
  my ($w, $type) = @_;
  my $oldtype = $w->{Configure}{-type};
  if ($type) {
    return $type if $oldtype eq $type;
    $w->{Configure}{-type} = $type;
    $w->{Configure}{-xmldata}->att("type", $type);
    return $w->loadConfig;
  }
  return $oldtype;
}

sub resp {
  my ($w, $resp) = @_;
  my $oldresp = $w->{Configure}{-resp};
  if ($resp) {
    return $resp if $oldresp and $oldresp eq $resp;
    $w->{Configure}{-resp} = $resp;
    $w->{Configure}{-xmldata}->att("resp", $resp);
  }
  return $oldresp;
}

sub name {
  my ($w, $name) = @_;
  my $oldname = $w->{Configure}{-name};
  if ($name) {
    return $name if $oldname and $oldname eq $name;
    $w->{Configure}{-name} = $name;
    $w->{Configure}{-xmldata}->att("name", $name);
  }
  return $oldname;
}

sub structurenames {
  my ($w, $structurenames) = @_;
  return 0 unless $structurenames;
  map {$w->{_structurenames}{$_} = 1} @$structurenames;
}

sub xmlconfig {
  my ($w) = @_;
  return $w->{Configure}{-xmlconfig};
}

sub colconf {
  my ($w, $cols) = @_;
  my $n = 0;
  foreach my $col (@$cols) {
    my $head = $col->{-head};
    if (exists $w->{_cols}{$head}) {
      die "Duplicate column header!"
    }
    $head =~ /^\D*$/ or die "Column headers cannot be numeric!";
    delete $col->{-head};
    my $width = $col->{-width};
    delete $col->{-width};
    my $style = $w->ItemStyle(
      "imagetext", 
      # TODO: a check, so that it does not crash when font is missing
      -font => 'normal',
      %$col
    );	

    $w->header("create", $n, -text => $head, -relief => "groove");
    $w->columnWidth($n, $width);
    $w->{_style}{$head} = $style;
    $w->{_style}{$n} = $style;
    $w->{_cols}{$head} = $n;
    $w->{_cols}{$n} = $head;
    $n++;
  }
}

sub loadConfig {
  # Due to internal restriction of HList, we have to 
  # destroy and build again the window everytime the config is
  # reloaded
  my ($w, $xmlconfig) = @_;

  $xmlconfig = $w->xmlconfig unless $xmlconfig;

  my $parent = $w->parent->parent;
  my $xml = $w->xml;


  my $type = $w->cget("-type");
  $type = $xml->nodeName unless $type;
  my $name = $w->cget("-name");
  $name = $type unless $name;

  my %options = ();
  # see this method in KhTree for explanation
  %options = Tk::KhTree::loadCols($type, $xmlconfig->root) if $xmlconfig; 

  $options{-xmldata} = $w->{Configure}{-xmldata};
  $options{-xmlconfig} = $xmlconfig if $xmlconfig;

  map {$_->destroy} $parent->children;

  my $analysistab = $parent->Scrolled(
    "KhTree", 
    -scrollbars => "ose", 
    %options, 
    -type => $type,
    -name => $name
  )->pack(-expand => 1, -fill => "both");


  if ($xmlconfig) {
    # FIXME: why pass type explicitely?
    $analysistab->loadImages($type, $xmlconfig->root);
    $analysistab->loadStructures($type, $xmlconfig->root);
    $parent->toplevel->loadBindings($analysistab, $type, $xmlconfig->root);
  } 


  $analysistab->Subwidget("scrolled")->{_configchange} = 0;
  $analysistab->loadXML($xml);

  return $analysistab;
}

sub _purge {
	# this method doesnot purge the clipboard
	my $w = shift;
	$w->delete("all");
	$w->{_xml} = "";
	$w->{Configure}{-xmldata} = "";
}


sub loadXML {
	my ($w, $xml) = @_;
	$xml = $w->{Configure}{-xmldata} unless $xml;
	$w->_purge;
	$w->{_xml} = $xml;
	$w->{Configure}{-xmldata} = $xml;
        # get rid of any existing word index (see _fromXML)
        $w->toplevel->{_words} = undef;
	$w->_fromXML($xml);
}

sub loadImages {
	my ($w, $type, $xml) = @_;
	foreach ($xml->findnodes("/config/widget[\@kind = 'default']/images")) {
		$w->_imagesFromXML($_);
	}
	# after default, specific images override default
	foreach ($xml->findnodes("/config/widget[\@kind = '$type']/images")) {
		$w->_imagesFromXML($_);
	}
}

sub _imagesFromXML {
	my ($w, $xml) = @_;
	my $path = $xml->att("dir");
	$path = $w->toplevel->img unless $path;
	$path .= "/" unless $path =~ /\/$/;
	my $ext = $xml->att("ext");
	foreach ($xml->findnodes("./image")) {
		my $image = $_->att("src");
		my $key = $_->att("key");
		my $type = $_->att("type");
		my $pix = $w->toplevel->Photo($image, -file => "$path$image$ext");
		$w->{_images}{$type}{$key} = $image;
	}
}

sub loadStructures {
	my ($w, $type, $xml) = @_;
	foreach ($xml->findnodes("/config/widget[\@kind = 'default']/structures")) {
		$w->_structuresFromXML($_);
	}
	# after default, specific images override default
	foreach ($xml->findnodes("/config/widget[\@kind = '$type']/structures")) {
		$w->_structuresFromXML($_);
	}
}

sub _structuresFromXML {
	my ($w, $xml) = @_;
	my @structures = ();
	foreach ($xml->findnodes("./structure")) {
		push @structures, $_->att("name");
	}
	$w->configure(-structurenames => [@structures])
}

sub loadCols {

  # FIXME: UNFIXABLE BUG!! Due to restrictions in Tk::HList widget, number of cols has
  # to be given before the reference is blessed as a KhTree. This means this sub must be 
  # called the procedural way and its return value appended in the new KhTree options at 
  # creation time. Therefore, this is not a real loading... despite of the name.

  my ($type, $xml) = @_;
  my $configured = 0;
  my %options = (-colconf => []);
  foreach my $col ($xml->findnodes("/config/widget[\@kind='$type']/columns/column")) {
    $configured = 1 unless $configured;
    my %atts = _colsFromXML($col);
    push @{$options{-colconf}}, {%atts};
  }
  unless ($configured) {
    foreach my $col ($xml->findnodes("/config/widget[\@kind='default']/columns/column")) {
      my %atts = _colsFromXML($col);
      push @{$options{-colconf}}, {%atts};
    }
  }
  $options{-columns} = scalar @{$options{-colconf}} + 1;
  return %options;
}

sub _colsFromXML {
	my ($col) = @_;
	my %atts;
	foreach ($col->findnodes("./@*") ) {
		$atts{"-".$_->nodeName} = $_->value;
	}
	return %atts;
}

sub xml {
	my $w = shift;
	return $w->{Configure}{-xmldata}
}


sub line {
	my ($w) = @_;
	my $id = $w->info("data", $w->info("anchor"));
	my $xmldata = $w->xml->getOwnerDocument;
	my $ref = $xmldata->elt_id($id)->att("id-ref");
	my $l = 0;
	map {$l++} $xmldata->elt_id($ref)->findnodes("./preceding::l") if $ref;
	print $l;
}

sub _fromXML {
	my ($w, $xmlnode) = @_;
	my $type = $xmlnode->att("type");
	$type = $xmlnode->nodeName unless $type;
	$w->{Configure}{-type} = $type;
	my $mainb = Tk::KhTree::Branch->new(-tree => $w, -xml => $xmlnode, -structurenames => $w->{_structurenames});
	$w->paste( branch => $mainb, init => 1 );
	undef $mainb;	
}

sub paste {
  my ($w, %options) = @_;

  # %options = (
  #	target => $treenode, 
  #	branch => $Khtreebranch_to_paste, 
  # 	position => before|after|first|last
  # 	);

  my ($target, $newspot, $branch, $init, $id, $position, $recursed);
  my $xmldoc = $w->toplevel->{_xml};
  $init = $options{init};
  $recursed = $options{recursed};

    # first, we determine what will be pasted and where

    # 1. in place pasting
    if (!$options{branch} and !$options{target}) {
      $target = $w->info("anchor");
      $branch = $w->{_clipboard};
    } 
    # 2 . paste with parameters
    elsif ($options{branch} and defined $options{target}) {
      $branch = $options{branch};
      $target = $options{target};
    } 
    # 3. initial pasting
    elsif ( $branch = $options{branch} and !$options{target} ) {
      $target = "";
      $init = 1;
    }
    else { 
      return 0
    }

    return 0 unless $branch;

    # if the position parameter is given, compute the target according
    # to it
    my %position = ();
    my $mainspot = $target;
    my $lastdescendant = $mainspot;

    while (my @children = $w->info("children", $lastdescendant)) {
      $lastdescendant = pop @children;		
    }

    $position = $options{position};

    my $data = "";
    my $newnode = $branch->xml;
    $newnode = $newnode->cloneNode(1) unless $w->{_cut} or $recursed;


    # Then do xml stuff when paste is not at init
    unless ($init) {
      # there are two cases: either duplicate the node,
      # because it already exists, either just paste it
      # because it has been cut
      # IN ANY CASE, ID TABLE HAS TO BE MAINTAINED
      # This is a bug in XML::EasXML
      # FIXME: This slows down the whole process
      foreach my $node ($newnode->findnodes("descendant-or-self::*")) {
        if ($w->{_cut}) {
          my $id = $node->id;
          $w->toplevel->{_xml}{ids}{$id} = $node;
        } elsif (!$recursed) {
          $node->removeAttribute("id");
          $xmldoc->create_id($node);
        }
      }

      $id = $w->info("data", $target);
      my $refnode = $xmldoc->elt_id($id);

      # terminal nodes cannot get children
      my $name = $refnode->nodeName;
      $position = "after" unless $w->{_structurenames}{$name} or $recursed or $position eq "before";

      unless ($recursed) {
        if ($position and $position ne "last") {
          my $parent;
          # if position is given:
          $parent = $refnode->parentNode; 
          if ($position eq "after")     { $parent->insertAfter($newnode,$refnode)  ;} 
          elsif ($position eq "before") { $parent->insertBefore($newnode,$refnode) }
          elsif ($position eq "first")  { 
            my $firstchild = "";
            $firstchild = $refnode->firstChild;
            $firstchild ? $refnode->insertBefore($newnode, $firstchild) : $refnode->insertBefore($newnode, undef);
          }
        }
        else { 
          $refnode->appendChild($newnode); 
        }
      }
    }

    if ($position and $position ne "last") {
      if ($position eq "first") {
        $position{"-at"} = 0;
        $mainspot = $target;
      }
      else {
        $position{"-".$position} = $target;
        $mainspot = $w->info("parent", $target);
      }
    }

    $data = $newnode->id;
    $newspot = $w->addchild($mainspot, -data => $data, %position);

    # display and set look and feel
    $w->_display($newspot, $branch);

    # recurse
    if ( my @children = $newnode->findnodes("./*") ) {
      $w->setmode($newspot, "close");
      map { 
        my $branch = Tk::KhTree::Branch->new(-xml => $_, -tree => $w, -structurenames => $w->{_structurenames});
        # recursion naturally forgets about position
        $w->paste(target => $newspot, branch => $branch, init => $init, recursed => 1); 
      } @children;
    }

    unless ($options{recursed}) {
      $w->focusNode($newspot);
      $w->{_cut} = 0 if $w->{_cut};
      return $newspot;
    }


  }

	sub _display {
		my ($w, $newspot, $branch) = @_;
		my $id = $branch->xml->id;
		my $n = 0;
		my @text = ();
		if (@text = @{$branch->text}) {
			for (0 .. $w->cget("-columns") - 1) {
				my $text = "";
				$text = $text[$_] if $text[$_];
				my $type = "";
				$type = $w->{_cols}{$_} if $w->{_cols}{$_} ;
				my $image = "";
				$image = $w->{_images}{$type}{$text} if $w->{_images}{$type}{$text} and $w->{_structurenames}{$branch->xml->nodeName};

				if ($type =~ /note/ and !$branch->xml->att("id-ref") and $w->{Configure}{-notes}) {
					my $widgetstyle = $w->ItemStyle("window");
					my $e = $w->Entry(-width => 70, -text => $text);
					$e->bind(
						"<Return>" => sub {
							$w->_changeNote($w->info("data", $newspot), $e->get)
						}
					);
					$w->itemCreate($newspot, $_, -itemtype => "window", -widget => $e, -style => $widgetstyle) ;
				} else {
					$w->itemCreate(
						$newspot,
						$_, 
						-image => $image,
						-text => $text, 
						-style => $w->{_style}{$_}
					);
				}

			}
		} else {
			$text[0] = "/";
			for (0 .. $w->cget("-columns") - 2) {

				my $type = "";
				$type = $w->{_cols}{$_} if $w->{_cols}{$_} ;
				my $image = "";
				$image = $w->{_images}{$type}{"/"} if $w->{_images}{$type}{"/"};
				$w->itemCreate(
					$newspot,$_, 
					-image => $image,
					-text => $text[$_], 
					-style => $w->{_style}{$_}
				);
			}

		}
		$w->{_iddb}{$id} = $newspot;
	}

sub _changeNote {
  my ($w, $id, $text) = @_;
  my $node = $w->toplevel->{_xml}->elt_id($id);
  return 0 unless $node;
  $node->att("note", $text);
}

sub move {
  my ($w, $direction, $node) = @_;
  $node = $w->info("anchor") unless $node;
  DIRECTION: {
    if ($direction eq "up") {
      my $prev = $w->info("prev", $node);
      while ($w->info("hidden", $prev)) {
        $prev = $w->info("prev", $prev);
      }
      my $position = "after";

      my $parent = $w->info("parent", $node);
      my $prevparent = $w->info("parent", $prev);

      if ($w->isStructure($prev) ) {
        unless ($w->info("children", $prev)) {
          $position = "last" 
        } else {
          $position = "before";
        }
      }
      elsif ($parent eq $prevparent) {
        $position = "before"
      } 

      my $branch = $w->_cut($node);
      $w->paste(position => $position);
      last DIRECTION;
    }
    if ($direction eq "left") {
      my $parent = $w->info("parent", $node);
      my $b = $w->_cut($node);
      $w->paste(target => $parent, branch => $b, position => "after");
      last DIRECTION;
    }
    if ($direction eq "down") {

      # this is the trickiest

      my $parent = $w->info("parent", $node);
      my $next = $w->info("next", $node);
      my $nextparent;

      while ($nextparent = $w->info("parent", $next)) {
        # if node is higher in the tree than next, seach the first following item
        # that is sibling to node
        if ($nextparent eq $node) {
          $next = $w->info("next", $next);
          next;
        } 
        elsif ($parent ne $nextparent) {
          return $w->move("left")
        }
        else {
          last
        }
      }

      my $b = $w->_cut($next);
      $w->paste(target => $node, branch => $b, position => "before");

      # move the focus down to the node under the last descendant of the "next node"
      $next = $w->info("anchor");
      OFFSPRING: while (1) {
        my @children = $w->info("children", $next);
        last OFFSPRING unless @children;
        CHILDREN: foreach (reverse @children) {
          $next = $_;
          last CHILDREN
        }
      }
      $w->focusNode($w->info("next",$next))
    }

  }

}

sub _cut {
  # to avoid data corruption, cutting nodes should stay private
  # $not_check_id; see &XML::Khartes::delete 
  my ($w, $node, $not_check_id) = @_;

  $node = $w->info("anchor") unless $node;

  # fill the clipboard
  my $cut = $w->copy($node);

  # move focus
  my $focus = $w->info("prev", $node);

  $w->_remove($node, 1, $not_check_id);
  $w->{_cut} = 1;

  $w->focusNode($focus);

  return $cut;
}

sub _remove {
  # to avoid data corruption, unconditionnal deletion should stay private
  # $not_check_id; see &XML::Khartes::delete 
  my ($w, $node, $deep, $not_check_id) = @_;
  $node = $w->info("anchor") unless $node;
  my $id = $w->info("data", $node);
  my $xml = $w->toplevel->{_xml};
  my $xmlnode = $xml->elt_id($id);
  if (! $xmlnode->hasChildNodes or $deep) {
    $xml->delete($xmlnode, $not_check_id);
    $w->delete("entry", $node);
  } else {
    # If you just want to delete an element, but not its children,
    # it is first emptied and its content pasted after it;

    # first, harvest the content:
    my @harvest = ();
    my @children = $w->info("children", $node);
    foreach (reverse @children) {
      my $cut = $w->_cut($_);
      unshift @harvest, $cut;
    }

    my %position = (position => "after");

    # compute position if necessary
    my $prev = $w->info("prev", $node);
    my $ancestorid = $w->info("data", $prev);
    my $ancestor = $xml->elt_id($ancestorid);
    my $isroot = 0;
    if (! $ancestor) {
      # s'il n'y a pas d'anctre, 
      # on est dans la racine
      $isroot = 1;
      %position = (position => "before");
    }


    if ($isroot) {
      @harvest = reverse @harvest;
      $prev = $w->info("next", $prev) 
    }

    my $focus = $prev;

    # set focus
    $w->focusNode($focus);


    # paste the harvest
    foreach (reverse @harvest) {
      $w->{_cut} = 1;
      $w->paste(target => $node, branch => $_, %position);
    }

    # then delete the node
    $xml->delete($xmlnode);
    $w->delete("entry", $node);

  }
}

sub safe_delete {
	my ($w, $node, $deep) = @_;
	$node = $w->info("anchor") unless $node;
	my @nodes = ($node);
	my $focus = $w->info("prev", $node);

	my $xml = $w->toplevel->{_xml};
	my $clipboard = $w->{_clipboard};
	foreach my $node (@nodes) {
		my $id = $w->info("data", $node);
		# TODO: generic
		my $target = $xml->elt_id($id)->att("id-ref");
		my @links;
		@links = $w->{_xml}->findnodes(".//*[\@id-ref = '$target'][1 or 2]") if $target;
		my $remove = 0;
		if (@links > 1 or !$target) {$remove = 1}
		if ($deep and $remove) {
			foreach ($w->info("children", $node)) {
				$w->safe_delete($_, $deep)
			}
		}
		if ($remove) {
			$w->_remove($node); 
			# move focus
			$w->focusNode($focus);
		}
	}	

	
	$w->{_clipboard} = $clipboard;
	
}

sub copy {
	my ($w, $node) = @_;
	$node = $w->info("anchor") unless $node;
	my $id = $w->info("data", $node);
	my $xmlnode = $w->toplevel->{_xml}->elt_id($id)->cloneNode(1);
	my $branch = Tk::KhTree::Branch->new(-tree => $w, -xml => $xmlnode, -structurenames => $w->{_structurenames});
	$w->{_clipboard} = $branch;
	return $branch;
}

sub duplicate {
	my ($w, $node) = @_;
	$w->copy($node);
	return $w->paste(position => 'before');
}

sub wrapOrChange {
	# the method chooses wether it has to wrap the selection (text, multiple label)
	# or to change the label (unique label);
	my ($w, %options) = @_;
	my @selection = $w->info("selection");
	if (@selection > 1) {
		return $w->wrap(%options);
	} 
	my $node = $w->info("anchor");
	my $id = $w->info("data", $node);
	my $xmlnode = $w->{_xml}->getOwnerDocument->elt_id($id);
	if ($w->{_structurenames}{$xmlnode->nodeName}) {
		return $w->dataChange(%options);
	} else {
		return $w->wrap(%options);
	}
}

sub dataChange {
  my ($w, %options) = @_;
  my $node = $w->info("anchor");
  my $id = $w->info("data", $node);
  my $xmlnode = $w->{_xml}->getOwnerDocument->elt_id($id);
  foreach my $key (keys %{$options{atts}}) {
    my $value = $options{atts}->{$key};
    $xmlnode->att($key, $value);
    my $image = "";
    $image = $w->{_images}{$key}{$value} if $w->{_images}{$key}{$value};
    unless ($key eq "note") {
      $w->itemConfigure($node,$w->{_cols}{$key}, -text => $value, -image => $image);
    } else {
      $w->itemCget($node,$w->{_cols}{$key}, '-window')->configure(-text => $value);
    }	
  }
}


sub goto_id {
  my ($w, $id) = @_;
  my $treenode = "";
  return 0 unless $id;
  if ($w->xml->findnodes(".//*[\@id = '$id']")) {
    $w->loadXML;
    $treenode = $w->{_iddb}{$id};
    if ($treenode) {
      $w->focusNode($treenode) ;
      return $treenode;
    }
  }
  return 0
}

sub focusNode {
	my ($w, $focus) = @_;
	$w->selectionClear();
	$w->selectionSet($focus);
	$w->anchorSet($focus);
	return $w->see($focus);
	
}

sub isStructure {
	my ($w, $node) = @_;
	my $id = $w->info("data", $node);
	my $name = $w->xml->getOwnerDocument->elt_id($id)->nodeName;
	return 1 if $w->{_structurenames}{$name};
	return 0
}

sub wrap {
	my ($w, %options) = @_;
	my ($nodes, $atts, $element);
	$nodes = [$w->info("selection")];
	$atts = {} unless $atts = $options{atts};

	#TODO: generic;
	$element = "POS" unless $element = $options{element};
	
	# create the new xml element
	my $xmldoc = $w->toplevel->{_xml};
	my $xmlnode = $xmldoc->xml->createElement($element);
	foreach my $key (keys %$atts) {
		$xmlnode->att($key, $atts->{$key})
	}
	$xmldoc->create_id($xmlnode);

	my %nodes = ();
	map {$nodes{$_} = 1} @$nodes;
	my @harvest = ();

	# create the branch
	my $branch = Tk::KhTree::Branch->new(
		-tree => $w, 
		-xml => $xmlnode, 
		-structurenames => $w->{_structurenames}
	);

	my $new;
	my $l = scalar(@$nodes) - 1;
	
	# this is tricky: we have to paste the new element into the tree
	# before its content is totally deleted, but we have to do it in a way that
	# does not change the place where the last child is. We reverse the list of children
	# and insert the new node just before we delete the first one (last loop)
	#
	# Each iteration cuts the child and harvest it
	
	while ($l + 1) {
		
		my $node = $nodes->[$l];
		# since pasting is recursive, don't handle children
		if ($nodes{$w->info("parent", $node)}) {
			$l--;
			next;
		}
		unless ($l) {
			$new = $w->paste(target => $node, branch => $branch, position => "before") ;
			$node = $w->info("next", $new) if $node eq $new;
		}
		unshift @harvest, $w->_cut($node, 1);
		$l--;
	}

	$w->setmode($new, "close") if @harvest;
	
	# paste the harvest
	foreach (@harvest) {
		$w->{_cut} = 1; # never forget all these are cut nodes
		$w->paste(target => $new, branch => $_);
	}
	
	$w->focusNode($new);
	
	
}

sub find {
  my ($w, $direction, %keys) = @_;
  my $current = $w->info("anchor");
  return 0 unless $current;
  
  CHECKATTS: while (1) {
    $current = $w->info($direction, $current);
    my $id = $w->info("data", $current);
    my $xmlnode = "";
    $xmlnode = $w->{_xml}->getOwnerDocument->elt_id($id) if $id;
    next CHECKATTS unless $w->find_match($current, {%keys});
    $w->focusNode($current);
    last    
  }
  # FIXME: alert when find fails;
  return 0 unless $current;
}

sub find_match {
  # renvoie si l'entre matche ou pas
  my ($w, $entry, $keys) = @_;
  my $id = $w->info("data", $entry);
  my $xmlnode = "";
  $xmlnode = $w->{_xml}->getOwnerDocument->elt_id($id) if $id;
  return 0 unless $xmlnode;
  return 0 unless $w->{_structurenames}{$xmlnode->nodeName};
  foreach (keys %$keys) {
      next unless $keys->{$_};
      my $att = $xmlnode->att($_);
      next if $att and $att =~ /$keys->{$_}/;
      return 0;
  }
  return 1;
}

sub find_replace {
  my ($w, $direction, $replace, $by) = @_;
  my $current = $w->info("anchor");
  return 0 unless $current;
  $w->find($direction, %$replace) unless $w->find_match($current, $replace);
  map {
    delete $by->{$_} unless $by->{$_} ;
    # TODO Advertise the reserved kw.
    $by->{$_} = "" if $by->{$_} and $by->{$_} eq "null"
  } keys %$by;
  $w->dataChange(atts => {%$by});
  $w->find($direction, %$replace);
}

sub dump_text {
  my ($w, $node) = @_;
  $node = $w->info("anchor") unless $node;
  my $ref = $w->info("data", $node);
  my $top = $w->toplevel;
  my $xml = $top->{_xml};

  my $s = $xml->Syntax;
  my $r = $s->extract(expression => ".//POS[\@id = '$ref']");
  # FIXME: there is a bug in XML::Khartes::Syntax; merge_form should not modify the "real" syntax node.
  # otherwise, it is part of the XML::Khartes->{xml} property.
  my $xmlr = $r->toXML(order => "original");
  my $string = $xmlr->root->toString(1);
  my $prompt = $top->prompt(
    -text => "Voulez-vous rediriger le contexte vers STDOUT?\n\n $string.",
    -choices => {qw/XML 1 TXT 1/},
  );
  if ($prompt eq "XML") {
    $xmlr->print(1);
  } elsif ($prompt eq "TXT") {
    my $out = $xmlr->root->trimmed_text;
    $out =~ s/\s+/ /gos;
    print $out;
  }
}

sub dump {
	my ($w, $node) = @_;
	$node = $w->info("anchor") unless $node;
	my $id = $w->info("data", $node);
	print STDERR "\n--------- $id -------------\n";
	print STDERR $w->toplevel->{_xml}{ids}{$id}->toString(1);
	print STDERR "\n--------- $id -------------\n";
}



# TODO: UNDO/REDO

sub snapshot {
	# for undo purpose, still don't know how...
	my ($w) = @_;
	my $xml = $w->{_xml};
	my $clone = Storable::freeze $xml;
	push @{$w->{_undo_queue}}, $clone;
	if (@{$w->{_undo_queue}} > 20) {
		shift @{$w->{_undo_queue}} 
	}
}

sub undo {
	my ($w) = @_;
	my $state = Storable::thaw(shift @{$w->{_undo_queue}});
	# FIXME: id table!!
	$w->loadXML($state) if $state;
}

# TODO: rfrence de ce code
sub UpDown {
	my $w = shift;
	my $spec = shift;
	my $done = 0;
	my $anchor = $w->info('anchor');
	delete $w->{'shiftanchor'};
	unless( defined $anchor ) {
		$anchor = ($w->info('children'))[0] || '';
		return unless (defined($anchor) and length($anchor));
		if($w->entrycget($anchor, '-state') ne 'disabled') {
			# That's a good anchor
			$done = 1;
		}
		else {
			# We search for the first non-disabled entry (downward)
			$spec = 'next';
		}
	}

	my $ent = $anchor;

	# Find the prev/next non-disabled entry
	while(!$done) {
		$ent = $w->info($spec, $ent);
		last unless( defined $ent );
		next if( $w->entrycget($ent, '-state') eq 'disabled' );
		next if( $w->info('hidden', $ent) );
		next if( $w->_parentHidden($ent) );
		last;
	}

	unless( defined $ent ) {
		$w->yview('scroll', $spec eq 'prev' ? -1 : 1, 'unit');
		return;
	}
	$w->anchorSet($ent);
	$w->see($ent);
	if($w->cget('-selectmode') ne 'single') {
		$w->selectionClear;
		$w->selection('set', $ent);
		$w->Callback(-browsecmd =>$ent);
	}
}

sub _parentHidden {
	my ($w, $ent) = @_;
	my $hidden = 0;
	my $parent = $w->infoParent($ent) || ""; 
	while ($hidden == 0 && $parent ne "") {
		$hidden = 1 if $w->infoHidden($parent);
		$parent = $w->infoParent($parent) || "";
	} return $hidden;
}


1;

#######################################################
package Tk::KhTree::Branch;
#######################################################

use strict;
use Data::Dumper;

sub new {
	my $proto = shift;
        my $class = ref($proto) || $proto;
	my $object = {
		-xml => undef,
		# TODO: generic
		-structurenames => {}, 
		-tree => undef,
		-self => {
			-data => undef,
			-text => []
		},
		@_
	};
	$object->{-tree} and $object->{-xml} or die "Lacking XML data!";
	# TODO: make XML::EasXML more traceable
	$object->{-easxml} = $object->{-tree}->toplevel->{_xml};
	bless($object, $class);
	$object = $object->_buildFromXML($object->{-xml}, $object->{-structurenames});
	return $object;
}

sub _buildFromXML {
	my ($o, $xml, $structurenames) = @_;
	return 0 unless $xml->nodeType == 1;
	my $name = $xml->nodeName;
	
	# TODO: generic!!
	return 0 if $name =~ /^not/;
				
	my $id = $xml->id; 
	
	$o->{-self}{-data} = $id; 
	 
	if ($structurenames->{$name}) {
		foreach my $att ($xml->findnodes("./attribute::*")) {
			my $name = $att->nodeName;
			my $n = $o->{-tree}{_cols}{$name};
			next unless defined $n;
			my $value = $att->value;
			$o->{-self}{-text}[$n] = $value
		}
	} elsif ($name eq "C") {
		my $target = $xml->att("id-ref");
		my $text;
		# test wether indexed or not
		$text = $o->tree->toplevel->{_words}{$target};
		unless ($text) {
			my $elt = $o->{-easxml}->elt_id($target);
			if ($elt) {
				$text = $elt->trimmed_text;
				$text =~ s/\s//go;
				# index it to get faster!! (is it??)
				$o->tree->toplevel->{_words}{$target} = $text
			}
		}
		$o->{-self}{-text}[0] = $text;
		
	}
	
	
	return $o;	
}

sub tree {
	my ($o) = @_;
	return $o->{-tree};
}

sub xml {
	my ($o) = @_;
	return $o->{-xml};
}

sub info {
	my ($o) = @_;
	return $o->{-self};
}

sub data {
	my ($o) = @_;
	return $o->{-self}{-data};
}

sub text {
	my ($o) = @_;
	return $o->{-self}{-text};
}

sub children {
	my ($o) = @_;
	return $o->xml->findnodes("./*");
}

sub dump {
	my ($o) = @_;
	print Dumper($o);	
}

1;


