# Copyright (c) 2005 Universit de Lige, Nicolas Mazziotta
# $Id: KhTreeConfig.pm 75 2005-11-16 10:15:03Z mzi $
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.

use strict;

#######################################################
package Tk::KhTreeConfig;
#######################################################

use Tk;
use base qw/Tk::Frame/;
use Tk::ColorEditor;
use Data::Dumper;

Construct Tk::Widget "KhTreeConfig";

sub ClassInit{
	my ($class,$mw) = @_;
	$class->SUPER::ClassInit($mw);
}

sub Populate {

  my ($w, $args) = @_;


  $w->ConfigSpecs(
    -widget => ["METHOD", "widget", "Widget", undef],
    -widgetcontainer => ["PASSIVE", "widgetcontainer", "WidgetContainer", undef],
    -xmlconfig => ["METHOD", "xmlconfig", "XmlConfig", undef],
  );

  my $type = $args->{-widget}->type;

  $w->{Configure}{-widget} = $args->{-widget};

  map { 
    $w->{_xmlconfig} = $_
  } 
  $args->{-widget}->xmlconfig->root->findnodes("/config/widget[\@type = 'tree' and \@kind = '$type'][1]");


  $w->SUPER::Populate($args);

  # Affichage
  my %defaultPack = qw(
    -expand 1
    -fill both
    -side left
  );
  my %defaultBorder = qw(
    -relief groove
    -borderwidth 2
  );

  $w->fontCreate("normal", -size => "10", -family => "Helvetica");
  $w->fontCreate("italic", -size => "10", -slant => "italic", -family => "Helvetica");
  $w->fontCreate("small", -size => "8", -family => "Helvetica");
  $w->fontCreate("bold", -weight => "bold", -size=> 10, -family => "Helvetica");


  my $main = $w->Frame()->pack(%defaultPack);

  my $g = $main->Frame->pack(%defaultPack);
  my $a = $main->Frame->pack(%defaultPack);

  # init cols

  my $c = $g
  ->LabFrame(-label => "Colonnes")->pack(%defaultPack, -side => "top")
  ->Scrolled("HList", -scrollbars => 'ose', -columns => 5, -header => 1)->pack(-expand => 1, -fill => "both");

  my @heads = ("", qw/Type Largeur Fond Avant/);
  for (0 .. 4) {
    $c->header("create",$_, -text => $heads[$_], %defaultBorder);
  }

  # init structs

  my $s = $g
  ->LabFrame(-label => "Structures")->pack(%defaultPack)
  ->Scrolled("HList", -scrollbars => 'ose', -columns => 2, -header => 1)->pack(-expand => 1, -fill => "both");

  @heads = ("", qw/Nom/);
  for (0 .. 1) {
    $s->header("create",$_, -text => $heads[$_], %defaultBorder);
  }


  # init wraps

  my $e = $a
  ->LabFrame(-label => "Emballer/changer")->pack(%defaultPack)
  ->Scrolled("HList", -scrollbars => 'ose', -drawbranch => 0, -columns => 5, -header => 0)->pack(-expand => 1, -fill => "both");


  my $m = $a->LabFrame(-label => "Mouvements")->pack(-fill => "x", -expand => 0);

  $g->Button(-text => "Sauvegarder", -command => sub { $args->{-widget}->xmlconfig->toFile })->pack;
  $g->Button(-text => "Appliquer", -command => sub {
    my @trees = $w->trees;

    # reload config
    my $t = $w->tree->loadConfig; 

    # re-advertise the tree
    my $atab = $w->{Configure}{-widgetcontainer};
    my $nb = $atab->Subwidget("nb");
    $nb->Advertise($nb->raised => $t);
    # tell the trees that share the same config that it has changed

    map { 
      my $pagetype = "";
      my $reftype = "";
      $pagetype = $_->type if $_->type;
      $reftype = $t->type if $t->type;
      $_->{_configchange} = 1 if $reftype eq $pagetype and $_ ne $t 
    } @trees;

    # destroy the config pane
    # TODO: a way to reload it
    $w->toplevel->destroy;
  } )->pack;
  $g->Button(-text => "Dtruire la configuration", -command => sub {
    my $destroy = $w->tree->toplevel->prompt(-text => "Ceci va dtruire la configuration de cette analyse. Toutes les autres analyses du mme type seront affectes. tes-vous certain de vouloir continuer?");
    return 0 unless $destroy eq "oui";

    $w->{_xmlconfig}->parentNode->removeChild($w->{_xmlconfig});


    my @trees = $w->trees;

    # reload config
    my $t = $w->tree->loadConfig; 

    # re-advertise the tree
    my $atab = $w->{Configure}{-widgetcontainer};
    my $nb = $atab->Subwidget("nb");
    $nb->Advertise($nb->raised => $t);
    # tell the trees that share the same config that it has changed

    map { 
      my $pagetype = "";
      my $reftype = "";
      $pagetype = $_->type if $_->type;
      $reftype = $t->type if $t->type;
      if ($reftype eq $pagetype and $_ ne $t) {
        $_->{_configchange} = 1 ;
      }
    } @trees;

    # destroy the config pane
    # TODO: a way to reload it
    $w->toplevel->destroy;
  } )->pack;

  $w->Advertise(cols => $c );
  $w->Advertise(struct => $s );
  $w->Advertise(wrap => $e );
  $w->Advertise(moves => $m);

  $w->_loadXMLConfig();

}

sub widget {
	my ($w, $widget) = @_;
	$w->{Configure}{-widget} = $widget if $widget;
	return $w->{Configure}{-widget};
}

sub xmlconfig {
	my ($w, $xmlconfig) = @_;
	$w->{Configure}{-xmlconfig} = $xmlconfig if $xmlconfig;
	return $w->{Configure}{-xmlconfig};
}

sub trees {
	# trees that share the same config
	my ($w) = @_;
	my $config = $w->tree->xmlconfig;
	my $nb = $w->{Configure}{-widgetcontainer};
	my @tree = ();
	foreach ($nb->pages) {
		my $t = $nb->page($_)->Subwidget("scrolled");
		push @tree, $t if $t->xmlconfig eq $config;
		
	}
	return @tree;
}

sub tree {
	my ($w) = @_;
	return $w->{Configure}{-widget};
}

sub _loadXMLConfig {
  my ($w, $xml) = @_;
  $xml = $w->{_xmlconfig} unless $xml;
  $w->toplevel->destroy unless $xml;
  my $style = $w->ItemStyle("window");


  # cols

  my $cols = $w->Subwidget("cols");
  my $line = $cols->addchild("");

  foreach my $node ($xml->findnodes("./columns/column")) {
    $w->_addCol($cols, $style, $node);
  }

  my $ab = $cols->Button(-text => "add", -image => "duplicate", -command => ["_addCol", $w, $cols, $style]);
  $cols->itemCreate($line,0, -itemtype => "window", -widget => $ab, -style => $style);

  # structures

  my $struct = $w->Subwidget("struct");
  $line = $struct->addchild("");

  foreach my $node ($xml->findnodes("./structures/structure")) {
    $w->_addStruct($struct, $style, $node);
  }

  $ab = $struct->Button(-text => "add", -image => "duplicate", -command => ["_addStruct", $w, $struct, $style]);
  $struct->itemCreate($line,0, -itemtype => "window", -widget => $ab, -style => $style);


  # images

  foreach ($xml->findnodes("./images/image")) {
    my $name = $_->att("src");
    my $key = $_->att("type");
    my $value = $_->att("key");
    my $ext = $_->parentNode->att("ext");
    $w->{_images}{$key}{$value} = $w->Photo($name, -file => $w->tree->toplevel->img."/".$name."$ext");
  }

  # actions

  my $wrap = $w->Subwidget("wrap");
  $line = $wrap->addchild("");

  foreach my $node ($xml->findnodes("./bindings/binding[\@action = 'wrapOrChange']")) {
    $w->_addWrap($wrap, $style, $node);
  }

  $line = $wrap->addchild("");
  $ab = $wrap->Button(-text => "add", -image => "duplicate", -command => ["_addWrap", $w, $wrap, $style]);
  $wrap->itemCreate($line,0, -itemtype => "window", -widget => $ab, -style => $style);


}

sub _setWidth {
	my ($w, $widget, $xml) = @_;
	my $value = $widget->get();
	if ($value !~ /^\d*$/) {
		$widget->configure(-text => "Nombre seulement!");
		return 0
	}
	$xml->att("width", $value);
	$widget->configure(-background => $w->optionGet("background", ref $w));
	return $value;
}

sub _setField {
	my ($w, $widget, $xml, $key) = @_;
	my $value = $widget->get();
	$xml->att($key, $value);
	$widget->configure(-background => $widget->optionGet("background", ref $widget));
	return $value;
}

sub _setName {
	my ($w, $widget, $xml) = @_;
	my $value = $widget->get();
	$xml->att("name", $value);
	$widget->configure(-background => $w->optionGet("background", ref $w));
	return $value;
}

sub _setHead {
	my ($w, $widget, $xml) = @_;
	my $value = $widget->get();
	$xml->att("head", $value);
	$widget->configure(-background => $w->optionGet("background", ref $w));
	return $value;
}

sub _setColor {
	my ($w, $widget, $xml, $spot, $color) = @_;
	$color = "gray" unless $color;
	# $spot is foreground or background;
	my $newcolor = $w->chooseColor(-initialcolor => $color);
	return 0 unless $newcolor;
	$widget->configure(-background => $newcolor, -activebackground => $newcolor);
	$xml->att($spot, $newcolor);
	return $newcolor;
}

sub _deleteParams {
	my ($w, $cols, $line, $xml) = @_;
	$xml->parentNode->removeChild($xml);
	$cols->delete("entry", $line);
} 

sub _addStruct {
	my ($w, $structs, $style, $node) = @_;

	unless ($node) {
		my $parent;
		map { $parent = $_ } $w->xmlconfig->findnodes("./structures[1]");
		$node = $w->xmlconfig->getOwnerDocument->createElement("structure");
		$parent->appendChild($node);
	}

	my ($name) 
	= 
	($node->att('name'));

	my %position = ();
	my @children = $structs->info("children", "");
	%position = (-at => @children - 1) if @children;

	my $line = $structs->addchild("", %position);

	# name entry
	my $ne = $structs->Entry(-text => $name);
	$ne->bind("<KeyPress>" => sub {$ne->configure(-background => "red")});
	$ne->bind("<Return>" => sub {$w->_setName($ne, $node)});

	# delete button
	my $db = $structs->Button(
		-text => "delete", 
		-image => "delete",
		-command => sub {
			$w->_deleteParams($structs, $line, $node);
		}
	);

	# display
	$structs->itemCreate($line,0, -itemtype => "window", -widget => $db, -style => $style);
	$structs->itemCreate($line,1, -itemtype => "window", -widget => $ne, -style => $style);
	
}

sub _addWrap {
	my ($w, $wrap, $style, $node) = @_;

	unless ($node) {
		my $parent;
		map { $parent = $_ } $w->xmlconfig->findnodes("./bindings[1]");
		my $doc = $w->xmlconfig->getOwnerDocument;
		$node = $doc->createElement("binding");
		$node->att("action", "wrapOrChange");
		my $arg = $doc->createElement("argument");
		$arg->att("type", "scalar");
		$arg->att("value", "atts");
		$node->appendChild($arg);
		$arg = $doc->createElement("argument");
		$arg->att("type", "hash");
		my $pair = $doc->createElement("pair");
		$pair->att("key");
		$pair->att("value");
		$arg->appendChild($pair);
		$node->appendChild($arg);
		$parent->appendChild($node);
	}
	
	my %position = ();
	my @children = $wrap->info("children", "");
	%position = (-at => @children - 1) if @children;

	my $line = $wrap->addchild("", %position);

	# delete button
	my $db = $wrap->Button(
		-text => "delete", 
		-image => "delete",
		-command => sub {
			$w->_deleteParams($wrap, $line, $node);
		}
	);
	
	# keybutton
	my $key = $node->att("key");
	my $kb =  $wrap->Button(
		-text => $key, 
		-borderwidth => 2,
		-relief => "groove",
		
	);
	$kb->configure(-command => ["_changeShortcut", $w, $kb, $kb->pointerx, $kb->pointery, $node]);
	
	
	$wrap->itemCreate($line,0, -itemtype => "window", -widget => $db, -style => $style);
	$wrap->itemCreate($line,2, -itemtype => "window", -widget => $kb, -style => $style);
	

	foreach my $pairnode ($node->findnodes("argument[\@type = 'hash']/pair")) {
		$w->_addPair($wrap, $line, $style, $pairnode);
	}

	# TODO: add routine
	my $ab = $wrap->Button(-text => "add", -image => "duplicate", -command => ["_addPair", $w, $wrap, $line, $style, $node]);
	$wrap->itemCreate($line,1, -itemtype => "window", -widget => $ab, -style => $style);

}

sub _addPair {
	my ($w, $wrap, $line, $style, $pairnode) = @_;
	
	unless ($pairnode->nodeName eq "pair") {
		# if the nodeName ain't "pair", means we get the "binding" element
		my $parent;
		map {$parent = $_} $pairnode->findnodes("argument[\@type = 'hash'][1]");
		my $newnode = $w->xmlconfig->getOwnerDocument->createElement("pair");
		$newnode->att("key","");
		$newnode->att("value","");
		$pairnode = $parent->appendChild($newnode);
	}
	
	my %position = ();
	my @children = $wrap->info("children", $wrap->info("parent", $line));
	%position = (-at => @children - 1) if @children;

	$line = $wrap->addchild($line, %position);

	my $key = $pairnode->att('key');
	my $value = $pairnode->att("value");

	my $ke = $wrap->LabEntry(-label => "Type", -width => "10", -text => $key, -labelPack => [-side => "left" ]);
	$ke->bind("<KeyPress>" => sub {$ke->configure(-background => "red")});
	$ke->bind("<Return>" => sub {$w->_setField($ke, $pairnode, "key")}); # use cget to get the OLD value

	my $ve = $wrap->LabEntry(-label => "valeur", -width => "10", -text => $value, -labelPack => [-side => "left" ]);
	$ve->bind("<KeyPress>" => sub {$ve->configure(-background => "red")});
	$ve->bind("<Return>" => sub {$w->_setField($ve, $pairnode, "value")});

	# imagebutton
	my $image = "";
	$image = $w->{_images}{$key}{$value} if $w->{_images}{$key}{$value} ;
	my $ext = "";

	
	
	my $imgb =  $wrap->Button(
		-image => $image, 
		-borderwidth => 2,
		-relief => "groove",
	);
	
	# delete button
	my $db = $wrap->Button(
		-text => "delete", 
		-image => "delete",
		-command => sub {
			$w->_deleteParams($wrap, $line, $pairnode);
		}
	);

	# display
	$wrap->itemCreate($line,1, -itemtype => "window", -widget => $db, -style => $style);
	$wrap->itemCreate($line,2, -itemtype => "window", -widget => $ke, -style => $style);
	$wrap->itemCreate($line,3, -itemtype => "window", -widget => $ve, -style => $style);
	$wrap->itemCreate($line,4, -itemtype => "window", -widget => $imgb, -style => $style);
}

sub _changeShortcut {
	my ($w, $kb, $x, $y, $node) = @_;
	$x = $w->pointerx;
	$y = $w->pointery;
	my $k = $node->att("key");
	my $mw = MainWindow->new();
	$mw->geometry("+$x+$y");
	my $l = $mw->Entry(-textvariable => \$k)->pack(-side => "left");
	$l->focus;
	$l->bind("<KeyPress>" => sub {$k = $Tk::event->K;});
	$l->bind("<Alt-KeyPress>" => sub {$k = "Alt-" . $Tk::event->K;});
	$l->bind("<Control-KeyPress>" => sub {$k = "Control-" . $Tk::event->K;});
	$l->bind("<Control-Alt-KeyPress>" => sub {$k = "Control-Alt-" . $Tk::event->K;});
	my $ok = $mw->Button(-text => "Appliquer", -command => sub {
		$kb->configure(-text => $k);
		$node->att("key", $k);
		$mw->destroy
	})->pack(-side => "left");
	my $no = $mw->Button(-text => "Anuller", -command => sub {$mw->destroy})->pack(-side => "left");
	my $del = $mw->Button(-text => "Effacer", -command => sub {
		$kb->configure(-text => "");
		$node->att("key", "");
		$mw->destroy
	})->pack(-side => "left");

}

sub _addCol {
  my ($w, $cols, $style, $node) = @_;

  unless ($node) {
    my $parent;
    map { $parent = $_ } $w->xmlconfig->findnodes("./columns[1]");
    $node = $w->xmlconfig->getOwnerDocument->createElement("column");
    $parent->appendChild($node);
  }

  my ($head, $width, $bcolor, $fcolor) 
  = 
  ($node->att('head'), $node->att('width'), $node->att('background'), $node->att('foreground'));

  my %position = ();
  my @children = $cols->info("children", "");
  %position = (-at => @children - 1) if @children;

  my $line = $cols->addchild("", %position);

  # width entry
  my $we = $cols->Entry(-text => $width);
  $we->bind("<KeyPress>" => sub {$we->configure(-background => "red")});
  $we->bind("<Return>" => sub {$w->_setWidth($we, $node)});

  # type entry
  my $te = $cols->Entry(-text => $head);

  $te->bind("<KeyPress>" => sub {$te->configure(-background => "red")});
  $te->bind("<Return>" => sub {$w->_setHead($te, $node)});

  # delete button
  my $db = $cols->Button(
    -text => "delete", 
    -image => "delete",
    -command => sub {
      $w->_deleteParams($cols, $line, $node);
    }
  );

  # change background color button
  my $bgb = $cols->Button();
  $bgb->configure(-background => $bcolor, -activebackground => $bcolor) if $bcolor;
  $bgb->configure(-command => ["_setColor", $w, $bgb, $node, "background", $bcolor]);

  # change foreground color button
  my $fgb = $cols->Button();
  $fgb->configure(-background => $fcolor, -activebackground => $fcolor) if $fcolor;
  $fgb->configure(-command => ["_setColor", $w, $fgb, $node, "foreground", $fcolor]);

  # display
  $cols->itemCreate($line,0, -itemtype => "window", -widget => $db, -style => $style);
  $cols->itemCreate($line,1, -itemtype => "window", -widget => $te, -style => $style);
  $cols->itemCreate($line,2, -itemtype => "window", -widget => $we, -style => $style);
  $cols->itemCreate($line,3, -itemtype => "window", -widget => $bgb, -style => $style);
  $cols->itemCreate($line,4, -itemtype => "window", -widget => $fgb, -style => $style);
}

#######################################################
package Tk::KhTreeConfig::XML;
#######################################################

our @ISA = qw/XML::LibXML::Element/;


1;
